@@ -7,6 +7,7 @@
 #include "mri_genalign_util.c"
 #include "mri_genalign.c"
 #include "mri_nwarp.c"
+#include "thd_conformist.c"
 
 /*----------------------------------------------------------------------------*/
 
@@ -16,9 +17,10 @@ int main( int argc , char *argv[] )
    char *prefix = "AdjMean" ;
    int iarg , ii,kk , verb=1 , iv ;
    THD_3dim_dataset *dset_sbar=NULL , *dset_wbar , *dset_twarp ;
-   int nx=0,ny=0,nz=0,nxyz=0, nxs=0,nys=0,nzs=0;
+   int nx=0,ny=0,nz=0,nxyz=0, nxs=0,nys=0,nzs=0,nxyzs=0;
    IndexWarp3D *AA,*BB , *WWbin ;
    float *sbar=NULL , fac , Anorm,Bnorm ;
+   int *ijkpad=NULL ;
 
    /**----------------------------------------------------------------------*/
    /**----------------- Help the pitifully ignorant user? -----------------**/
@@ -60,24 +62,37 @@ int main( int argc , char *argv[] )
       "All these calculcations could be done with other programs and a script,\n"
       "but the goal of this program is to make them faster and simpler to combine.\n"
       "It is intended to be used in an incremental template-building script, and\n"
-      "probably has no other utility.\n"
+      "probably has no other utility (cf. the script @toMNI_Qwarpar).\n"
       "\n"
       "OPTIONS:\n"
       "--------\n"
       " -nwarp  w1 w2 ... = List of input 3D warp datasets (at least 5).\n"
       "                     The list ends when a command line argument starts\n"
       "                     with a '-' or the command line itself ends.\n"
-      "                     * This 'option' is required!\n"
+      "                     * This 'option' is REQUIRED!\n"
       "                -->>** Each input warp is adjusted, and the altered warp\n"
-      "                       over-writes the input dataset.\n"
+      "                       over-writes the input dataset. (Therefore, there is\n"
+      "                       no reason to run 3dNwarpAdjust twice over the same\n"
+      "                       collection of warp datasets!)\n"
+      "                     * These input warps do not have to be defined on\n"
+      "                       exactly the same grids, but the grids must be\n"
+      "                       'conformant' -- that is, they have to have the\n"
+      "                       the same orientation and grid spacings.  Warps\n"
+      "                       will be extended to match the minimum containing\n"
+      "                       3D rectangular grid, as needed.\n"
       "\n"
-      " -source d1 d2 ... = List of input 3D datasets to be warped.  There must\n"
-      "                     be exactly as many of these datasets as there are\n"
-      "                     input warps.\n"
-      "                     * This option is not required.\n"
+      " -source d1 d2 ... = List of input 3D datasets to be warped by the adjusted\n"
+      "                     warp datasets.  There must be exactly as many of these\n"
+      "                     datasets as there are input warps.\n"
+      "                     * This option is NOT required.\n"
       "                     * These datasets will NOT be altered by this program.\n"
+      "                     * These datasets DO have to be on the same 3D grid\n"
+      "                       (so they can be averaged after warping).\n"
       "\n"
       " -prefix ppp       = Use 'ppp' for the prefix of the output mean dataset.\n"
+      "                     (Only needed if the '-source' option is also given.)\n"
+      "                     The output dataset will be on the common grid shared\n"
+      "                     by the source datasets.\n"
      ) ;
 
      PRINT_AFNI_OMP_USAGE("3dNwarpAdjust",NULL) ; PRINT_COMPILE_DATE ;
@@ -121,6 +136,7 @@ int main( int argc , char *argv[] )
      /*---------------*/
 
      if( strcasecmp(argv[iarg],"-nwarp") == 0 || strcasecmp(argv[iarg],"-warp") == 0 ){
+       int pad_xm,pad_xp,pad_ym,pad_yp,pad_zm,pad_zp , npad ;
        if( nwset > 0 ) ERROR_exit("Can't have multiple %s options :-(",argv[iarg]) ;
        if( ++iarg >= argc ) ERROR_exit("No argument after '%s' :-(",argv[iarg-1]) ;
        for( kk=iarg ; kk < argc && argv[kk][0] != '-' ; kk++,nwset++ ) ; /*nada*/
@@ -131,6 +147,7 @@ int main( int argc , char *argv[] )
          if( dset_nwarp[kk] == NULL )
            ERROR_exit("can't open warp dataset '%s' :-(",argv[iarg+kk]);
          if( DSET_NVALS(dset_nwarp[kk]) < 3 ) ERROR_exit("dataset '%s' isn't a 3D warp",argv[iarg+kk]);
+#if 0
          if( kk == 0 ){
            nx = DSET_NX(dset_nwarp[0]); ny = DSET_NY(dset_nwarp[0]); nz = DSET_NZ(dset_nwarp[0]); nxyz = nx*ny*nz;
          } else if( DSET_NX(dset_nwarp[kk]) != nx ||
@@ -138,7 +155,34 @@ int main( int argc , char *argv[] )
                     DSET_NZ(dset_nwarp[kk]) != nz   ){
            ERROR_exit("warp dataset '%s' doesn't match with grid size %dx%dx%d",argv[iarg+kk],nx,ny,nz) ;
          }
+#endif
        }
+       ijkpad = (int *)calloc(sizeof(int),6*nwset) ;
+       kk = THD_conformist( nwset , dset_nwarp , CONFORM_NOREFIT , ijkpad ) ;
+       if( kk < 0 )
+         ERROR_exit("warp datasets grid do not conform to one another") ;
+       for( npad=kk=0 ; kk < nwset ; kk++ ){
+         pad_xm = ijkpad[6*kk+0] ; pad_xp = ijkpad[6*kk+1] ;
+         pad_ym = ijkpad[6*kk+2] ; pad_yp = ijkpad[6*kk+3] ;
+         pad_zm = ijkpad[6*kk+4] ; pad_zp = ijkpad[6*kk+5] ;
+         if( pad_xm > 0 || pad_xp > 0 || pad_ym > 0 || pad_yp > 0 || pad_zm > 0 || pad_zp > 0 ){
+           THD_3dim_dataset *qset = THD_nwarp_extend( dset_nwarp[kk] ,
+                                                      pad_xm,pad_xp,pad_ym,pad_yp,pad_zm,pad_zp ) ;
+           if( qset == NULL )
+             ERROR_exit("Cannot extend warp dataset %s to match containing grid",DSET_HEADNAME(dset_nwarp[kk])) ;
+
+           EDIT_dset_items( qset , ADN_prefix , DSET_prefix_noext(dset_nwarp[kk]) , ADN_none ) ;
+           DSET_delete(dset_nwarp[kk]) ; DSET_lock(qset) ; dset_nwarp[kk] = qset ; npad++ ;
+         }
+       }
+       nx = DSET_NX(dset_nwarp[0]); ny = DSET_NY(dset_nwarp[0]); nz = DSET_NZ(dset_nwarp[0]); nxyz = nx*ny*nz;
+       if( npad == 0 & verb )
+         INFO_message("All %d input warp datasets matched grid %dx%dx%d",nwset,nx,ny,nz) ;
+       else
+         INFO_message("%d input warp dataset%s (out of %d) %s padded to match grid %dx%dx%d",
+                      npad  , (npad > 1) ? "s"    : "\0"  ,
+                      nwset , (npad > 1) ? "were" : "was" ,
+                      nx,ny,nz ) ;
        iarg += nwset ; continue ;
      }
 
@@ -156,11 +200,11 @@ int main( int argc , char *argv[] )
            ERROR_exit("can't open warp dataset '%s' :-(",argv[iarg+kk]);
          if( DSET_NVALS(dset_src[kk]) > 1 ) ERROR_exit("dataset '%s' has more than 1 sub-brick",argv[iarg+kk]);
          if( kk == 0 ){
-           nxs = DSET_NX(dset_src[0]); nys = DSET_NY(dset_src[0]); nzs = DSET_NZ(dset_src[0]);
+           nxs = DSET_NX(dset_src[0]); nys = DSET_NY(dset_src[0]); nzs = DSET_NZ(dset_src[0]); nxyzs = nxs*nys*nzs;
          } else if( DSET_NX(dset_src[kk]) != nxs ||
                     DSET_NY(dset_src[kk]) != nys ||
                     DSET_NZ(dset_src[kk]) != nzs   ){
-           ERROR_exit("warp dataset '%s' doesn't match with grid size %dx%dx%d",argv[iarg+kk],nxs,nys,nzs) ;
+           ERROR_exit("source dataset '%s' doesn't match with grid size %dx%dx%d",argv[iarg+kk],nxs,nys,nzs) ;
          }
        }
        iarg += nsset ; continue ;
@@ -171,6 +215,8 @@ int main( int argc , char *argv[] )
      ERROR_exit("Unknown, Illegal, and Fattening option '%s' :-( :-( :-(",argv[iarg]) ;
    }
 
+   Hverb = (verb > 0) ;  /* for IW3D_invert */
+
    /*-------- check inputs to see if the user is completely demented ---------*/
 
    if( dset_nwarp == NULL )
@@ -179,15 +225,16 @@ int main( int argc , char *argv[] )
    if( nsset > 0 && nsset != nwset )
      ERROR_exit("Number of -source datasets %d doesn't match number of -nwarp datasets %d",nsset,nwset) ;
 
-   if( nsset > 0 && ( nxs != nx || nys != ny || nzs != nz ) )
-     ERROR_exit("-source datasets grid %dx%dx%d doesn't match -nwarp grid %dx%dx%d",
-                nxs,nys,nzs , nx,ny,nz ) ;
+
+   if( verb && nsset > 0 && ( nx != nxs || ny != nys || nz != nzs ) )
+     INFO_message("warp grid = %dx%dx%d is bigger than source grid = %dx%dx%d (this is NOT a problem)",
+                  nx,ny,nz , nxs,nys,nzs ) ;
 
    /*--- the actual work (bow your head in reverence) ---*/
 
    /* mean of input displacements */
 
-   if( verb ) fprintf(stderr,"++ Mean warp") ;
+   if( verb ) fprintf(stderr,"++ Computing mean warp") ;
 
    dset_wbar = THD_mean_dataset( nwset , dset_nwarp , 0,2 , verb ) ;
 
@@ -198,7 +245,7 @@ int main( int argc , char *argv[] )
 
    /* convert to an index warp and invert that */
 
-   if( verb ) fprintf(stderr,"++ Invert mean warp") ;
+   if( verb ) fprintf(stderr,"++ Inverting mean warp") ;
 
    AA    = IW3D_from_dataset( dset_wbar , 0,0 ); Anorm = IW3D_normL1(AA   ,NULL);
    WWbin = IW3D_invert( AA, NULL , MRI_WSINC5 ); Bnorm = IW3D_normL1(WWbin,NULL);
@@ -222,48 +269,64 @@ int main( int argc , char *argv[] )
                         ADN_prefix , prefix ,
                         ADN_brick_fac , NULL ,
                       ADN_none ) ;
-     sbar = (float *)calloc(sizeof(float),nxyz) ;
+     sbar = (float *)calloc(sizeof(float),nxyzs) ;
      EDIT_substitute_brick( dset_sbar , 0 , MRI_float , sbar ) ;
    }
 
-   if( verb ) fprintf(stderr,"++ Adjusting") ;
+   if( verb == 1 ) fprintf(stderr,"++ Adjusting") ;
+   else if( verb > 1 ) INFO_message("========== Beginning adjustment process ==========") ;
 
    for( kk=0 ; kk < nwset ; kk++ ){
+     if( verb > 1 ) INFO_message("convert dataset warp #%d to index warp",kk) ;
      AA = IW3D_from_dataset( dset_nwarp[kk] , 0,0 ) ;
      if( AA == NULL ) continue ;  /* should not happen */
+     if( verb > 1 ) ININFO_message("  compose with mean inverse") ;
      BB = IW3D_compose( AA , WWbin , MRI_WSINC5 ) ;
      IW3D_destroy(AA) ;
+     if( verb > 1 ) ININFO_message("  convert back to dataset warp") ;
      dset_twarp = IW3D_to_dataset( BB , "WeLoveTheLeader" ) ;
      IW3D_destroy(BB) ;
 
-     if( verb ) fprintf(stderr,".") ;
+     if( verb == 1 ) fprintf(stderr,".") ;
 
+     if( verb > 1 ) ININFO_message("  load input dataset warp again (just to be safe)") ;
      DSET_load( dset_nwarp[kk] ) ;
+     if( verb > 1 ) ININFO_message("  substitute brick #0") ;
      EDIT_substitute_brick( dset_nwarp[kk] , 0 , MRI_float , DSET_ARRAY(dset_twarp,0) ) ;
+     if( verb > 1 ) ININFO_message("  substitute brick #1") ;
      EDIT_substitute_brick( dset_nwarp[kk] , 1 , MRI_float , DSET_ARRAY(dset_twarp,1) ) ;
+     if( verb > 1 ) ININFO_message("  substitute brick #2") ;
      EDIT_substitute_brick( dset_nwarp[kk] , 2 , MRI_float , DSET_ARRAY(dset_twarp,2) ) ;
+     if( verb > 1 ) ININFO_message("  delete temporary warp dataset") ;
      DSET_NULL_ARRAY(dset_twarp,0) ;
      DSET_NULL_ARRAY(dset_twarp,1) ;
      DSET_NULL_ARRAY(dset_twarp,2) ; DSET_delete(dset_twarp) ;
      tross_Make_History( "3dNwarpAdjust" , argc,argv , dset_nwarp[kk] ) ;
+     if( verb > 1 ) ININFO_message("  write out adjusted warp dataset") ;
      DSET_write( dset_nwarp[kk] ) ;
 
      if( dset_src != NULL ){
        THD_3dim_dataset *dset_www ; float *bb ;
+       if( verb > 1 ) ININFO_message("  re-warp from source") ;
        dset_www = THD_nwarp_dataset( dset_nwarp[kk] , dset_src[kk] , NULL ,
-                                     "JamesBond" , MRI_WSINC5 , MRI_WSINC5 , 0.0f,1.0f,1,NULL ) ;
+                                     "BondJamesBond" , MRI_WSINC5 , MRI_WSINC5 , 0.0f,1.0f,1,NULL ) ;
        bb = (float *)DSET_ARRAY(dset_www,0) ;
-       for( ii=0 ; ii < nxyz ; ii++ ) sbar[ii] += bb[ii] ;
-       DSET_delete(dset_www) ; DSET_delete(dset_src[kk]) ;
+       if( verb > 1 ) ININFO_message("  add to mean") ;
+       for( ii=0 ; ii < nxyzs ; ii++ ) sbar[ii] += bb[ii] ;
+       if( verb > 1 ) ININFO_message("  delete www dataset") ;
+       DSET_delete(dset_www) ;
+       if( verb > 1 ) ININFO_message("  delete src dataset") ;
+       DSET_delete(dset_src[kk]) ;
      }
 
+     if( verb > 1 ) ININFO_message("  delete warp dataset from memory") ;
      DSET_delete( dset_nwarp[kk] ) ;
    }
 
-   if( verb ) fprintf(stderr,"\n") ;
+   if( verb == 1 ) fprintf(stderr,"\n") ;
 
    fac = 1.0f / nwset ;
-   for( ii=0 ; ii < nxyz ; ii++ ) sbar[ii] *= fac ;
+   for( ii=0 ; ii < nxyzs ; ii++ ) sbar[ii] *= fac ;
 
    DSET_write(dset_sbar) ;
    if( verb ) WROTE_DSET(dset_sbar) ;
