@@ -36,36 +36,8 @@ void Error_Exit(char *message)
     exit(1);
 }
 
+void usage_3dROIstats(int detail) {
 
-int main(int argc, char *argv[])
-{
-    THD_3dim_dataset *mask_dset = NULL, *input_dset = NULL ;
-    int mask_subbrik = 0;
-    int sigma = 0, nzsigma = 0, mean = 1, nzmean = 0, nzcount = 0;
-    int debug = 0, quiet = 0, summary = 0;
-    int minmax = 0, nzminmax = 0, donzsum = 0;		/* 07 July, 2004 [rickr] */
-    short *mask_data;
-    int nvox, i, brik;
-    int num_ROI, ROI, mask_f2s = 0;
-    int force_num_ROI = 0;	/* Added 5/00 */
-    int narg = 1;
-    double *sum=NULL, *sumsq=NULL, *nzsum=NULL, sig, *sumallbriks=NULL;
-    double  *min=NULL, *max=NULL, *nzsumsq=NULL,
-            *nzmin=NULL, *nzmax=NULL;		/* 07 July, 2004 [rickr] */
-    long *voxels=NULL, *nzvoxels=NULL;
-    float *input_data;
-    byte *temp_datab;
-    short *temp_datas;
-    double *percentile=NULL;
-    float *fv = NULL;
-    int nfv = 0, perc = 0, nzperc = 0;
-    int nobriklab=0 ;  /* 14 Mar 2008 */
-    int disp1d=1;   /* ZSS May 2008 */
-    byte *roisel=NULL;
-    char sbuf[257]={""};
-    char zerofill[32]={" "};
-
-    if (argc < 3 || strcmp(argv[1], "-help") == 0) {
 	printf("Usage: 3dROIstats -mask[n] mset [options] datasets\n"
 "\n"
 "   Display statistics over masked regions.  The default statistic\n"
@@ -120,7 +92,8 @@ int main(int argc, char *argv[])
 "                 invocations of 3dROIstats.  Confused?  Then don't use\n"
 "                 this option!\n"
 "  -zerofill ZF   For ROI labels not found, use 'ZF' instead of a blank\n"
-"                 in the output file. This option is useless without -numROI\n"
+"                 in the output file. This option is useless without -numROI.\n"
+"                 The option -zerofill defaults to '0'.\n"
 "\n"
 "  -roisel SEL.1D Only considers ROIs denoted by values found in SEL.1D\n"
 "                 Note that the order of the ROIs as specified in SEL.1D\n"
@@ -155,6 +128,8 @@ int main(int argc, char *argv[])
 "  -summary      Only output a summary line with the grand mean \n"
 "                across all briks in the input dataset. \n"
 "                This option cannot be used with -nomeanout.\n"
+"  -mode       Compute the mode of all voxels. (integral valued sets only)\n"
+"  -nzmode     Compute the mode of non_zero voxels.\n"
 "\n"
 "The output is printed to stdout (the terminal), and can be\n"
 "saved to a file using the usual redirection operation '>'.\n"
@@ -165,23 +140,62 @@ int main(int argc, char *argv[])
 
 	printf("\n" MASTER_SHORTHELP_STRING);
 
-	PRINT_COMPILE_DATE ; exit(0);
-    }
+	PRINT_COMPILE_DATE ;    
+   return;
+}
+
+int main(int argc, char *argv[])
+{
+    THD_3dim_dataset *mask_dset = NULL, *input_dset = NULL ;
+    int mask_subbrik = 0;
+    int sigma = 0, nzsigma = 0, mean = 1, nzmean = 0, nzcount = 0;
+    int debug = 0, quiet = 0, summary = 0;
+    int minmax = 0, nzminmax = 0, donzsum = 0;		/* 07 July, 2004 [rickr] */
+    short *mask_data;
+    int nvox, i, brik;
+    int num_ROI, ROI, mask_f2s = 0;
+    int force_num_ROI = 0;	/* Added 5/00 */
+    int narg = 1;
+    double *sum=NULL, *sumsq=NULL, *nzsum=NULL, sig, *sumallbriks=NULL;
+    double  *min=NULL, *max=NULL, *nzsumsq=NULL,
+            *nzmin=NULL, *nzmax=NULL;		/* 07 July, 2004 [rickr] */
+    long *voxels=NULL, *nzvoxels=NULL;
+    float *input_data;
+    byte *temp_datab;
+    short *temp_datas;
+    double *percentile=NULL;
+    float *fv = NULL;
+    int *iv=NULL, niv=0, *modes=NULL;
+    int nfv = 0, perc = 0, nzperc = 0, mode = 0, nzmode=0;
+    int nobriklab=0 ;  /* 14 Mar 2008 */
+    int disp1d=1;   /* ZSS May 2008 */
+    byte *roisel=NULL;
+    char sbuf[257]={""};
+    char sklab[128]={""};
+    char zerofill[32]={"0"};
 
    /*-- 20 Apr 2001: addto the arglist, if user wants to [RWCox] --*/
 
-   machdep() ; AFNI_logger("3dROIstats",argc,argv) ;
+   mainENTRY("3dROIstats");machdep() ; AFNI_logger("3dROIstats",argc,argv) ;
 
    { int new_argc ; char ** new_argv ;
      addto_args( argc , argv , &new_argc , &new_argv ) ;
      if( new_argv != NULL ){ argc = new_argc ; argv = new_argv ; }
    }
 
-    /* scan argument list */
+   /* scan argument list */
+   if (argc == 1) { usage_3dROIstats(1); exit(0); } /* Bob's help shortcut */
+
    disp1d = 0;
    mean = 1;   /* LEAVE this as the default ZSS March 09 2010 */
     while (narg < argc && argv[narg][0] == '-') {
 
+    if (strcmp(argv[narg], "-h") == 0 || strcmp(argv[narg], "-help") == 0) {
+      usage_3dROIstats(strlen(argv[narg]) > 3 ? 2:1);
+      exit(0);
+    }
+
+
 	if (strncmp(argv[narg], "-mask_f2short", 9) == 0) {
             mask_f2s = 1;   /* convert float mask to short */
 	    narg++;
@@ -275,7 +289,7 @@ int main(int argc, char *argv[])
 	    narg++;
 	    continue;
 	}
-       if (strncmp(argv[narg], "-nzmedian", 9) == 0) {
+   if (strncmp(argv[narg], "-nzmedian", 9) == 0) {
 	    if (perc) {
              Error_Exit("nzperc cannot be used with perc");
             }
@@ -283,6 +297,22 @@ int main(int argc, char *argv[])
 	    narg++;
 	    continue;
 	}
+   if (strcmp(argv[narg], "-mode") == 0) {
+	    if (nzmode) {
+             Error_Exit("mode cannot be used with nzmode");
+            }
+            mode = 1;
+	    narg++;
+	    continue;
+	}
+   if (strcmp(argv[narg], "-nzmode") == 0) {
+	    if (mode) {
+             Error_Exit("nzmode cannot be used with mode");
+            }
+            nzmode = 1;
+	    narg++;
+	    continue;
+	}
 	if (strncmp(argv[narg], "-nzmean", 7) == 0) {
 	    nzmean = 1;
 	    narg++;
@@ -336,7 +366,7 @@ int main(int argc, char *argv[])
 	/* added 5/00 */
 	if (strncmp(argv[narg], "-numROI", 5) == 0) {
 	    force_num_ROI = (int) atol(argv[++narg]);
-	    narg++;
+       narg++;
 	    continue;
 	}
 	if (strncmp(argv[narg], "-zerofill", 5) == 0) {
@@ -344,10 +374,18 @@ int main(int argc, char *argv[])
 	    narg++;
 	    continue;
 	}
-        fprintf(stderr,"\n** have option %s\n", argv[narg]);
-	Error_Exit("Unknown option");
+
+	   ERROR_message("Unknown option %s\n", argv[narg]);
+      suggest_best_prog_option(argv[0], argv[narg]);
+      exit(1);
     }
 
+    if (argc < 3) {
+      ERROR_message("Too few options, use -help for details");
+      exit(1);
+    }
+
+
    if (!mean && summary) {
       Error_Exit("Cannot use -nomeanout with -summary");
    }
@@ -477,31 +515,41 @@ int main(int argc, char *argv[])
 	    non_zero[i] = num_ROI;
 	    num_ROI++;
 	    if (!quiet && !summary && !force_num_ROI) {
-		if (mean) fprintf(stdout, "\tMean_%d  ", i - 32768);
+          AFNI_get_dset_val_label(mask_dset, (double)(i - 32768), sklab);
+          if (sklab[0]=='\0') {
+            sprintf(sklab,"%d",i - 32768);
+          }
+		if (mean) fprintf(stdout, "\tMean_%s  ", sklab);
 		if (nzmean)
-		    fprintf(stdout, "\tNZMean_%d", i - 32768);
+		    fprintf(stdout, "\tNZMean_%s", sklab);
 		if (nzcount)
-		    fprintf(stdout, "\tNZcount_%d", i - 32768);
+		    fprintf(stdout, "\tNZcount_%s", sklab);
 		if (sigma)
-		    fprintf(stdout, "\tSigma_%d", i - 32768);
+		    fprintf(stdout, "\tSigma_%s", sklab);
 		if (nzsigma)
-		    fprintf(stdout, "\tNZSigma_%d", i - 32768);
+		    fprintf(stdout, "\tNZSigma_%s", sklab);
 		if (minmax) {
-		    fprintf(stdout, "\tMin_%d   ", i - 32768);
-		    fprintf(stdout, "\tMax_%d   ", i - 32768);
+		    fprintf(stdout, "\tMin_%s   ", sklab);
+		    fprintf(stdout, "\tMax_%s   ", sklab);
 		}
 		if (nzminmax) {
-		    fprintf(stdout, "\tNZMin_%d ", i - 32768);
-		    fprintf(stdout, "\tNZMax_%d ", i - 32768);
+		    fprintf(stdout, "\tNZMin_%s ", sklab);
+		    fprintf(stdout, "\tNZMax_%s ", sklab);
 		}
       if (perc) {
-         fprintf(stdout, "\tMed_%d ", i - 32768);
+         fprintf(stdout, "\tMed_%s ", sklab);
       }
       if (nzperc) {
-         fprintf(stdout, "\tNZMed_%d ", i - 32768);
+         fprintf(stdout, "\tNZMed_%s ", sklab);
       }
       if (donzsum) {
-         fprintf(stdout, "\tNZSum_%d ", i - 32768);
+         fprintf(stdout, "\tNZSum_%s ", sklab);
+      }
+      if (mode) {
+         fprintf(stdout, "\tMod_%s ", sklab);
+      }
+      if (nzmode) {
+         fprintf(stdout, "\tNZMod_%s ", sklab);
       }
 	    }
 	}
@@ -510,31 +558,41 @@ int main(int argc, char *argv[])
 	for (i = 1; i <= force_num_ROI; i++) {
 	    non_zero[i + 32768] = (i - 1);
 	    if (!quiet && !summary) {
-		if (mean) fprintf(stdout, "\tMean_%d  ", i );
+          AFNI_get_dset_val_label(mask_dset, (double)(i), sklab);
+          if (sklab[0]=='\0') {
+            sprintf(sklab,"%d",i);
+          }
+		if (mean) fprintf(stdout, "\tMean_%s  ", sklab );
 		if (nzmean)
-		    fprintf(stdout, "\tNZMean_%d", i );
+		    fprintf(stdout, "\tNZMean_%s", sklab );
 		if (nzcount)
-		    fprintf(stdout, "\tNZcount_%d", i );
+		    fprintf(stdout, "\tNZcount_%s", sklab );
 		if (sigma)
-		    fprintf(stdout, "\tSigma_%d", i );
+		    fprintf(stdout, "\tSigma_%s", sklab );
 		if (nzsigma)
-		    fprintf(stdout, "\tNZSigma_%d", i );
+		    fprintf(stdout, "\tNZSigma_%s", sklab );
 		if (minmax) {
-		    fprintf(stdout, "\tMin_%d   ", i );
-		    fprintf(stdout, "\tMax_%d   ", i );
+		    fprintf(stdout, "\tMin_%s   ", sklab );
+		    fprintf(stdout, "\tMax_%s   ", sklab );
 		}
 		if (nzminmax) {
-		    fprintf(stdout, "\tNZMin_%d ", i );
-		    fprintf(stdout, "\tNZMax_%d ", i );
+		    fprintf(stdout, "\tNZMin_%s ", sklab );
+		    fprintf(stdout, "\tNZMax_%s ", sklab );
 		}
       if (perc) {
-         fprintf(stdout, "\tMed_%d ", i );
+         fprintf(stdout, "\tMed_%s ", sklab );
       }
       if (nzperc) {
-         fprintf(stdout, "\tNZMed_%d ", i );
+         fprintf(stdout, "\tNZMed_%s ", sklab );
       }
       if (donzsum) {
-         fprintf(stdout, "\tNZSum_%d ", i );
+         fprintf(stdout, "\tNZSum_%s ", sklab );
+      }
+      if (mode) {
+         fprintf(stdout, "\tMod_%s ", sklab );
+      }
+      if (nzmode) {
+         fprintf(stdout, "\tNZMod_%s ", sklab );
       }
 	    }
 	}
@@ -590,12 +648,17 @@ int main(int argc, char *argv[])
 	    continue;
 	}
 	if (DSET_NVOX(input_dset) != nvox) {
-	    WARNING_message("Input dataset %s is different size than mask - skipping",
-                       argv[narg]);
+	    WARNING_message(
+         "Dataset %s has %d voxels/nodes while the mask has %d - skipping",
+                       argv[narg], DSET_NVOX(input_dset), nvox);
 	    continue;
 	}
    if( !EQUIV_GRIDS(mask_dset,input_dset) )
-     WARNING_message("Input dataset %s grid mismatch from mask",argv[narg]) ;
+     WARNING_message("Input dataset %s grid mismatch from mask.\n"
+             "Try the following command for grid comparison:\n"
+             " 3dinfo -header_line -prefix -same_all_grid %s %s\n"
+             ,argv[narg], 
+             DSET_HEADNAME(mask_dset), DSET_HEADNAME(input_dset)) ;
 
 	DSET_load(input_dset);
 
@@ -645,12 +708,17 @@ int main(int argc, char *argv[])
 
 	    case MRI_float:{
 		    float fac = DSET_BRICK_FACTOR(input_dset, brik);
-		    input_data = (float *) DSET_ARRAY(input_dset, brik);
-		    if (fac == 0)
+          input_data = (float *) DSET_ARRAY(input_dset, brik);
+		    
+          if (fac == 0)
 			fac = 1.0;
-		    else
-			for (i = 0; i < nvox; input_data[i++] *= fac);
-		    break;
+		    else {
+			DSET_unload(input_dset); 
+         DSET_mallocize(input_dset); DSET_load(input_dset);
+         input_data = (float *) DSET_ARRAY(input_dset, brik);
+         for (i = 0; i < nvox; input_data[i++] *= fac);
+		    }
+          break;
 		}
 
 	    default:{
@@ -704,7 +772,9 @@ int main(int argc, char *argv[])
 			sumsq[ROI] += input_data[i] * input_data[i];
 		}
 	    }
-       /* do the damned median, simple, not fastest implementation but good enough*/
+       
+       /* do the damned median, simple, not fastest implementation 
+          but good enough*/
        if (perc || nzperc) {
          percentile = (double *)malloc(sizeof(double)*num_ROI);
          fv = (float *)malloc(sizeof(float)*nvox);
@@ -729,6 +799,41 @@ int main(int argc, char *argv[])
          } /* ROI */
          free(fv); fv = NULL;
 	    }
+       
+       /* do the mode */
+       if (mode || nzmode) {
+         modes = (int *)malloc(sizeof(int)*num_ROI);
+         iv = (int *)malloc(sizeof(int)*nvox);
+         if (!iv || !modes) {
+            Error_Exit("Failed to allocate for iv");
+         }
+         for (ROI=0; ROI < num_ROI; ++ROI){ /* ROI */
+            memset(iv, 0, sizeof(int)*nvox);
+            niv = 0;
+            for (i = 0; i < nvox; i++) { /* i */
+               if (mask_data[i] && ROI == non_zero[mask_data[i] + 32768]) {
+                  if (mode) {
+                     iv[niv] = (int)input_data[i]; ++niv;
+                  } else { /* non zero only */
+                     if (input_data[i] != 0.0) {
+                        iv[niv] = (int)input_data[i]; ++niv;
+                     }
+                  }
+               }
+            }/* i */
+            /* get the mode */
+            if (niv) {
+               modes[ROI] = qmode_int( iv, niv) ;
+            } else {
+               WARNING_message("ROI %d has no values in it, mode set to 0\n",
+                              ROI);
+               modes[ROI] = 0;
+            }
+         } /* ROI */
+         free(iv); iv = NULL;
+	    }
+       
+       
 
        /* print the next line of results */
 	    if (!quiet && !summary){
@@ -789,6 +894,9 @@ int main(int argc, char *argv[])
          if (donzsum) {
              fprintf(stdout, "\t%f", nzsum[i]);
 		   }
+         if (mode || nzmode) {
+			    fprintf(stdout, "\t%d", modes[i] );
+			}
           } else {	/* no voxels, so just leave blanks */
 			if (mean) fprintf(stdout, "\t%s", zerofill);
 			if (nzmean)
@@ -811,7 +919,9 @@ int main(int argc, char *argv[])
 			    fprintf(stdout, "\t%s", zerofill);
          if (donzsum)
              fprintf(stdout, "\t%s", zerofill);
-		    }
+		   if (mode || nzmode)
+			    fprintf(stdout, "\t%s", zerofill); 
+          }
 		}		/* loop over ROI for print */
 
 		fprintf(stdout, "\n");
@@ -865,6 +975,9 @@ int main(int argc, char *argv[])
     if (perc || nzperc) {
 	if (percentile) free(percentile); percentile = NULL;
 	 }
+    if (mode || nzmode) {
+	if (modes) free(modes); modes = NULL;
+	 }
     if (roisel) free(roisel); roisel=NULL;
     exit(0);
 }
