@@ -286,7 +286,7 @@ int main( int argc , char * argv[] )
   "Explanation of 3dclust Output:                                          \n"
   "-----------------------------                                           \n"
   "                                                                        \n"
-  "   Volume       : Volume that makes us the cluster, in microliters (mm^3)\n"
+  "   Volume       : Volume that makes up the cluster, in microliters (mm^3)\n"
   "                  (or the number of voxels, if -dxyz=1 is given)        \n"
   "                                                                        \n"
   "   CM RL        : Center of mass (CM) for the cluster in the Right-Left \n"
@@ -727,7 +727,8 @@ int main( int argc , char * argv[] )
                                ADN_nvals  , 1 ,
                              ADN_none ) ;
             EDIT_substitute_brick(qset,0,MRI_short,mmm) ; mmm = NULL ;
-            tross_Make_History( "3dclust" , argc , argv , dset ) ;
+            tross_Copy_History( dset , qset ) ;
+            tross_Make_History( "3dclust" , argc , argv , qset ) ;
             DSET_write(qset) ; WROTE_DSET(qset) ; DSET_delete(qset) ;
           }
 
