@@ -239,7 +239,7 @@ if (detail > 1) {
 "                         ~ Available keys and their variants are:\n"
 "                         [, ], comma (or ','), period (or '.'), space,\n"
 "                         a, b, d, G, j, m, n, p, r, t, z, \n"
-"                         up, down, left, right, and F1 to F9.\n"
+"                         up, down, left, right, and F1 to F12.\n"
 "                         ~ Key variants are specified this way:\n"
 "                         ctrl+Up or ctrl+alt+Down etc.\n"
 "                         ~ For help on key actions consult SUMA's\n"
@@ -273,7 +273,8 @@ if (detail > 1) {
 "                  At the shell you would enter:\n"
 "                    DriveSuma -com viewer_cont '-key:v\"0.8 0 10.3\"' ctrl+j\n"
 "                  In another example, say you want to jump to node 54 on the\n"
-"                  right hemisphere (hence the 'R' in '54R'), then you would execute:\n"
+"                  right hemisphere (hence the 'R' in '54R'), then you would\n"
+"                  execute:\n"
 "                    DriveSuma -com viewer_cont '-key:v54R' j\n"
 "        -viewer VIEWER: Specify which viewer should be acted \n"
 "                        upon. Default is viewer 'A'. Viewers\n"
@@ -281,6 +282,9 @@ if (detail > 1) {
 "                        they can be acted upon.\n"
 "                        You can also refer to viewers with integers\n"
 "                        0 for A, 1 for B, etc.\n"
+"                        For -viewer to take effect it must be in the\n"
+"                        same -com viewer_cont ... commands. For example:\n"
+"               ... -com viewer_cont -viewer B -viewer_size 600 900 ...\n"
 "        -viewer_width or (-width) WIDTH: Set the width in pixels of\n"
 "                                     the current viewer.\n"
 "        -viewer_height or (-height) HEIGHT: Set the height in pixels of\n"
@@ -288,6 +292,8 @@ if (detail > 1) {
 "        -viewer_size WIDTH HEIGHT : Convenient combo of -viewer_width \n"
 "                                    and -viewer_height\n"
 "        -viewer_position X Y: Set position on the screen\n"
+"        -controller_position X Y: Set position of the object (surface)\n"
+"                                  controller on the screen\n"
 "        -inout_notify y/n: Turn on or off function call tracing\n"
 "        -N_foreg_smooth n: Number of foreground smoothing iterations\n"
 "                           Same as suma's interactive '8' key or what\n"
@@ -382,9 +388,11 @@ if (detail > 1) {
 "                         is symmetric from -|BR0| to |BR0|.\n"
 "       -B_scale BS0 BS1: Modulate brightness by BS0 factor for BR0 or lower\n"
 "                         by BS1 factor for BR1 or higher, and linearly \n"
-"                         interpolate scaling for BR0 < values < BR1\n" 
+"                         interpolate scaling for BR0 < values < BR1\n"
 "       -Dim DIM: Set the dimming factor.\n"
 "       -Opa OPA: Set the opacity factor.\n"
+"       -Clst RAD AREA: Set the clustering parameters\n"
+"       -UseClst y/n: Turn on/off clustering\n"
 "       -setSUMAenv \"'ENVname=ENVvalue'\": Set an ENV in SUMA. Note that\n"
 "                      most SUMA env need to be set at SUMA's launch time. \n"
 "                      Setting the env from DriveSuma may not achieve what \n" 
@@ -938,6 +946,7 @@ int SUMA_DriveSuma_ParseCommon(NI_group *ngr, int argtc, char ** argt)
          }
          brk = YUP;
       }
+      
       if (!brk && ( (strcmp(argt[kar], "-B_scale") == 0) ) )
       {
          if (kar+2 >= argtc)
@@ -1034,6 +1043,67 @@ int SUMA_DriveSuma_ParseCommon(NI_group *ngr, int argtc, char ** argt)
          brk = YUP;
       }
       
+      if (!brk && ( (strcmp(argt[kar], "-Clst") == 0) ||
+                    (strcmp(argt[kar], "-Clust") == 0) ) )
+      {
+         if (kar+1 >= argtc)
+         {
+            fprintf (SUMA_STDERR, "need two values after -Clust \n");
+            SUMA_RETURN(0);
+         }
+         
+         argt[kar][0] = '\0';
+         ++kar; N = 1; stmp = NULL; nums = 0;
+         while (  kar < argtc && 
+                  argt[kar] && 
+                  SUMA_isNumString(argt[kar],(void *)((long int)N))) {
+            stmp = SUMA_append_replace_string(stmp, argt[kar], " ", 1); ++nums;
+            argt[kar][0] = '\0'; ++kar;
+         } --kar;
+         if (!stmp || nums != 2) {
+            SUMA_S_Err( "Bad format for -Clst option values;\n"
+                        " 2 and only 2 values allowed.");
+            SUMA_RETURN(0);
+         }
+         nv = SUMA_StringToNum(stmp, (void *)dv3, 3,2);
+         if (nv < 1 || nv > 2) {
+            SUMA_S_Err("Bad Clst string.");
+            SUMA_RETURN(0);
+         }else {
+            /* have range, set it please */
+            SUMA_free(stmp); stmp = NULL; 
+            stmp = (char *)SUMA_malloc(sizeof(char)*nv*50);
+            sprintf(stmp,"%f , %f", dv3[0], dv3[1]);
+            NI_set_attribute(ngr, "Clst", stmp);
+            SUMA_LHv("Clst of %s\n", stmp);
+            SUMA_free(stmp); stmp = NULL;
+         }
+         brk = YUP;
+      }
+      
+      if (!brk && (  (strcmp(argt[kar], "-UseClst") == 0) ||
+                     (strcmp(argt[kar], "-UseClust") == 0) ) )
+      {
+         if (kar+1 >= argtc)
+         {
+            fprintf (SUMA_STDERR, "need a 'y/n' after -UseClust \n");
+            SUMA_RETURN(0);
+         }
+         argt[kar][0] = '\0';
+         ++kar;
+         if (argt[kar][0] == 'y' || argt[kar][0] == 'Y')  
+            NI_set_attribute(ngr, "UseClst", "y");
+         else if (argt[kar][0] == 'n' || argt[kar][0] == 'N')  
+            NI_set_attribute(ngr, "UseClst", "n");
+         else {
+            fprintf (SUMA_STDERR, "need a 'y/n' after -UseClust \n");
+            SUMA_RETURN(0);
+         }
+         argt[kar][0] = '\0';
+         brk = YUP;
+      }
+   
+      
       if (!brk && (strcmp(argt[kar], "-viewer") == 0))
       {
          if (kar+1 >= argtc)
@@ -1644,6 +1714,34 @@ int SUMA_DriveSuma_ParseCommon(NI_group *ngr, int argtc, char ** argt)
          argt[kar][0] = '\0';
          brk = YUP;
       }
+      if (  !brk && 
+            (  (strcmp(argt[kar], "-controller_position") == 0)  ) )
+      {
+         if (kar+2 >= argtc)
+         {
+            fprintf (SUMA_STDERR, "need 2 numbers after %s \n", argt[kar]);
+            SUMA_RETURN(0);
+         }
+         argt[kar][0] = '\0';++kar;
+         if (atoi(argt[kar]) < 0 || atoi(argt[kar]) > 4000) {
+            fprintf (SUMA_STDERR, 
+               "Have %d for X in pixels. Range [0 4000].! \n", atoi(argt[kar]));
+            SUMA_RETURN(0);
+         }
+         NI_set_attribute(ngr, "ContX", argt[kar]);
+         argt[kar][0] = '\0';++kar;
+         if (atoi(argt[kar]) < 0 || atoi(argt[kar]) > 4000) {
+            fprintf (SUMA_STDERR, 
+               "Have %d for Y in pixels!  Range [0 4000].\n", atoi(argt[kar]));
+            SUMA_RETURN(0);
+         }
+         NI_set_attribute(ngr, "ContY", argt[kar]);
+         NI_set_attribute(ngr, "DoViewerSetup","y"); /* flag indicating 
+                                                      need to setup viewer, 
+                                                      a la vvs */
+         argt[kar][0] = '\0';
+         brk = YUP;
+      }
       if (0 && !brk) { /* do not enforce this here */
 			fprintf (SUMA_STDERR,
                   "Error %s:\n"
@@ -2313,11 +2411,21 @@ int SUMA_ProcessCommand(char *com, SUMA_GENERIC_ARGV_PARSE *ps, char *EchoNel)
          SUMA_SL_Warn("Failed in SUMA_SendToSuma\nCommunication halted.");
       }
       if (EchoNel) NEL_WRITE_TX(ngr, EchoNel, suc);
-      SUMA_Wait_Till_Stream_Goes_Bad(ps->cs, 100, 1000, 0);
+      
+      if (1) {
+         /* go bad anyway without waiting for stream to go bad 
+         because suma will be dying. No point in waiting if
+         if there was a communication error. 
+               Added per Yaroslav Halchenko's request     Sept 2013 */
+         ps->cs->GoneBad = YUP;
+      } else {
+         SUMA_Wait_Till_Stream_Goes_Bad(ps->cs, 100, 1000, 0);
+      }
       NI_free_element(ngr); ngr = NULL;
       ans = -1; 
    } else {
-      fprintf(SUMA_STDERR, "Error %s: Action '%s' not supported.\n", FuncName, act);
+      fprintf(SUMA_STDERR, 
+               "Error %s: Action '%s' not supported.\n", FuncName, act);
       ans = NOPE;
    }
    
@@ -2394,8 +2502,10 @@ int main (int argc,char *argv[])
             /* recalculate surface normals */
             SUMA_RECOMPUTE_NORMALS(SO); 
             if (ps->cs->Send) {
-               if (!SUMA_SendToSuma (SO, ps->cs, (void *)SO->NodeList, SUMA_NODE_XYZ, 1)) {
-                  SUMA_SL_Warn("Failed in SUMA_SendToSuma\nCommunication halted.");
+               if (!SUMA_SendToSuma (SO, ps->cs, 
+                                     (void *)SO->NodeList, SUMA_NODE_XYZ, 1)) {
+                  SUMA_SL_Warn("Failed in SUMA_SendToSuma\n"
+                               "Communication halted.");
                }
             }
          ++cnt;
@@ -2408,18 +2518,20 @@ int main (int argc,char *argv[])
             fprintf(SUMA_STDERR,"Command %d: %s\n", i, Opt->com[i]);
          }
          if (!(exflag = SUMA_ProcessCommand(Opt->com[i], ps, Opt->s))) {
-            fprintf(SUMA_STDERR,"Error %s: Failed in processing command\n%s\n", FuncName, Opt->com[i]); 
+            SUMA_S_Errv("Failed in processing command\n%s\n", Opt->com[i]); 
             exit(1);
          }   
          if (exflag == -1) { /*gone daddy gone */ 
-            fprintf(SUMA_STDERR,"There's no more reason to exist.\nFarewell dear friends.\n");
+            SUMA_S_Note("There's no more reason to exist.\n"
+                        "Farewell dear friends.\n");
             exit(0);
          }
       }
    }
    
    SUMA_LH("Freedom");
-   /* you don't want to exit rapidly because the SUMA might not be done processing the last elements*/
+   /* you don't want to exit rapidly because the SUMA might 
+       not be done processing the last elements*/
    if (ps->cs->Send && !ps->cs->GoneBad) {
       /* cleanup and close connections */
       if (!SUMA_SendToSuma (SO, ps->cs, NULL, SUMA_NODE_XYZ, 2)) {
@@ -2428,7 +2540,8 @@ int main (int argc,char *argv[])
    }   
    if (ps) SUMA_FreeGenericArgParse(ps); ps = NULL;
    if (Opt) Opt = SUMA_Free_Generic_Prog_Options_Struct(Opt);
-   if (!SUMA_Free_CommonFields(SUMAg_CF)) SUMA_error_message(FuncName,"SUMAg_CF Cleanup Failed!",1);
+   if (!SUMA_Free_CommonFields(SUMAg_CF)) 
+      SUMA_error_message(FuncName,"SUMAg_CF Cleanup Failed!",1);
    
    exit(0);
    
