@@ -1,14 +1,19 @@
 #ifndef SUMA_STRING_PARSE_INCLUDED
 #define SUMA_STRING_PARSE_INCLUDED
 
-/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> Begin string parsing macros <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */
+/* >>>>>>>>>>>>>>>>>>>>>>> Begin string parsing macros <<<<<<<<<<<<<<<<<<<<<<< */
+/* See also SUMA_IS_DIGIT_CHAR, SUMA_IS_NUM_CHAR */
 #define SUMA_IS_DIGIT(c) ( ((c) < '0' || (c) > '9') ? 0 : 1 )
  
+#define SUMA_IS_PURE_BLANK(c) ( ((c) == ' ' || (c) == '\t' ) )
+
 /*! a macro version of C's isspace
    returns 1 if charater is considered a blank
    \sa SUMA_SKIP_BLANK
 */
-#define SUMA_IS_BLANK(c) ( ((c) == ' ' || (c) == '\t' || (c) == '\n' || (c) == '\v' || (c) == '\f' || (c) == '\r') ? 1 : 0 )
+#define SUMA_IS_BLANK(c) ( (SUMA_IS_PURE_BLANK(c) || (c) == '\n' || (c) == '\v' || (c) == '\f' || (c) == '\r') ? 1 : 0 )
+
+/* See also SUMA_IS_EOL */
 #define SUMA_IS_LINE_END(c) ( ((c) == '\n' || (c) == '\f' || (c) == '\r') ? 1 : 0 )
 #define SUMA_IS_NICE_PREFIX_CHAR(c) ( (SUMA_IS_BLANK(c) \
                                        || (c) == ':' || (c) == ';' || (c) == '[' || (c) == ']' || (c) == '{' || (c) == '}'  \
@@ -24,6 +29,11 @@
 #define SUMA_SKIP_BLANK(op, eop){  \
    while (*op != '\0' && op != eop && SUMA_IS_BLANK(*op)) ++op; \
 }
+
+#define SUMA_SKIP_PURE_BLANK(op, eop){  \
+   while (*op != '\0' && op != eop && SUMA_IS_PURE_BLANK(*op)) ++op; \
+}
+
 #define SUMA_SKIP_LINE(op, eop){   \
    while (*op != '\0' && op != eop && *op != '\n' && *op != '\f' && *op != '\r') ++op; \
    SUMA_SKIP_BLANK(op, eop);\
@@ -77,6 +87,20 @@
    }  \
 }
 
+/* See also SUMA_IS_LINE_END */
+#define SUMA_IS_EOL(cc) ((  (cc) == '\n' || (cc) == '\v'                      \
+                          ||(cc) == '\f' || (cc) == '\r' || (cc) == '\0') )
+#define SUMA_SKIP_TO_EOL(op, eop){  \
+   char m_quote_open = '\0';   \
+   while (*op != '\0' && op !=eop && !(!m_quote_open && SUMA_IS_EOL(*op)) ) { \
+      if (*op == '"' || *op == '\'') {  \
+         if (!m_quote_open) m_quote_open = *op; \
+         else if (m_quote_open == *op) m_quote_open = '\0'; \
+      }  \
+      ++op; \
+   }  \
+}
+
 #define SUMA_SKIP_TO_NEXT_CHAR(op, eop, ch){  \
    char m_quote_open = '\0';   \
    while (*op != '\0' && op !=eop && !( !m_quote_open && (*op == ch)) ) { \
@@ -104,6 +128,14 @@
    SUMA_SKIP_TO_NEXT_BLANK(op2, eop);  \
 }
 
+#define SUMA_GET_TO_EOL(op, eop, op2){  \
+   if (!SUMA_IS_EOL(*op)) { \
+      SUMA_SKIP_BLANK(op, eop); /* skip first blanks*/   \
+      op2 = op;                 /* skip till next blanks */ \
+      SUMA_SKIP_TO_EOL(op2, eop);  \
+   } else { op2 = op; } /* Stay right where you are */ \
+}
+
 /*! \brief Count number of blank delimited words. 
       Does not alter op or eop
       Returns number of words in N_word;
@@ -276,20 +308,73 @@
 }
 
 /*!
-   \brief Fills characters between [op,op2[ into a preallocated string str
+   \brief Fills characters between [m_op,m_op2[ into a preallocated string str
 */
-#define SUMA_FILL_STRING(op,op2,strinp){   \
+#define SUMA_FILL_STRING(m_op,m_op2,strinp){   \
    char *sval = strinp;\
-   if (!op2) { /* copy till end */  \
-      while (*op != '\0') { *sval = *op; ++sval; ++op; } \
+   if (!m_op2) { /* cm_opy till end */  \
+      while (*m_op != '\0') { *sval = *m_op; ++sval; ++m_op; } \
    } else { \
-      while (*op != '\0' && op < (char*)op2) {  \
-         *sval = *op; ++sval; ++op; \
+      while (*m_op != '\0' && m_op < (char*)m_op2) {  \
+         *sval = *m_op; ++sval; ++m_op; \
       } \
    }  \
    *sval = '\0'; \
 }
 
+/*!
+   \brief Fills N characters between [m_op,m_op2[ into a preallocated string str
+   Make sure str can take N+1 chars
+*/
+#define SUMA_NFILL_STRING(m_op,m_op2,strinp,N){   \
+   char *sval = strinp; int m_n=0;\
+   if (!m_op2) { /* cm_opy till end */  \
+      while (*m_op != '\0' && m_n < N) { *sval = *m_op; ++sval; ++m_op; ++m_n;} \
+   } else { \
+      while (*m_op != '\0' && m_op < (char*)m_op2 && m_n < N) {  \
+         *sval = *m_op; ++sval; ++m_op; ++m_n;\
+      } \
+   }  \
+   *sval = '\0'; \
+}
+
+/*!
+   \brief print N characters between [op,op2[ into file pointer out
+   Leaves op and op2 in place
+*/
+#define SUMA_NPRINT_STRING(opi,op2,out,N, head, foot){   \
+   int m_n=0; FILE *m_outp=SUMA_STDERR; char *m_op=opi, *m_op2=op2;\
+   if (out) m_outp=out; \
+   if (head) fprintf(m_outp,"%s", head);  \
+   if (!m_op2) { /* write till end */  \
+      while (*m_op != '\0' && m_n < N) { \
+         fprintf(m_outp,"%c", *m_op); ++m_op; ++m_n;\
+      } \
+   } else { \
+      while (*m_op != '\0' && m_op < (char*)m_op2 && m_n < N) {  \
+         fprintf(m_outp,"%c", *m_op); ++m_op; ++m_n;\
+      } \
+   }  \
+   if (foot) fprintf(m_outp,"%s", foot);  \
+}
+/*!
+   \brief print characters between [op,op2[ into file pointer out
+   Leaves op and op2 in place
+*/
+#define SUMA_PRINT_STRING(opi,opi2,out,head, foot){   \
+   FILE *m_outp=SUMA_STDERR; char *m_op=opi, *m_op2 = opi2;\
+   if (out) m_outp=out; \
+   if (head) fprintf(m_outp,"%s", head);  \
+   if (!m_op2) { /* write till end */  \
+      while (*m_op != '\0') { fprintf(m_outp,"%c", *m_op); ++m_op;} \
+   } else { \
+      while (*m_op != '\0' && m_op < (char*)m_op2) {  \
+         fprintf(m_outp,"%c", *m_op); ++m_op;\
+      } \
+   }  \
+   if (foot) fprintf(m_outp,"%s", foot);  \
+}
+
 #define SUMA_DEBLANK_RHS(s) {\
    int ns;  \
    if (s && (ns=strlen(s))) { --ns; \
@@ -297,6 +382,7 @@
    }\
 }\
 
-/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> End string parsing macros <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */
+
+/* >>>>>>>>>>>>>>>>>>>>>>> End string parsing macros <<<<<<<<<<<<<<<<<<<<<<<< */
 
 #endif
