@@ -23,6 +23,12 @@
 #define SUMA_WITHDRAW   2
 #define SUMA_UNREALIZE  3
 
+/* Flags to move sub-bricks up or down using 
+   same functions to set a particular index 
+   Values must be < 0 and != -1 */
+#define SUMA_FORWARD_ONE_SUBBRICK    -444
+#define SUMA_BACK_ONE_SUBBRICK       -555 
+
 #define SUMA_XmArrowFieldMenu -123
 
 #define SUMA_CLOSE_MODE       SUMA_WITHDRAW
@@ -30,8 +36,19 @@
 
 #define SUMA_HOLD_IT  { \
    SUMA_S_Note("Waiting...");\
-   glXWaitGL();glXWaitX(); \
+   glXWaitGL();glXWaitX(); glFinish();\
    SUMA_S_Note("Done.");}  
+
+#define SUMA_CHECK_GL_ERROR(str)                                           \
+{                                                                  \
+    GLenum error;                                                  \
+    while((error = glGetError()) != GL_NO_ERROR)                   \
+       fprintf(stderr,"**************GL Error: %s (%s)\n", \
+         gluErrorString(error), str);  \
+}
+
+
+
                                   
 typedef struct suma_menu_item {
     char        *label;         /*!< the label for the item */
@@ -136,14 +153,17 @@ sets the select color of the widget to its foreground color */
                   "%s: Setting up matrix mode and perspective ...\n", \
                   FuncName); \
       if (csv->FOV[csv->iState] < 0.00001) { \
-         SUMA_S_Warnv("Fov (%f) seems messed up, resetting to %f\n",\
-            csv->FOV[csv->iState], FOV_INITIAL);   \
+         /* This can happen when only Non-SO objects are loaded
+            Just do nothing  without fanfare */ \
+         SUMA_LHv("Fov (%f) seems messed up, ignoring call\n",\
+            csv->FOV[csv->iState]);   \
+      }  else {\
+         glMatrixMode (GL_PROJECTION); \
+         glLoadIdentity ();   \
+         gluPerspective((GLdouble)csv->FOV[csv->iState], csv->Aspect, \
+                        SUMA_PERSPECTIVE_NEAR, SUMA_PERSPECTIVE_FAR); \
+                        /*lower angle is larger zoom,*/   \
       }  \
-      glMatrixMode (GL_PROJECTION); \
-      glLoadIdentity ();   \
-      gluPerspective((GLdouble)csv->FOV[csv->iState], csv->Aspect, \
-                     SUMA_PERSPECTIVE_NEAR, SUMA_PERSPECTIVE_FAR); \
-                     /*lower angle is larger zoom,*/   \
    }  else { \
       GLdouble m_sz = \
          0.5 *tan(SUMA_PI * csv->FOV[csv->iState] / 180.0) * \
@@ -187,6 +207,19 @@ sets the select color of the widget to its foreground color */
                   -csv->GVS[csv->StdView].RotaCenter[2]); \
 }   
 
+#define SUMA_GL_MAT_SHOW(mm,str) {\
+   int m_i; double m_dmatrix[16];  \
+   glGetDoublev(mm, m_dmatrix); \
+   SUMA_S_Note(str); \
+   for (m_i=0; m_i<4; ++m_i) {   \
+      fprintf(stderr,"\t");  \
+      fprintf(stderr,"%+3.3f\t%+3.3f\t%+3.3f\t%+3.3f\t", \
+         m_dmatrix[m_i],m_dmatrix[4+m_i],m_dmatrix[8+m_i],m_dmatrix[12+m_i]); \
+      fprintf(stderr,"\n");  \
+   }\
+}
+
+
 #define SUMA_SET_AS_NEEDED_2D_VIEW_ANGLE(sv) {  \
    int m_j = SUMA_BestStandardView(sv, SUMAg_DOv, SUMAg_N_DOv); \
    float m_a[3]; \
@@ -196,6 +229,8 @@ sets the select color of the widget to its foreground color */
    } else if (m_j == SUMA_2D_Z0L) { \
       m_a[0] = 0.0; m_a[1] = 0.0; m_a[2] = 1.0; \
       axis_to_quat(m_a, SUMA_PI, sv->GVS[m_j].currentQuat); \
+   } else if (m_j == SUMA_3D_Z0) { \
+      SUMA_HOME_QUAT(SUMA_3D_Z0, sv->GVS[m_j].currentQuat); \
    } \
 }
 
@@ -203,7 +238,13 @@ sets the select color of the widget to its foreground color */
 
 /* return the character for a viewer struct */
 #define SUMA_SV_CHAR(csv) \
-   (char)( 65+SUMA_WhichSV((csv), SUMAg_SVv, SUMA_MAX_SURF_VIEWERS) )
+   (char)( csv ? (65+SUMA_WhichSV((csv), SUMAg_SVv, SUMA_MAX_SURF_VIEWERS)):'-' )
+
+#define SV_IN_PRYING(sv) (((sv) && \
+                           (((sv)->GVS[(sv)->StdView].vLHpry[0] != 0.0f ) || \
+                            ((sv)->GVS[(sv)->StdView].vLHpry[1] != 0.0f ) || \
+                            ((sv)->GVS[(sv)->StdView].vLHpry[2] != 0.0f )) ) \
+                           ? 1:0)
 
 /* Make sure recording path is legit */
 #define SUMA_VALIDATE_RECORD_PATH(autorecord) {\
@@ -240,18 +281,28 @@ sets the select color of the widget to its foreground color */
 
 String *SUMA_get_fallbackResources ();         
 void SUMA_CullOption(SUMA_SurfaceViewer *, const char *action);
+SUMA_Boolean SUMA_glXMakeCurrent(Display *dpy, Window wdw, GLXContext cont,
+      char *fname, char *wlab, int force);
 Boolean SUMA_handleRedisplay (XtPointer w);
 void SUMA_postRedisplay(Widget w, XtPointer clientData, XtPointer call);
 GLenum SUMA_index_to_clip_plane(int iplane) ;
-int SUMA_PixelsToDisk(SUMA_SurfaceViewer *csv, int w, int h, GLubyte *pixels, 
-                      int colordepth, int verb, char *ufname); 
+int SUMA_PixelsToDisk(SUMA_SurfaceViewer *csv, int w, int h, GLvoid *pixels, 
+                      int colordepth, int verb, char *ufname, 
+                      int autoname, int over); 
 int SUMA_SnapToDisk(SUMA_SurfaceViewer *csv, int verb, int getback);
+SUMA_DO_LOCATOR *SUMA_SV_SortedRegistDO(SUMA_SurfaceViewer *csv, int *N_regs,
+                                        SUMA_DO *dov);
 void SUMA_display(SUMA_SurfaceViewer *csv, SUMA_DO *dov);
+void SUMA_display_one(SUMA_SurfaceViewer *csv, SUMA_DO *dov);
 Colormap SUMA_getShareableColormap_Eng(XVisualInfo * vi, Display *dpy);
 Colormap SUMA_getShareableColormap(SUMA_SurfaceViewer * csv);
 void SUMA_graphicsInit(Widget w, XtPointer clientData, XtPointer call);
 void SUMA_expose(Widget w, XtPointer clientData, XtPointer call);
 void SUMA_resize(Widget w, XtPointer clientData, XtPointer call);
+SUMA_Boolean SUMA_SV_DrawAreaDims_From_WindDims(SUMA_SurfaceViewer *sv);
+SUMA_Boolean SUMA_SV_WindDims_From_DrawAreaDims(SUMA_SurfaceViewer *sv);
+SUMA_Boolean SUMA_SV_InitDrawAreaOffset(SUMA_SurfaceViewer *sv);
+int SUMA_XErrHandler( Display *d , XErrorEvent *x );
 SUMA_Boolean SUMA_X_SurfaceViewer_Create (void);
 void SUMA_ButtOpen_pushed (Widget w, XtPointer cd1, XtPointer cd2);
 void SUMA_ButtClose_pushed (Widget w, XtPointer cd1, XtPointer cd2);
@@ -297,8 +348,12 @@ void SUMA_cb_XHviewlock_toggled(Widget w, XtPointer data, XtPointer callData);
 void SUMA_cb_closeSurfaceCont(Widget w, XtPointer data, XtPointer callData);
 void SUMA_cb_createSurfaceCont(Widget w, XtPointer data, XtPointer callData);
 void SUMA_cb_createSurfaceCont_SO(Widget w, XtPointer data, XtPointer callData);
+void SUMA_cb_createSurfaceCont_TDO(Widget w, XtPointer data, XtPointer callData);
+void SUMA_cb_createSurfaceCont_VO(Widget w, XtPointer data, XtPointer callData);
 void SUMA_cb_createSurfaceCont_GLDO(Widget w, XtPointer data, 
                                      XtPointer callData);
+void SUMA_cb_createSurfaceCont_MDO(Widget w, XtPointer data, 
+                                     XtPointer callData);
 void SUMA_cb_newSumaCont(Widget w, XtPointer client_data, XtPointer callData);
 void  SUMA_cb_doneSumaCont(Widget wcall, XtPointer cd1, XtPointer cbs);
 void SUMA_quit_timeout_CB( XtPointer client_data , XtIntervalId * id );
@@ -321,21 +376,27 @@ void SUMA_cb_FileSaveView (Widget w, XtPointer client_data, XtPointer callData);
 void SUMA_cb_FileLoadView (Widget w, XtPointer client_data, XtPointer callData);
 void SUMA_cb_moreSumaInfo (Widget w, XtPointer client_data, XtPointer callData);
 void SUMA_cb_moreSurfInfo (Widget w, XtPointer client_data, XtPointer callData);
-void SUMA_cb_moreViewerInfo (Widget w, XtPointer client_data, XtPointer callData);
+void SUMA_cb_moreViewerInfo (Widget w, XtPointer client_data, 
+                                                            XtPointer callData);
 void SUMA_ViewerInfo_destroyed (void *p);
 void SUMA_ViewerInfo_open (void *p);
 void SUMA_SumaInfo_destroyed (void *p);
 void SUMA_SumaInfo_open (void *p);
-SUMA_CREATE_TEXT_SHELL_STRUCT * SUMA_CreateTextShellStruct (void (*opencallback)(void *data), void *opendata, 
-                                                            void (*closecallback)(void*data), void *closedata);
+SUMA_CREATE_TEXT_SHELL_STRUCT * SUMA_CreateTextShellStruct (
+   void (*opencallback)(void *data), void *opendata, char *opendatatype, 
+   void (*closecallback)(void*data), void *closedata);
 SUMA_CREATE_TEXT_SHELL_STRUCT * SUMA_CreateTextShell (char *s, char *title, SUMA_CREATE_TEXT_SHELL_STRUCT *TextShellStruct);
-void SUMA_cb_search_text(Widget widget, XtPointer client_data, XtPointer call_data);
+void SUMA_cb_search_text(Widget widget, XtPointer client_data, 
+                         XtPointer call_data);
 char * SUMA_WriteStringToFile(char *fname, char *s, int, int);
 void SUMA_SaveTextShell(Widget w, XtPointer ud, XtPointer cd);
+void SUMA_RefreshTextShell(Widget w, XtPointer ud, XtPointer cd);
 void SUMA_DestroyTextShell (Widget w, XtPointer ud, XtPointer cd);
 void SUMA_SurfInfo_open (void *SO);
 void SUMA_SurfInfo_destroyed (void *SO);
-void SUMA_cb_ToggleCaseSearch (Widget widget, XtPointer client_data, XtPointer call_data);
+void SUMA_cb_ToggleCaseSearch (Widget widget, XtPointer client_data, 
+                               XtPointer call_data);
+void SUMA_cb_Mask (Widget w, XtPointer client_data, XtPointer callData);
 void SUMA_cb_helpUsage (Widget w, XtPointer data, XtPointer callData);
 void SUMA_cb_helpIO_notify(Widget w, XtPointer data, XtPointer callData);
 void SUMA_cb_helpEchoKeyPress(Widget w, XtPointer data, XtPointer callData);
@@ -350,11 +411,32 @@ void SUMA_cb_SetRenderMode(Widget widget, XtPointer client_data,
                            XtPointer call_data);
 void SUMA_cb_SetTransMode(Widget widget, XtPointer client_data, 
                            XtPointer call_data);
+void SUMA_cb_SetATransMode(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
 int SUMA_SetDsetViewMode(SUMA_ALL_DO *ado, int imenu, int update_menu) ;
+int SUMA_SetDsetFont(SUMA_ALL_DO *ado, int imenu, int updatemenu);
+int SUMA_SetDsetNodeRad(SUMA_ALL_DO *ado, int imenu, int updatemenu);
+int SUMA_SetDsetThrough(SUMA_ALL_DO *ado, int imenu, int updatemenu);
+int SUMA_SetDsetNodeCol(SUMA_ALL_DO *ado, int imenu, int updatemenu);
 void SUMA_cb_SetDsetViewMode(Widget widget, XtPointer client_data, 
                               XtPointer call_data);
+void SUMA_cb_SetDsetFont(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+void SUMA_cb_SetDsetNodeCol(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+void SUMA_cb_SetDsetNodeRad(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+void SUMA_cb_SetDsetThrough(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+void SUMA_cb_SetDsetGmatBord(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+int SUMA_SetDsetGmatBord(SUMA_ALL_DO *ado, int imenu, int updatemenu);
+int SUMA_SetDsetTxtShad(SUMA_ALL_DO *ado, int imenu, int updatemenu); 
+void SUMA_cb_SetDsetTxtShad(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
 void SUMA_cb_ToolsDrawROI (Widget w, XtPointer client_data, XtPointer call_data);
-void SUMA_cb_CloseDrawROIWindow(Widget w, XtPointer client_data, XtPointer call_data);
+void SUMA_cb_CloseDrawROIWindow(Widget w, XtPointer client_data, 
+                                XtPointer call_data);
 void SUMA_CreateDrawROIWindow(void);
 SUMA_Boolean SUMA_InitializeDrawROIWindow (SUMA_DRAWN_ROI *DrawnROI);
 SUMA_Boolean SUMA_OpenDrawROIWindow (SUMA_DRAWN_ROI *DrawnROI);
@@ -386,8 +468,10 @@ void SUMA_DrawROI_NewLabel (void * data);
 void SUMA_ATF_change_value (XtPointer client_data, XtIntervalId *id);
 void SUMA_ATF_start_stop (Widget w, XtPointer client_data, XtPointer call_data);
 void SUMA_DrawROI_NewValue (void * data);
-void SUMA_ATF_cb_label_change (Widget w, XtPointer client_data, XtPointer call_data);
+void SUMA_ATF_cb_label_change (Widget w, XtPointer client_data, 
+                               XtPointer call_data);
 void SUMA_ATF_SetString (SUMA_ARROW_TEXT_FIELD * AF);
+void SUMA_ATF_GetString (SUMA_ARROW_TEXT_FIELD * AF, char *sbuf);
 void SUMA_ATF_SetValue (SUMA_ARROW_TEXT_FIELD * AF);
 void SUMA_ATF_cb_label_Modify (Widget w, XtPointer client_data, XtPointer call_data);
 void SUMA_leave_EV( Widget w , XtPointer client_data ,
@@ -417,20 +501,24 @@ SUMA_LIST_WIDGET * SUMA_AllocateScrolledList (
                                           XtPointer calldata), 
                      void *CloseListData);
 SUMA_Boolean SUMA_UpdateScrolledListData(SUMA_LIST_WIDGET *LW, void *Default_Data, void *Select_Data, void *CloseList_Data); 
-void SUMA_CreateScrolledList (    char **clist, int N_clist, SUMA_Boolean Partial, 
-                                  SUMA_LIST_WIDGET *LW);
+void SUMA_CreateScrolledList (char **clist, int N_clist, SUMA_Boolean Partial, 
+                              SUMA_LIST_WIDGET *LW);
 void SUMA_cb_CloseSwitchROI(Widget w, XtPointer data, XtPointer call_data);
 void SUMA_cb_SelectSwitchROI(Widget w, XtPointer data, XtPointer call_data);
-void SUMA_FileSelection_popdown_cb (Widget w, XtPointer client_data, XtPointer call_data);
-void SUMA_FileSelection_file_select_cb(Widget dialog, XtPointer client_data, XtPointer call_data);
-SUMA_SELECTION_DIALOG_STRUCT *SUMA_CreateFileSelectionDialog (char *title, SUMA_SELECTION_DIALOG_STRUCT **dlg);
+void SUMA_FileSelection_popdown_cb (Widget w, XtPointer client_data, 
+                                    XtPointer call_data);
+void SUMA_FileSelection_file_select_cb(Widget dialog, XtPointer client_data, 
+                                    XtPointer call_data);
+SUMA_SELECTION_DIALOG_STRUCT *SUMA_CreateFileSelectionDialog (char *title, 
+                                          SUMA_SELECTION_DIALOG_STRUCT **dlg);
 SUMA_SELECTION_DIALOG_STRUCT *SUMA_CreateFileSelectionDialogStruct (
    Widget daddy, SUMA_FILE_SELECT_MODE Mode, SUMA_Boolean preserve,
    void (*SelectCallback)(char *filename, void *data), void *SelectData,
    void (*CancelCallback)(void *data), void *CancelData,
    char *FilePattern,
    SUMA_SELECTION_DIALOG_STRUCT *dlg);
-void SUMA_FileSelection_Unmap_cb (Widget w, XtPointer client_data, XtPointer call_data);
+void SUMA_FileSelection_Unmap_cb (Widget w, XtPointer client_data, 
+                                  XtPointer call_data);
 void SUMA_FreeFileSelectionDialogStruct(SUMA_SELECTION_DIALOG_STRUCT *dlg);
 SUMA_PROMPT_DIALOG_STRUCT *SUMA_CreatePromptDialogStruct (
    SUMA_PROMPT_MODE Mode, char *TextFieldLabel, 
@@ -454,7 +542,8 @@ void SUMA_PromptHelp_cb (Widget w, XtPointer data, XtPointer calldata);
 void SUMA_PromptActivate_cb (Widget w, XtPointer data, XtPointer calldata);
 void SUMA_PromptUnmap_cb (Widget w, XtPointer data, XtPointer calldata);
 void SUMA_FreePromptDialogStruct(SUMA_PROMPT_DIALOG_STRUCT *prmpt);
-void  SUMA_cb_ToggleManagementColPlaneWidget(Widget w, XtPointer data, XtPointer client_data);
+void  SUMA_cb_ToggleManagementColPlaneWidget(Widget w, XtPointer data, 
+                                                      XtPointer client_data);
 void SUMA_cb_SurfCont_SwitchPage (void *data);
 void SUMA_cb_ColPlane_NewOrder (void *data);
 int SUMA_ColPlane_NewOrder     (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp,
@@ -466,11 +555,24 @@ int SUMA_ColPlane_NewOpacity     (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp,
                                  float newopa, int cb_direct);
 int SUMA_ColPlane_NewOpacity_one (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp,
                                  float newopa, int cb_direct);
+int SUMA_Tract_NewGray (SUMA_ALL_DO *ado, 
+                           float newgray, int cb_direct );
+void SUMA_cb_Tract_NewGray(void *data);
+int SUMA_ColPlane_NewAlphaThresh_one (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp, 
+                                  float newAlphaThresh, int cb_direct);
+int SUMA_ColPlane_NewAlphaThresh (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp, 
+                              float newAlphaThresh, int cb_direct);
+void SUMA_cb_ColPlane_NewAlphaThresh (void *data);
 void SUMA_cb_ColPlane_NewDimFact (void *data);
 int SUMA_ColPlane_NewDimFact     (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp,
                                  float newdimfact, int cb_direct);
 int SUMA_ColPlane_NewDimFact_one (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp,
                                  float newdimfact, int cb_direct);
+int SUMA_ColPlane_NewNodeRadGain_one (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp, 
+                                       float newdimfact, int cb_direct);
+int SUMA_ColPlane_NewNodeRadGain (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp, 
+                              float newdimfact, int cb_direct);
+void SUMA_cb_ColPlane_NewNodeRadGain (void *data);
 
 void SUMA_cb_ColPlaneShow_toggled (Widget w, XtPointer data, 
                                    XtPointer client_data);
@@ -483,7 +585,8 @@ int SUMA_ColPlaneShowOneFore_Set_one (SUMA_ALL_DO *ado, SUMA_Boolean state,
 void SUMA_cb_ColPlane_Delete(Widget w, XtPointer data, XtPointer client_data);
 void SUMA_cb_ColPlane_Load(Widget w, XtPointer data, XtPointer client_data);
 void SUMA_cb_Dset_Load(Widget w, XtPointer data, XtPointer client_data);
-void SUMA_cb_SurfCont_SwitchColPlane(Widget w, XtPointer data, XtPointer client_data);
+void SUMA_cb_SurfCont_SwitchColPlane(Widget w, XtPointer data, 
+                                                         XtPointer client_data);
 void SUMA_cb_CloseSwitchColPlane(Widget w, XtPointer data, XtPointer call_data);
 void SUMA_cb_SelectSwitchColPlane(Widget w, XtPointer data, XtPointer call_data);
 int SUMA_SelectSwitchColPlane(SUMA_ALL_DO *ado, 
@@ -506,8 +609,14 @@ SUMA_Boolean SUMA_InitializeColPlaneShell_SO(SUMA_SurfaceObject *SO,
                                              SUMA_OVERLAYS *ColPlane);
 SUMA_Boolean SUMA_InitializeColPlaneShell_GLDO(SUMA_ALL_DO *ado,
                                              SUMA_OVERLAYS *ColPlane);
+SUMA_Boolean SUMA_InitializeColPlaneShell_TDO(SUMA_ALL_DO *ado,
+                                             SUMA_OVERLAYS *ColPlane);
+SUMA_Boolean SUMA_InitializeColPlaneShell_VO(SUMA_ALL_DO *ado,
+                                             SUMA_OVERLAYS *ColPlane);
+SUMA_Boolean SUMA_InitializeColPlaneShell_MDO (SUMA_ALL_DO *ado, 
+                                               SUMA_OVERLAYS *ColPlane);
 SUMA_Boolean SUMA_UpdateColPlaneShellAsNeeded(SUMA_ALL_DO *SO);
-SUMA_Boolean SUMA_RemixRedisplay (SUMA_ALL_DO *ado);
+SUMA_Boolean SUMA_Remixedisplay (SUMA_ALL_DO *ado);
 void SUMA_cb_SetDrawROI_SaveMode(Widget w, XtPointer data, XtPointer call_data);
 void SUMA_cb_SetDrawROI_SaveWhat(Widget w, XtPointer data, XtPointer call_data);
 void SUMA_response(Widget widget, XtPointer client_data, XtPointer call_data);
@@ -526,25 +635,38 @@ void SUMA_LoadVisualState(char *fname, void *csvp);
 int SUMA_ApplyVisualState(NI_element *nel, SUMA_SurfaceViewer *csv);
 void SUMA_SaveVisualState(char *fname, void *csvp);
 void SUMA_LoadSegDO (char *s, void *csvp);
+SUMA_Boolean SUMA_LoadVolDO (char *fname, 
+                        SUMA_DO_CoordUnits coord_type, SUMA_VolumeObject **VOp);
+int SUMA_Set_VO_Slice_Params(char *params, SUMA_VolumeObject *VO);
 void SUMA_SiSi_I_Insist(void);
 void SUMA_BuildMenuReset(int nchar);
 void SUMA_MenuArrowFieldCallback (void *CB);
 int SUMA_PageWidgetToNumber(Widget NB, Widget page);
+#define SUMA_IS_CONTPAGE_ON_TOP(SC) ( (SC) && (SC)->Page \
+                                      &&  SUMAg_CF->X->UseSameSurfCont \
+                && SUMA_isCurrentContPage(SUMAg_CF->X->SC_Notebook,(SC)->Page) )
 int SUMA_isCurrentContPage(Widget NB, Widget page);
 SUMA_Boolean SUMA_SetSurfContPageNumber(Widget NB, int i);
 int SUMA_NotebookLastPageNumber(Widget NB);
 SUMA_Boolean SUMA_Init_SurfCont_SurfParam(SUMA_ALL_DO *SO);
 SUMA_Boolean SUMA_Init_SurfCont_SurfParam_SO(SUMA_SurfaceObject *SO);
 SUMA_Boolean SUMA_Init_SurfCont_SurfParam_GLDO(SUMA_ALL_DO *ado);
+SUMA_Boolean SUMA_Init_SurfCont_SurfParam_TDO(SUMA_ALL_DO *ado);
+SUMA_Boolean SUMA_Init_SurfCont_SurfParam_VO(SUMA_ALL_DO *ado);
+SUMA_Boolean SUMA_Init_SurfCont_SurfParam_MDO(SUMA_ALL_DO *ado);
+SUMA_Boolean SUMA_Init_SurfCont_SurfParam_ADO(SUMA_ALL_DO *ado);
 int SUMA_NodeNeighborAlongScreenDirection(SUMA_SurfaceViewer *sv,
                                           SUMA_SurfaceObject *SO,
                                           int inode, double *dd);
 SUMA_Boolean SUMA_World2ScreenCoords (SUMA_SurfaceViewer *sv, int N_List, 
                               double *WorldList, double *ScreenList, int *Quad, 
-                              SUMA_Boolean ApplyXform);
+                              SUMA_Boolean ApplyXform, SUMA_Boolean ScreenY);
 SUMA_Boolean SUMA_World2ScreenCoordsF (
                      SUMA_SurfaceViewer *sv, int N_List, float *WorldList, 
-                     float *ScreenList, int *Quad, SUMA_Boolean ApplyXform);
+                     float *ScreenList, int *Quad, 
+                     SUMA_Boolean ApplyXform, SUMA_Boolean ScreenY);
+int * SUMA_DepthSort(float *NodeList, int N_Node, char **names, 
+                     int cpxform, float *scrxyz);
 SUMA_Boolean SUMA_DrawWindowLine(SUMA_SurfaceViewer *sv, int x0, int y0, 
                                                 int x1, int y1, int meth);
 void SUMA_cb_SetDrawROI_WhatDist(Widget widget, XtPointer client_data, 
@@ -567,8 +689,52 @@ void SUMA_cb_XformOpts_Apply (Widget w, XtPointer data,
 void SUMA_setIO_notify(int val);
 int SUMA_RenderMode2RenderModeMenuItem(int Mode);
 int SUMA_TransMode2TransModeMenuItem(int Mode);
+SUMA_TRANS_MODES SUMA_ATransMode2TransMode(SUMA_ATRANS_MODES ii);
+SUMA_ATRANS_MODES SUMA_TransMode2ATransMode(SUMA_TRANS_MODES ii);
+int SUMA_ATransMode2ATransModeMenuItem(int Mode);
 int SUMA_ShowMode2ShowModeMenuItem(int Mode);
 int SUMA_ShowModeStr2ShowModeMenuItem(char *str); 
+int SUMA_Font2FontMenuItem(int Mode);
+int SUMA_FontStr2FontMenuItem(char *str); 
+void * SUMA_Font2GLFont(int Mode);
+int SUMA_Through2ThroughMenuItem(int Mode);
+int SUMA_ThroughStr2ThroughMenuItem(char *str); 
+int SUMA_NodeRad2NodeRadMenuItem(int Mode);
+int SUMA_NodeRadStr2NodeRadMenuItem(char *str);
+int SUMA_NodeCol2NodeColMenuItem(int Mode);
+int SUMA_NodeColStr2NodeColMenuItem(char *str);
+float *SUMA_NodeCol2Col(int Mode, float *here);
+void SUMA_cb_SetDsetEdgeStip(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+int SUMA_SetDsetEdgeStip(SUMA_ALL_DO *ado, int imenu, int updatemenu); 
+void SUMA_cb_SetDsetAlphaVal(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+int SUMA_SetDsetAlphaVal(SUMA_ALL_DO *ado, int imenu, int updatemenu); 
+void SUMA_cb_SetTractMask(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+int SUMA_SetTractMask(SUMA_ALL_DO *ado, int imenu, int updatemenu);
+void SUMA_cb_SetDsetEdgeThick(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+int SUMA_SetDsetEdgeThick(SUMA_ALL_DO *ado, int imenu, int updatemenu);
+int SUMA_ColPlane_NewEdgeThickGain_one (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp, 
+                                       float newdimfact, int cb_direct);
+int SUMA_ColPlane_NewEdgeThickGain (SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp, 
+                              float newdimfact, int cb_direct);
+void SUMA_cb_ColPlane_NewEdgeThickGain (void *data);
+int SUMA_GDSET_ShowBundles ( SUMA_ALL_DO *ado, 
+                                SUMA_Boolean state, int cb_direct);
+void SUMA_cb_GDSET_ShowBundles_toggled (Widget w, XtPointer data, 
+                                          XtPointer client_data);
+int SUMA_GDSET_ShowUncon ( SUMA_ALL_DO *ado, 
+                                SUMA_Boolean state, int cb_direct);
+void SUMA_cb_GDSET_ShowUncon_toggled (Widget w, XtPointer data, 
+                                          XtPointer client_data);
+int SUMA_FlushPickBufferForDO(SUMA_ALL_DO *curDO);
+SUMA_TRANS_MODES SUMA_1dig_to_T(int i);
+int SUMA_T_to_1dig(SUMA_TRANS_MODES stm);
+float SUMA_1dig_to_A(int i);
+int SUMA_A_to_1dig(float v);
+SUMA_Boolean SUMA_DispExpr_To_EvalExpr(char *expr, char *evale, char *tight);
 
 
 #define SUMA_MAX_XFCB_OBJS 32       /*!< Max number of callbacks or xforms 
@@ -629,7 +795,102 @@ void SUMA_OpenXformOrtFile (char *filename, void *data);
 SUMA_Boolean SUMA_WildcardChoice(int filetype, 
                   SUMA_SurfaceObject *SO, char wild[]); 
 SUMA_Boolean SUMA_Set_Menu_Widget(SUMA_MENU_WIDGET *men, int i);
-      
+void SUMA_MaskTableCell_EV ( Widget w , XtPointer cd ,
+                XEvent *ev , Boolean *continue_to_dispatch );
+void SUMA_cb_createSurfaceCont_MDO(Widget w, XtPointer data, 
+                                     XtPointer callData);
+void SUMA_cb_SetMaskTableValue (void *data);
+void SUMA_MaskTableLabel_EV ( Widget w , XtPointer cd ,
+                      XEvent *ev , Boolean *continue_to_dispatch );
+void SUMA_MaskTableCell_EV ( Widget w , XtPointer cd ,
+                      XEvent *ev , Boolean *continue_to_dispatch );
+void SUMA_MaskEvalTableCell_EV ( Widget w , XtPointer cd ,
+                      XEvent *ev , Boolean *continue_to_dispatch );
+void SUMA_MaskEvalTableLabel_EV ( Widget w , XtPointer cd ,
+                      XEvent *ev , Boolean *continue_to_dispatch );
+int SUMA_SetMaskEvalTableValueNew(  int row, int col,
+                                char *s1, 
+                                int setmen, 
+                                int redisplay, 
+                                SUMA_NUMERICAL_UNITS num_units);
+char *SUMA_GetMaskEvalExpr(void);
+void SUMA_cb_SetMaskEvalTableValue (void *data);
+void SUMA_cb_UseMaskEval_toggled(Widget w, XtPointer data, 
+                                 XtPointer client_data);
+SUMA_Boolean SUMA_Set_UseMaskEval(int v, int redisp, int setmen);
+void SUMA_MaskLenTableCell_EV ( Widget w , XtPointer cd ,
+                      XEvent *ev , Boolean *continue_to_dispatch );
+void SUMA_MaskLenTableLabel_EV ( Widget w , XtPointer cd ,
+                      XEvent *ev , Boolean *continue_to_dispatch );
+int SUMA_SetMaskLenTableValueNew(  int row, int col,
+                                float, 
+                                int setmen, 
+                                int redisplay, 
+                                SUMA_NUMERICAL_UNITS num_units);
+void SUMA_cb_SetMaskLenTableValue (void *data);
+void SUMA_cb_UseMaskLen_toggled(Widget w, XtPointer data, 
+                                 XtPointer client_data);
+SUMA_Boolean SUMA_Set_UseMaskLen(int v, int redisp, int setmen);
+SUMA_Boolean SUMA_ModifyTable(SUMA_TABLE_FIELD *TF, int Nrows);
+void SUMA_delete_mask_timeout_CB( XtPointer client_data , XtIntervalId * id);
+void SUMA_cb_Mask_Delete(Widget wcall, XtPointer cd1, XtPointer cbs);
+SUMA_Boolean SUMA_DeleteMask(char *ado_id);
+SUMA_MaskDO * SUMA_findanyMDOp_inDOv(SUMA_DO *dov, int N_dov, int *dov_id);
+SUMA_MaskDO * SUMA_findanyMDOp(int *dov_id);
+DList *SUMA_AssembleMasksList(int withShadow);
+DList *SUMA_AssembleMasksList_inDOv(SUMA_DO *dov, int N_dov, int withShadow);
+SUMA_Boolean  SUMA_InitMasksTable(SUMA_X_SurfCont *SurfCont);
+SUMA_Boolean  SUMA_InitMasksTable_row(SUMA_X_SurfCont *SurfCont, 
+                                      SUMA_MaskDO *mdo, int row);
+int SUMA_NewSymMaskDO(SUMA_ALL_DO *ado); 
+int SUMA_ShadowMaskDO(SUMA_MaskDO **mdop);
+int SUMA_SetTractStyle(SUMA_ALL_DO *ado, int imenu, int updatemenu);
+void SUMA_cb_SetTractStyle(Widget widget, XtPointer client_data, 
+                           XtPointer call_data);
+void SUMA_CreateVrFields(  Widget parent,
+                        char *tit, char *hint, char *help, 
+                        int Nslc, SUMA_ALL_DO *ado,
+                        void (*NewValueCallback)(void * data), void *cb_data,
+                        SUMA_VR_FIELD *VrF);
+void SUMA_cb_ShowVrF_toggled(Widget w, XtPointer data, XtPointer client_data);
+void SUMA_cb_VSliceAtXYZ_toggled(Widget w, XtPointer data,XtPointer client_data);
+void SUMA_leave_NslcField( Widget w , XtPointer client_data ,
+                            XEvent * ev , Boolean * continue_to_dispatch );
+void SUMA_VrF_cb_N_slc_change (  Widget w, XtPointer client_data, 
+                                    XtPointer call_data);
+void SUMA_VrF_SetNslcString(SUMA_VR_FIELD * VrF);
+int SUMA_SetMaskTableValueNew(int row, int col,
+                              char *s1,
+                              int setmen, 
+                              int redisplay,
+                              SUMA_NUMERICAL_UNITS num_units);
+SUMA_Boolean SUMA_Set_ADO_TransMode(SUMA_ALL_DO *ado, int i);
+void SUMA_cb_Masks_Save (Widget w, XtPointer data, XtPointer client_data);
+void SUMA_cb_Masks_Load(Widget w, XtPointer data, XtPointer client_data);
+SUMA_Boolean SUMA_LoadMultiMasks_eng (char *filename, 
+                              int SetupOverlay, 
+                              int LaunchDisplay);
+void SUMA_LoadMultiMasks (char *filename, void *data);
+void SUMA_SaveMultiMasks (char *filename, void *data);
+SUMA_Boolean SUMA_SaveMultiMasks_eng (char *filename);
+
+/* 
+   *************** Convolution utilities *************** 
+   based on example in glut's convolve.c by  
+   Tom McReynolds, SGI 
+   *****************************************************
+*/
+void SUMA_C_identity(SUMA_C_FILTER *mat);
+SUMA_C_FILTER * SUMA_C_newfilter(int rows, int cols);
+void SUMA_C_free(SUMA_C_FILTER *mat);
+void SUMA_C_resize(SUMA_C_FILTER *mat, int rows, int cols);
+void SUMA_C_box(SUMA_C_FILTER *mat);
+void SUMA_C_sobel(SUMA_C_FILTER *mat);
+void SUMA_C_laplace(SUMA_C_FILTER *mat);
+void SUMA_C_convolve(SUMA_SurfaceViewer *csv, SUMA_DO *dov, SUMA_C_FILTER *mat);
+
+/* *************** End Convolution utilities *************** */
+                  
 #define SUMA_XformOrtFile_Load_help   \
    "Load an ort file"
 
@@ -761,6 +1022,11 @@ SUMA_Boolean SUMA_Set_Menu_Widget(SUMA_MENU_WIDGET *men, int i);
    "This operation is not reversible,\n"  \
    "(no Undo here) so you'll have to click twice."
    
+#define SUMA_SurfCont_DeleteMask_help   \
+   "Deletes a Mask.\n" \
+   "This operation is not reversible,\n"  \
+   "(no Undo here) so you'll have to click twice."
+   
 #define SUMA_DrawROI_SaveFormat_help   \
    "File format for saving ROI:\n"  \
    "Format options are 1D and NIML. \n"   \
@@ -901,12 +1167,73 @@ SUMA_Boolean SUMA_Set_Menu_Widget(SUMA_MENU_WIDGET *men, int i);
 #define SUMA_SurfCont_ColPlaneDim_hint \
    "Dimming factor to apply to colormap." \
 
+#define SUMA_SurfCont_NodeRadGain_hint \
+   "Gain factor to apply to node radius." \
+
+#define SUMA_SurfCont_EdgeThickGain_hint \
+   "Gain factor to apply to edge thickness." \
+
+#define SUMA_SurfCont_TractMask_hint \
+   "Gray level (0--100) of tracts outside of mask (only for Msk --> Gry)" \
+
 #define SUMA_SurfCont_ColPlaneOrder_hint \
    "Order of Dset's colorplane." \
 
+#define SUMA_SurfCont_ColPlaneAlphaThresh_hint \
+   "Threshold for voxel alpha value." \
+
 #define SUMA_SurfCont_ColPlaneOpacity_hint \
    "Opacity of Dset's colorplane." \
 
+#define SUMA_VR_help \
+   "Volume Rendering Settings.\n"
 
+#define SUMA_VR_hint \
+   "VR params (use BHelp for details)"
+
+#define SUMA_SliceSelect_axial_help \
+   "Select axial slice(s) to render.\n"\
+   "If the dataset is oblique, that would be the slice that is closest\n"\
+   "to the axial plane.\n"\
+   "Move slider bar or enter slice number directly in adjoining field.\n"\
+   "To show a stack of axial slices set the second text field to N:S\n"\
+   "where N is the number of slices in the stack and S is the spacing\n"\
+   "in number of slices between consecutive slices. The stack is centered\n"   \
+   "on the chosen slice number. So when N > 1 and even, the 'selected' slice\n" \
+   "is not rendered. In that case, set N to the next odd number to see it.\n"   \
+   "To hide/show all displayed axial slices, use right-side toggle button."
+
+#define SUMA_SliceSelect_axial_hint \
+   "Select axial slice(s) to render (use BHelp for details)"
+   
+#define SUMA_SliceSelect_sagittal_help \
+   "Select sagittal slice(s) to render.\n"\
+   "If the dataset is oblique, that would be the slice that is closest\n"\
+   "to the sagittal plane.\n"\
+   "Move slider bar or enter slice number directly in adjoining field.\n"\
+   "To show a stack of sagittal slices set the second text field to N:S\n"\
+   "where N is the number of slices in the stack and S is the spacing\n"\
+   "in number of slices between consecutive slices. The stack is centered\n"   \
+   "on the chosen slice number. So when N > 1 and even, the 'selected' slice\n" \
+   "is not rendered. In that case, set N to the next odd number to see it.\n"  \
+   "To hide/show all displayed sagittal slices, use right-side toggle button."
+
+#define SUMA_SliceSelect_sagittal_hint \
+   "Select sagittal slice(s) to render (use BHelp for details)"
+
+#define SUMA_SliceSelect_coronal_help \
+   "Select coronal slice(s) to render.\n"\
+   "If the dataset is oblique, that would be the slice that is closest\n"\
+   "to the coronal plane.\n"\
+   "Move slider bar or enter slice number directly in adjoining field.\n"\
+   "To show a stack of coronal slices set the second text field to N:S\n"\
+   "where N is the number of slices in the stack and S is the spacing\n"\
+   "in number of slices between consecutive slices. The stack is centered\n"   \
+   "on the chosen slice number. So when N > 1 and even, the 'selected' slice\n" \
+   "is not rendered. In that case, set N to the next odd number to see it.\n"   \
+   "To hide/show all displayed coronal slices, use right-side toggle button."
+
+#define SUMA_SliceSelect_coronal_hint \
+   "Select coronal slice(s) to render (use BHelp for details)"
    
 #endif
