@@ -1147,6 +1147,60 @@ char * SUMA_help_Cmap_message_Info(SUMA_COLOR_MAP * ColMap)
    SUMA_RETURN (s);
 
 }
+
+char *SUMA_help_SPEC_symbolic(void)
+{
+   static char FuncName[]={"SUMA_help_SPEC_symbolic"};
+   char stmp[1000], *s = NULL;
+   SUMA_STRING *SS = NULL;
+   
+   SUMA_ENTRY;
+   
+   SS = SUMA_StringAppend (NULL, NULL);
+
+   
+   SS = SUMA_StringAppend_va (SS, 
+"     As with option -i, you can load template\n"
+"     spec files with symbolic notation trickery as in:\n"
+"                    suma -spec MNI_N27 \n"
+"     which will load the all the surfaces from template MNI_N27\n"
+"     at the original FreeSurfer mesh density.\n"
+"  The string following -spec is formatted in the following manner:\n"
+"     HEMI:TEMPLATE:DENSITY where:\n"
+"     HEMI specifies a hemisphere. Choose from 'l', 'r', 'lh', 'rh', 'lr', or\n"
+"          'both' which is the default if you do not specify a hemisphere.\n"
+"     TEMPLATE: Specify the template name. For now, choose from MNI_N27 if\n"
+"               you want surfaces from the MNI_N27 volume, or TT_N27\n"
+"               for the Talairach version.\n"
+"               Those templates must be installed under this directory:\n"
+"                 %s\n"
+"               If you have no surface templates there, download\n"
+"                 http:afni.nimh.nih.gov:/pub/dist/tgz/suma_MNI_N27.tgz\n"
+"               and/or\n"
+"                 http:afni.nimh.nih.gov:/pub/dist/tgz/suma_TT_N27.tgz\n"
+"               and untar them under directory %s\n"
+"     DENSITY: Use if you want to load standard-mesh versions of the template\n"
+"              surfaces. Note that only ld20, ld60, ld120, and ld141 are in\n"
+"              the current distributed templates. You can create other \n"
+"              densities if you wish with MapIcosahedron, but follow the\n"
+"              same naming convention to enable SUMA to find them.\n"
+"              This parameter is optional.\n"
+"     The order in which you specify HEMI, TEMPLATE, and DENSITY, does\n"
+"     not matter.\n"
+"     For template surfaces, the -sv option is provided automatically, so you\n"
+"     can have SUMA talking to AFNI with something like:\n"
+"             suma -spec MNI_N27:ld60 &\n"
+"             afni -niml %s/suma_MNI_N27 \n"
+, THD_datadir(1), THD_datadir(1), THD_datadir(0)
+);
+   
+   SS = SUMA_StringAppend (SS, NULL);   
+   s = SS->s;
+   SUMA_free(SS); 
+   
+   SUMA_RETURN (s);
+}
+
 static char PlotCommonHelp[]={
       "        Open a graphing window for the dataset\n"
       "        currently selected. The graphing window\n"
@@ -1656,6 +1710,19 @@ char * SUMA_help_message_Info(void)
    SS = SUMA_StringAppend (SS, 
       "     F9: Labels at cross hair, toggle.\n"
       );
+   SS = SUMA_StringAppend (SS, 
+      "     F10: Toggle prying axis between surfaces' Z and Y axes.\n"
+      );
+   SS = SUMA_StringAppend (SS, 
+      "     F11: Change object rendering order.\n"
+      "          This order will affect the resultant image in\n"
+      "          the few instances where alpha transparency is\n"
+      "          used. The order can be specified for only three types of \n"
+      "          objects for now: graphs, surfaces, and volumes. \n"
+      "          If you want to render graphs first, followed by volumes then\n"
+      "          surfaces then set SUMA_ObjectDisplayOrder to something like:\n"
+      "          'graph,vol,surf', or 'GVS'"
+      );
    SS = SUMA_StringAppend (SS, 
       "     F12: Time 20 scene renderings.\n\n");
    SS = SUMA_StringAppend (SS, 
@@ -1668,8 +1735,10 @@ char * SUMA_help_message_Info(void)
    SS = SUMA_StringAppend (SS, 
       "     Mouse Controls:\n");
    SS = SUMA_StringAppend (SS, 
-      "     Button 1-Motion: rotation as if you were using\n"
-      "                      a trackball.\n"
+      "     Button 1-Motion: For 3D scenes, rotation as if you were \n"
+      "                      using a trackball. For matrix displays \n"
+      "                      Button 1-Motion causes translation because\n"
+      "                      rotations are of little use.\n"
       "       Pure vertical motion is equivalent to using \n"
       "       the up/down arrow keys.\n"
       "       Pure horizontal motion is equivalent to using \n"
@@ -1690,6 +1759,9 @@ char * SUMA_help_message_Info(void)
       "       about the Y axis passing through its center. Also, prying\n"
       "       is only enabled when the state you are viewing contains two \n"
       "       surfaces, one on the left and one on the right.\n"
+      "       in 3D views, left right mouse movement cause a rotation about\n"
+      "       the front or rear I/S axis. Up down movements cause a shift\n"
+      "       along the left/right direction.\n" 
       "     Control+Button 1-DoubleClick to get back to original setting\n"
       "       You can select nodes (Button 3) on pried surfaces and still\n"
       "       have AFNI jump to the proper location, and vice versa. However\n"
@@ -1699,18 +1771,31 @@ char * SUMA_help_message_Info(void)
       "       SUMA_LHunify = YES (see your ~/.sumarc for help)\n"
       "     Button 1-DoubleClick: Reset to Home vieweing angle, zooming is\n"
       "       left unchanged. See also 'Home' key\n"
-      "     Control+Button 1-DoubleClick: Reset surface prying.n" 
+      "     Control+Button 1-DoubleClick: Reset surface prying.\n" 
       );
    SS = SUMA_StringAppend (SS, 
-      "     Button 2-Motion: translation\n"); 
+      "     Button 2-Motion: Translation for 3D scenes. Rotation for\n"
+      "                      matrix displays.\n"); 
    SS = SUMA_StringAppend (SS, 
       "     Button 1+2-Motion    OR \n"
       "     Shift+Button2-Motion: \n"
       "          Zoom in/out\n");
    SS = SUMA_StringAppend (SS, 
-      "     Button 3-Press: Node picking \n"
+      "     Button 3-Press: Node picking whenever surfaces are present\n"
       "                     Initiates a path to new node in DrawROI mode.\n"
-      "                     No calls in Dot xform mode, or GroupInCorr\n" 
+      "                     No calls in Dot xform mode, or GroupInCorr.\n"
+      "                     Graph Edge/Matrix Cell picking whenever displaying\n"
+      "                     graph datasets only (no surfaces in sight).\n"
+      "                     Intersections with graph edges/nodes are ignored \n"
+      "                     when surfaces are displayed. \n"
+      "                     See Alt+Button 3-Press next.\n" 
+      "     Alt+Button 3-Press: Graph edge/node picking in the presence \n"
+      "                         of surfaces. Intersections with surfaces\n"
+      "                         are ignored.\n"
+      "     Shift+Alt+Button 3-Press: Same as Alt+Button 3, but also display\n"
+      "                         pick buffer. This is mostly for debugging or \n"
+      "                         for understanding why selection is behaving\n"
+      "                         strangely.\n"
       "     Shift+Button 3-Press: Same as without shift, except does not draw\n"
       "                           in DrawROI mode.\n"
       "     Ctrl+Button 3-Press: Yoke intensity selection to index of \n"
@@ -1719,17 +1804,45 @@ char * SUMA_help_message_Info(void)
       "                          many sub-bricks as the surface has nodes. \n"
       "                          K is an integer.\n"
       "     Shift+Ctrl+Button 3-Press: Pick and initiate call in Dot xform\n"
-      "                               mode, or to GroupInCorr\n");
+      "                               mode, or to GroupInCorr\n"
+      "     Button 3-DoubleClick: If double clicking on a tract mask, select\n"
+      "                           the tract mask and turn the viewer into Mask\n"
+      "                           Manipulation Mode. In this mode, the mask is\n"
+      "                           shown as a wiremesh, and selections on any \n"
+      "                           object will move the mask to that location.\n"
+      "                           New tract/masks intersections are computed\n"
+      "                           at the new location.\n"
+      "                           To leave Mask Manipulation Mode, double \n"
+      "                           click with button 3 either on the mask or \n"
+      "                           in an area void of any objects.\n" 
+      "                           If double clicking with a graph object in \n"
+      "                           focus and only connections from one node  \n"
+      "                           are shown, then revert to showing all graph\n"
+      "                           connections. Without this, you can loose all\n"
+      "                           other clickables if a certain node is not \n"
+      "                           connected to anything.\n");
    SS = SUMA_StringAppend (SS, 
-      "     Button 3-Motion: continuous picking\n"
+      "     Button 3-Motion: continuous picking whenever surface are present.\n"
       "                      No calls for dot product (InstaCorr)\n"
-      "                           or GroupInCorr, while dragging.\n" 
+      "                           or GroupInCorr, while dragging.\n"
+      "                      Continuous picking of graph edges/nodes if no\n"
+      "                      surfaces are displayed.\n"
+      "     Alt+Button 3-Motion: Continuous picking of graph edges/nodes. \n"
+      "                      Intersections with surfaces are ignored.\n" 
       "     Ctrl+Button 3-Motion: continous yoking of intensity selection to\n"
       "                           selected node*K.\n" 
       "     Shift+Ctrl+Button 3-Motion: Continuous picking and calls \n"
       "                                for dot product (InstaCorr)\n"
       "                               or GroupInCorr, while dragging.\n"
-      ); 
+      );
+   SS = SUMA_StringAppend(SS,
+      "     Wheel/Scroll: Zoom in/out\n"
+      "     Shift+Wheel/Scroll: change selected slice if current selected\n"
+      "                         object is a volume.\n"
+      "     Ctrl+Wheel/Scroll: change the size of the currently selected \n"
+      "                        tract mask. This only works when you're in\n"
+      "                        mask manipulation mode.\n"
+      );
    SS = SUMA_StringAppend (SS, 
       "    \n");
    SS = SUMA_StringAppend (SS, 
@@ -2187,7 +2300,8 @@ char * SUMA_Help_AllSurfCont ()
          "shw 0\n%s\n"
          "\n",      
          SUMA_SurfContHelp_Col, SUMA_SurfContHelp_Bias, SUMA_SurfContHelp_Cmp,
-         SUMA_SurfContHelp_CmpNew, SUMA_SurfContHelp_AbsThr, SUMA_SurfContHelp_Isym,
+         SUMA_SurfContHelp_CmpNew, SUMA_SurfContHelp_AbsThr, 
+         SUMA_SurfContHelp_Isym,
          SUMA_SurfContHelp_Shw0);
 
    SS = SUMA_StringAppend_va(SS, 
