@@ -101,6 +101,21 @@ No callback is made*/
    }  \
 }
 
+#define SUMA_CELL_ROW_COL_2_1D(m_TF, m_row, m_col) \
+   ( ((m_TF) && (m_row) >= 0 && (m_row) < (m_TF)->Ni && \
+                (m_col) >= 0 && (m_col) < (m_TF)->Nj) \
+         ? (m_row)+(m_TF)->Ni*(m_col):-1 )
+
+#define SUMA_CELL_1D_2_ROW_COL(m_TF, m_n, m_row, m_col) {\
+   (m_row) = (m_col) = -1;   \
+   if (((m_TF)) && (m_n) >= 0) {   \
+      (m_row) = (m_n) % (m_TF)->Ni; \
+      (m_col) = (m_n) / (m_TF)->Ni; \
+      if ((m_row) >= (m_TF)->Ni || (m_col) >= (m_TF)->Nj) {\
+         (m_row) = (m_col) = -1;   \
+      }\
+   }  \
+}
 
 /* scale size gets messed up, see afni_widg.c and afni.h's
 FIX_SCALE_SIZE*/
@@ -108,6 +123,11 @@ FIX_SCALE_SIZE*/
   XtVaSetValues(  SurfCont->thr_sc, XmNheight,  SUMA_CMAP_HEIGHT-40, NULL ) ; \
 }
 
+#define SUMA_FORCE_SLICE_SCALE_WIDTH(SurfCont) {\
+  /* Not needed, if you have problems make it mirror SUMA_FORCE_SCALE_HEIGHT \
+  for scales inside SurfCont->Ax_slc, Sa_slc, and Co_slc */ \
+}
+
 #define SUMA_UPDATE_ALL_NODE_GUI_FIELDS(ado) {\
       SUMA_UpdateNodeNodeField(ado); \
       /* Now get the data values at that node */   \
@@ -125,6 +145,7 @@ typedef struct {
 SUMA_Boolean SUMA_isTopColPlane(SUMA_OVERLAYS *cp, SUMA_ALL_DO *ado);
 SUMA_Boolean SUMA_isCurColPlane(SUMA_OVERLAYS *cp, SUMA_ALL_DO *ado);
 SUMA_X_SurfCont *SUMA_ADO_Cont(SUMA_ALL_DO *ado);
+SUMA_Boolean SUMA_ADO_ShowCurForeOnly(SUMA_ALL_DO *ado);
 SUMA_ALL_DO *SUMA_Cont_ADO(SUMA_X_SurfCont *SurfCont);
 SUMA_SurfaceObject *SUMA_Cont_SO(SUMA_X_SurfCont *SurfCont);
 SUMA_OVERLAYS * SUMA_ADO_CurColPlane(SUMA_ALL_DO *ado);
@@ -134,23 +155,61 @@ SUMA_Boolean SUMA_isTopColPlane(SUMA_OVERLAYS *cp, SUMA_ALL_DO *ado);
 SUMA_Boolean SUMA_isCurColPlane(SUMA_OVERLAYS *cp, SUMA_ALL_DO *ado);
 float *SUMA_ADO_DatumXYZ(SUMA_ALL_DO *ado, int isel, char *variant);
 float *SUMA_GDSET_EdgeXYZ(SUMA_DSET *dset, int isel, char *variant, float *here);
+SUMA_Boolean SUMA_GDSET_EdgeXYZ_eng(SUMA_DSET *dset, int isel, 
+                                    char *variant, float *here);
+SUMA_SurfaceObject *SUMA_GDSET_FrameSO(SUMA_DSET *dset);
+SUMA_Boolean SUMA_GDSET_GMATRIX_Aff(SUMA_DSET *dset, double Aff[4][4], int I2X);
+float *SUMA_GDSET_XYZ_Range(SUMA_DSET *dset,  char *variant, float *here);
+float *SUMA_GDSET_XYZ_Center(SUMA_DSET *dset,  char *variant, float *here);
 float *SUMA_GDSET_NodeXYZ(SUMA_DSET *dset, int node, char *variant, float *here);
+SUMA_Boolean SUMA_GDSET_NodeXYZ_eng(SUMA_DSET *dset, int node, 
+                                    char *variant, float *here);
+SUMA_Boolean SUMA_TDO_PointXYZ_eng(SUMA_TractDO *tdo, int point, 
+                                   int *BTP, float *here);
+float *SUMA_TDO_PointXYZ(SUMA_TractDO *tdo, int point, int *BTP, float *here);
+SUMA_Boolean SUMA_VO_PointXYZ_eng(SUMA_VolumeObject *vo, int point, 
+                                   int *IJK, float *here);
+float *SUMA_VO_PointXYZ(SUMA_VolumeObject *vo, int point, int *IJK, float *here);
+SUMA_Boolean SUMA_MDO_PointXYZ_eng(SUMA_MaskDO *mo, int point, 
+                                   int *IJK, float *here);
+float *SUMA_MDO_PointXYZ(SUMA_MaskDO *mdo, int point, int *BTP, float *here);
 char *SUMA_ADO_LDP(SUMA_ALL_DO *ado);
 char * SUMA_ADO_Label(SUMA_ALL_DO *ado);
+char * SUMA_ADO_CropLabel(SUMA_ALL_DO *ado, int len);
+SUMA_Boolean SUMA_ADO_isLabel(SUMA_ALL_DO *ado, char *lbl);
+char *SUMA_ADO_sLabel(SUMA_ALL_DO *ado);
 char * SUMA_ADO_idcode(SUMA_ALL_DO *ado);
 char * SUMA_ADO_Parent_idcode(SUMA_ALL_DO *ado);
 SUMA_GRAPH_SAUX *SUMA_ADO_GSaux(SUMA_ALL_DO *ado);
+SUMA_TRACT_SAUX *SUMA_ADO_TSaux(SUMA_ALL_DO *ado);
+SUMA_MASK_SAUX *SUMA_ADO_MSaux(SUMA_ALL_DO *ado);
+SUMA_SURF_SAUX *SUMA_ADO_SSaux(SUMA_ALL_DO *ado);
+SUMA_VOL_SAUX *SUMA_ADO_VSaux(SUMA_ALL_DO *ado);
+void *SUMA_ADO_Saux(SUMA_ALL_DO *ado);
+void SUMA_cb_ShowCoSlice_toggled(Widget w, XtPointer data,XtPointer client_data);
+int SUMA_SetShowSlice(SUMA_VolumeObject *vdo, char *variant, int val);
+void SUMA_cb_ShowSaSlice_toggled(Widget w, XtPointer data,XtPointer client_data);
+void SUMA_cb_ShowAxSlice_toggled(Widget w, XtPointer data,XtPointer client_data);
+SUMA_DSET *SUMA_ADO_Dset(SUMA_ALL_DO *ado);
+int SUMA_Anatomical_DOs(SUMA_DO *dov, int N_dov, int *rdov);
 int SUMA_ADO_N_Datum(SUMA_ALL_DO *ado);
+int SUMA_ADO_N_Datum_Lev(SUMA_ALL_DO *ado, SUMA_DATUM_LEVEL dtlvl);
 int SUMA_ADO_Max_Datum_Index(SUMA_ALL_DO *ado);
+int SUMA_ADO_Max_Datum_Index_Lev(SUMA_ALL_DO *ado, SUMA_DATUM_LEVEL dtlvl);
 char * SUMA_ADO_variant(SUMA_ALL_DO *ado);
-int SUMA_ADO_SelectedDatum(SUMA_ALL_DO *ado);
+int SUMA_ADO_ColPlane_SelectedDatum(SUMA_ALL_DO *ado, SUMA_OVERLAYS *Sover);
+int SUMA_ADO_SelectedDatum(SUMA_ALL_DO *ado, void *extra, void *extra2);
+int SUMA_ADO_SelectedSecondary(SUMA_ALL_DO *ado);
 SUMA_Boolean SUMA_is_ADO_Datum_Primitive(SUMA_ALL_DO *ado,
                                           SUMA_COLID_OFFSET_DATUM *codf);
-SUMA_Boolean SUMA_ADO_Set_SelectedDatum(SUMA_ALL_DO *ado, int sel);
+SUMA_Boolean SUMA_ADO_Set_SelectedDatum(SUMA_ALL_DO *ado, int sel, 
+                                        void *extra, void *extra2);
 int SUMA_ADO_N_Overlays(SUMA_ALL_DO *ado);
 SUMA_OVERLAYS * SUMA_ADO_Overlay0(SUMA_ALL_DO *ado);
 SUMA_OVERLAYS * SUMA_ADO_Overlay(SUMA_ALL_DO *ado, int i);
 SUMA_OVERLAYS * SUMA_ADO_CurColPlane(SUMA_ALL_DO *ado);
+SUMA_OVERLAYS **  SUMA_ADO_Overlays(SUMA_ALL_DO *ado, int *N_over);
+SUMA_Boolean SUMA_ADO_Append_Overlay(SUMA_ALL_DO *ado, SUMA_OVERLAYS **over);
 void SUMA_ShowMeTheChildren(Widget w);
 void SUMA_UnmanageTheChildren(Widget w);
 void SUMA_ManageTheChildren(Widget w);
@@ -242,6 +301,11 @@ void SUMA_cb_SetCoordBias(Widget widget, XtPointer client_data,
                           XtPointer call_data);
 SUMA_Boolean SUMA_RedisplayAllShowing(char *SO_idcode_str, 
                                       SUMA_SurfaceViewer *SVv, int N_SVv);
+void SUMA_CreateSliceFields(  Widget parent,
+                        char *tit, char *hint, char *help, 
+                        int Nslc, char *var, SUMA_ALL_DO *ado,
+                        void (*NewValueCallback)(void * data), void *cb_data,
+                        SUMA_SLICE_FIELD *SF);
 void SUMA_CreateTable(  Widget parent,
             int Ni, int Nj, 
             char **row_tit, char **col_tit, 
@@ -256,27 +320,33 @@ void SUMA_CreateTable(  Widget parent,
                                   XEvent *ev , Boolean *ctd), 
             void *CellEVHandlerData,
                         SUMA_TABLE_FIELD *TF);
-void SUMA_TableF_cb_label_Modify (Widget w, XtPointer client_data, XtPointer call_data);
+void SUMA_TableF_cb_label_Modify (Widget w, XtPointer client_data, 
+                                  XtPointer call_data);
 void SUMA_TableF_SetString (SUMA_TABLE_FIELD * AF);
-void SUMA_TableF_cb_label_change (Widget w, XtPointer client_data, XtPointer call_data);
+void SUMA_TableF_cb_label_change (Widget w, XtPointer client_data, 
+                                  XtPointer call_data);
 void SUMA_leave_TableField( Widget w , XtPointer client_data ,
                            XEvent * ev , Boolean * continue_to_dispatch );
-#if 0 /* Kill after a while. ZSS July 2012 */
-void SUMA_SetRangeValueOld (void *data);
-void SUMA_SetRangeValueOld_one (void *data);
-#endif
+void SUMA_SliceF_cb_mont_change (  Widget w, XtPointer client_data, 
+                                    XtPointer call_data);
+void SUMA_leave_SliceField( Widget w , XtPointer client_data ,
+                           XEvent * ev , Boolean * continue_to_dispatch );
+void SUMA_leave_MontField( Widget w , XtPointer client_data ,
+                            XEvent * ev , Boolean * continue_to_dispatch );
 int SUMA_SetRangeValueNew(SUMA_ALL_DO *ado, 
                           SUMA_OVERLAYS *colp,
                           int row, int col,
                           float v1, float v2,
                           int setmen, 
-                          int redisplay, float *reset);
+                          int redisplay, float *reset,
+                          SUMA_NUMERICAL_UNITS num_units);
 int SUMA_SetRangeValueNew_one(SUMA_ALL_DO *ado, 
                           SUMA_OVERLAYS *colp,
                           int row, int col,
                           float v1, float v2,
                           int setmen, 
-                          int redisplay, float *reset);
+                          int redisplay, float *reset,
+                          SUMA_NUMERICAL_UNITS num_units);
 void SUMA_cb_SetRangeValue (void *data);
 int SUMA_SetClustValue(SUMA_ALL_DO *ado, 
                           SUMA_OVERLAYS *colp,
@@ -301,12 +371,38 @@ SUMA_Boolean SUMA_SetTableTitleButton1(SUMA_TABLE_FIELD *TF, int i, int j,
                                        byte flag);
 SUMA_TABLE_FIELD * SUMA_AllocTableField(void);
 SUMA_TABLE_FIELD * SUMA_FreeTableField(SUMA_TABLE_FIELD *TF);
+SUMA_SLICE_FIELD * SUMA_AllocSliceField(void);
+SUMA_SLICE_FIELD * SUMA_FreeSliceField(SUMA_SLICE_FIELD *SF);
+SUMA_VR_FIELD * SUMA_AllocVRField(void);
+SUMA_VR_FIELD * SUMA_FreeVRField(SUMA_VR_FIELD *VrF);
+int SUMA_set_slice_label(SUMA_ALL_DO *ado, char *variant, float val);
+int SUMA_set_slice_scale(SUMA_ALL_DO *ado, char *variant, float val);
+void SUMA_cb_set_Ax_slice_label(Widget w, XtPointer clientData, XtPointer call);
+void SUMA_cb_set_Sa_slice_label(Widget w, XtPointer clientData, XtPointer call);
+void SUMA_cb_set_Co_slice_label(Widget w, XtPointer clientData, XtPointer call);
+void SUMA_SliceF_SetString (SUMA_SLICE_FIELD * SF);
+void SUMA_SliceF_cb_label_change (  Widget w, XtPointer client_data, 
+                                    XtPointer call_data);
+int SUMA_set_slice(SUMA_ALL_DO *ado, char *variant, float *valp, 
+                   char *caller, int redisp);
+int SUMA_set_mont(SUMA_ALL_DO *ado, char *variant, 
+                  float *val1p, float *val2p,
+                  char *caller, int redisp);
+void SUMA_cb_set_Co_slice(Widget w, XtPointer clientData, XtPointer call);
+void SUMA_cb_set_Sa_slice(Widget w, XtPointer clientData, XtPointer call);
+void SUMA_cb_set_Ax_slice(Widget w, XtPointer clientData, XtPointer call);
 SUMA_CELL_VARIETY SUMA_cellvariety (SUMA_TABLE_FIELD *TF, int n);
+int SUMA_RowTitCell(SUMA_TABLE_FIELD *TF, int r);
+int SUMA_ColTitCell(SUMA_TABLE_FIELD *TF, int c);
+int SUMA_ObjectID_Row(SUMA_TABLE_FIELD *TF, char *id);
 SUMA_Boolean SUMA_InitRangeTable(SUMA_ALL_DO *ado, int what);
 SUMA_Boolean SUMA_InitClustTable(SUMA_ALL_DO *ado);
 void SUMA_CreateXhairWidgets(Widget parent, SUMA_ALL_DO *ado);
 void SUMA_CreateXhairWidgets_SO(Widget parent, SUMA_ALL_DO *ado);
 void SUMA_CreateXhairWidgets_GLDO(Widget parent, SUMA_ALL_DO *ado);
+void SUMA_CreateXhairWidgets_TDO(Widget parent, SUMA_ALL_DO *ado);
+void SUMA_CreateXhairWidgets_VO(Widget parent, SUMA_ALL_DO *ado);
+void SUMA_CreateXhairWidgets_MDO(Widget parent, SUMA_ALL_DO *ado);
 SUMA_Boolean SUMA_UpdateXhairField(SUMA_SurfaceViewer *sv);
 SUMA_Boolean SUMA_UpdateCrossHairNodeLabelField(SUMA_SurfaceViewer *sv);
 void SUMA_XhairInput (void* data);
@@ -336,12 +432,15 @@ void SUMA_cb_SetLinkMode(Widget widget, XtPointer client_data,
                                     XtPointer call_data);
 void SUMA_set_cmap_options_GLDO(SUMA_ALL_DO *ado, SUMA_Boolean NewDset,
                                 SUMA_Boolean NewMap);
+void SUMA_set_cmap_options_VO(SUMA_ALL_DO *ado, SUMA_Boolean NewDset,
+                                SUMA_Boolean NewMap);
 void SUMA_cb_Cmap_Load(Widget w, XtPointer data, XtPointer client_data);
 SUMA_COLOR_MAP *SUMA_LoadCmapFile_eng(char *filename);
 void SUMA_LoadCmapFile (char *filename, void *data);
 SUMA_Boolean  SUMA_Insert_Cmap_of_Dset(SUMA_DSET *dset);
 void SUMA_CreateUpdatableCmapMenu(SUMA_ALL_DO *ado);
 int SUMA_ThreshVal2ScalePos(SUMA_ALL_DO *ado, float *val);
+int  SUMA_SliceVal2ScalePos (SUMA_ALL_DO *ado, char *variant, float *val);
 void SUMA_cb_SetScaleThr(void *data);
 int SUMA_SetScaleThr_one(SUMA_ALL_DO *ado, SUMA_OVERLAYS *colp,
                           float *val, int setmen, int redisplay);
@@ -355,11 +454,14 @@ void SUMA_UpdatePvalueField (SUMA_ALL_DO *ado, float thresh);
 double SUMA_Pval2ThreshVal (SUMA_ALL_DO *ado, double pval);
 SUMA_Boolean SUMA_UpdatePointField(SUMA_ALL_DO*ado);
 SUMA_Boolean SUMA_UpdateNodeField(SUMA_ALL_DO *ado);
-char *SUMA_GetLabelsAtNode(SUMA_ALL_DO *ado, int node);
-char *SUMA_GetLabelsAtNode_ADO(SUMA_ALL_DO *ado, int node);
+char *SUMA_GetLabelsAtSelection(SUMA_ALL_DO *ado, int node, int sec);
+char *SUMA_GetLabelsAtSelection_ADO(SUMA_ALL_DO *ado, int node, int sec);
 SUMA_Boolean SUMA_SetCmodeMenuChoice(SUMA_ALL_DO *ado, char *str);
 SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado, 
                                      SUMA_SurfaceViewer *sv);
+XmFontList SUMA_AppendToFontList(XmFontList fontlisti, Widget w, 
+                                 char *fontname, char *tag);
+
 /* the help strings */
 
 /* Surface Properties Block */
@@ -377,6 +479,19 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    "   Line:   Mesh rendering mode.\n"    \
    "   Points: Points rendering mode."   
 
+#define  SUMA_SurfContHelp_VTransMode  \
+   "Choose the transparency for this volume.\n" \
+   "   Viewer: Surface's transparency is set\n"  \
+   "           by the viewer's setting which can\n"   \
+   "           be changed with the 'o', 'O' options.\n"  \
+   "           Cheesecloth transparency only is allowed in\n"\
+   "           this setting.\n"\
+   "   A :   Alpha blending. May look good, but not always\n"\
+   "         accurate.\n"  \
+   "   0 :   No transparency, opaque.\n"  \
+   "   ...\n"    \
+   "   16:   Maximum transparency, invisibile"   
+
 #define  SUMA_SurfContHelp_TransMode  \
    "Choose the transparency for this surface.\n" \
    "   Viewer: Surface's transparency is set\n"  \
@@ -398,12 +513,108 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    " Contours are not created if colormap has panes\n"   \
    " of unequal sizes.\n"   
 
+#define  SUMA_SurfContHelp_DsetFont  \
+   "Choose the font size for labels of nodes.\n" \
+   "   8: 8x13.\n"  \
+   "   9: 9x15.\n"  \
+   "   TR10: Times New Roman 10.\n"    \
+   "   HE10: Helvetica 10.\n"    \
+   "   HE12: Helvetica 12.\n"    \
+   "   HE18: Helvetica 18.\n"    \
+   "   TR24: Times New Roman 24.\n"    \
+   "   XXX: Show no text.\n"  
+
+#define  SUMA_SurfContHelp_DsetNodeRad  \
+   "Choose the radius sizing for nodes of this dataset.\n" \
+   "   Const: All nodes have a radius of 1 x Gain.\n"  \
+   "   Val: Nodes size equals its dset value x Gain\n"  \
+   "   XXX: Show no balls.\n"  
+
+#define  SUMA_SurfContHelp_DsetThrough  \
+   "Choose how connections to the selected node are displayed.\n" \
+   "   Edg: Show with edges, a subset of the whole graph\n"  \
+   "   Col: Change the color of the connecting nodes, based on edge value\n"  \
+   "   Rad: Change the radius of the connecting nodes, based on edge value\n"\
+   "   CaR: Both Col and Rad\n"\
+   "   XXX: Do nothing special, keep showing whole graph.\n"  
+
+#define  SUMA_SurfContHelp_DsetEdgeThick  \
+   "Choose the thickness sizing for edges of this graph dataset.\n" \
+   "   Const: All nodes have a radius of 1 x Gain.\n"  \
+   "   Val: Nodes size equals its dset value x Gain\n"  
+
+#define  SUMA_SurfContHelp_TractStyle  \
+   "Choose the line drawing style.\n" \
+   "   Digits specify number of pixels to mask out of each 16 pixels\n"\
+   "   1 :   One pixel/16, almost solid\n"  \
+   "   ...\n"    \
+   "   15:   15/16 pixels on, almost invisible\n"   \
+   "   HDE: Hide all the tracts\n"  \
+   "   SLD: No stippling, solid line.\n"  
+
+#define  SUMA_SurfContHelp_DsetEdgeStip  \
+   "Choose the stippling for edges of this graph dataset.\n" \
+   "   1 :   One pixel/16, almost solid\n"  \
+   "   ...\n"    \
+   "   15:   15/16 pixels on, almost invisible\n"   \
+   "   Val: Set stippling based on the dset value\n"  \
+   "   XXX: No stippling, solid line.\n"  
+
+#define  SUMA_SurfContHelp_DsetAlphaVal  \
+   "Choose the method for assigning an alpha value (A) to a voxel's color.\n" \
+   "   Avg :  A = average of R, G, B values\n"  \
+   "   Max :  A = maximum of R, G, B values\n"    \
+   "   Min :  A = minimum of R, G, B values\n"    \
+   "   I :  A is based on I selection. I range parameters apply \n"   \
+   "   T :  A is based on T selection. Full range is used.\n"  \
+   "   B :  A is based on B selection. B range parameters apply\n" \
+   "   XXX: A is set to 0, nothing will show.\n"  
+
+#define  SUMA_SurfContHelp_TractMask  \
+   "Select how masked tracts are displayed\n" \
+   "   Hde:   Hide 'em masked tracts\n"  \
+   "   Gry:   Gray 'em masked tracts (gray color set by 'Gry' arrow field)\n"   \
+   "   Ign:   Ignore 'em good for nothing masks\n"
+      
+#define  SUMA_SurfContHelp_TractMaskGray  \
+   "Set the gray level for masked tracts. 0 for black, 100 for white\n" \
+   "   This arrow field only has an effect when 'Msk' menu is set to 'Gry'\n"   \
+      
+#define  SUMA_SurfContHelp_DsetNodeCol  \
+   "Choose the colorization method for nodes of this dataset.\n" \
+   "   White: Alle weiss.\n"  \
+   "   Black: Tutti nero.\n"  \
+   "   Red: Sve crveno.\n"  \
+   "   Green: Killon akhdar.\n" \
+   "   Blue: Tous bleu.\n"   \
+   "   Yellow: Todos amarillo.\n"   \
+   "   Gray50: Not there yet.\n" \
+   "   Val: Nodes color is based its dset value and the chosen colormap\n"
+
+#define  SUMA_SurfContHelp_DsetTxtShad  \
+   "Choose the variants for how labels are handled.\n" \
+   "   T : Text shown unless more than 50% occluded.\n"  \
+   "   Ts: Foreground text bright, occluded text shaded\n"  \
+   "   B : Text shown with background box unless more than 50% occluded\n"   \
+   "   Bs: Foreground text with background, occluded text shaded\n"   \
+   "   Ta: All text shown, occlusions be damned.\n"\
+   "   Ba: All text shown with background, damn the torpedoes.\n"
+
+#define  SUMA_SurfContHelp_DsetGmatBord  \
+   "Choose the partition ratio of the matrix border.\n" \
+   "   XX: No partition.\n"  \
+   "   5: Border is 1/5 of cell width.\n"  \
+   "   10: Border is 1/10 of cell width.\n"  \
+   "   20: Border is 1/20 of cell width.\n" \
+   "   30: Border is 1/30 of cell width.\n"   \
+   "   40: Border is 1/40 of cell width.\n"
+
 #define SUMA_SurfContHelp_Dsets  \
    "Show/Hide Dataset (previously Color Plane) controllers"
 
 #define SUMA_SurfContHelp_Xhr \
    "Crosshair coordinates on\n"   \
-   "this controller's surface.\n"   \
+   "this controller's selected object.\n"   \
    "Entering new coordinates \n"   \
    "makes the crosshair jump\n"   \
    "to that location (like 'ctrl+j').\n"   \
@@ -423,9 +634,31 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    "Use 'alt+l' to center the\n"   \
    "cross hair in your viewer."
 
- #define SUMA_SurfContHelp_GNode   \
-   "FILL ME"
+#define SUMA_SurfContHelp_GNode   \
+   "Index of edge in focus on this\n"  \
+   "controller's graph.\n"   \
+   "Entering a new edge's index\n"   \
+   "will put that edge in focus\n"   \
+   "and send the crosshair to its\n"   \
+   "center (like 'j').\n"   \
+   "Use 'alt+l' to center the\n"   \
+   "cross hair in your viewer."
         
+#define SUMA_SurfContHelp_I   \
+   "Index for the selection points in the entire network\n" 
+
+#define SUMA_SurfContHelp_BTP   \
+   "Triplet of indices for the selection on the displayed bundles.\n"   \
+   "The 1st index is that of the selected bundle in the network,\n"   \
+   "the second is for the selected tract in that bundle,\n"  \
+   "and the third is the index of the point selected along that tract.\n" 
+
+#define SUMA_SurfContHelp_IJK   \
+   "Triplet of indices (I) of selected voxel.\n"   \
+   "The mm RAI coordinate X = M I with M being the matrix transforming\n"  \
+   "voxel indices to voxel coordinates.\n" 
+
+
 #define SUMA_SurfContHelp_Tri   \
    "1- Triangle (faceset) index of\n"   \
    "triangle in focus on this \n"   \
@@ -488,7 +721,23 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    "on top of the stack. Separate \n"  \
    "stacks exits for foreground (fg:)\n" \
    "and background planes (bg:)."
-   
+ 
+#define SUMA_SurfContHelp_DsetAlphaThresh \
+   "Alpha threshold of Dset's rendered slices.\n"  \
+   "When datasets' voxels get colored, they get an Alpha (A) value\n"\
+   "in addition to the R, G, B values. A is computed based on\n"\
+   "the setting of the 'Avl' menu.\n"\
+   "Voxels (or more precisely, their openGL realization) \n"   \
+   "with Alpha lower than this value will not get rendered.\n"  \
+   "This is another way to 'threshold' a rendered volume, and \n" \
+   "is comparable to thresholding with the slider bar if using a\n"\
+   "monochromatic increasingly monotonic colormap with 'Avl' set to\n"\
+   "one of Max, Min, or Avg.\n"  \
+   "Note that thresholding with the slider bar sets A for thresholded \n" \
+   "voxels to 0.0 regardless of the setting for 'Avl'.\n"   \
+   "Thresholding with Ath is faster than using the slider bar because \n" \
+   "it does not require recreating the whole texture."
+  
 #define SUMA_SurfContHelp_ArrowFieldMenu \
    "For datasets with sub-bricks exceeding what you have\n" \
    "set in environment variable SUMA_ArrowFieldSelectorTrigger\n"\
@@ -521,6 +770,18 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    "colors mapped onto the surface are.\n"   \
    "For RGB Dsets (.col files), Dim is\n" \
    "applied to the RGB colors directly"
+
+#define SUMA_SurfContHelp_DsetNodeRadGain  \
+   "Gain to apply to node radius.\n" \
+   "This multiplier is always applied to whatever\n" \
+   "radius value the node gets, whether is it constant\n"   \
+   "or data derived.\n"   
+   
+#define SUMA_SurfContHelp_DsetEdgeThickGain  \
+   "Gain to apply to edge thickness.\n" \
+   "This multiplier is always applied to whatever\n" \
+   "thickness value the edge gets, whether is it constant\n"   \
+   "or data derived.\n"   
    
 #define SUMA_SurfContHelp_DsetView  \
    "View (ON)/Hide Dset node colors."
@@ -549,7 +810,15 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    "Set the threshold.\n"  \
    "For statistical parameters, you can \n"  \
    "append a 'p' to set by the p value.\n" \
-   "For example 0.001p\n"
+   "For example 0.001p.\n" \
+   "For percentile thresholding, append a '%' to\n"\
+   "the value, such as 25%\n"
+
+#define SUMA_SurfContHelp_MasksLoad  \
+   "Load a set of masks.\n"   
+
+#define SUMA_SurfContHelp_MasksSave  \
+   "Save a set of masks.\n"   
    
 #define SUMA_SurfContHelp_DsetLoad  \
    "Load a new dataset (Dset).\n"   \
@@ -727,6 +996,11 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
 #define SUMA_SurfContHelp_SelBrtTgl \
    "View (ON)/Ignore brightness modulation"
 
+#define SUMA_SurfContHelp_ShowSliceTgl \
+   "View (ON)/Hide slice"
+#define SUMA_SurfContHelp_ShowVrFTgl \
+   "View (ON)/Hide Volume Rendering"
+
 #define SUMA_SurfContHelp_SetRngTbl_r0 \
    "Used for setting the clipping ranges."   \
    "Clipping is only done for \n"   \
@@ -741,6 +1015,8 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    "colormap. \n"   \
    "Values larger than Max are mapped \n"   \
    "to the top color.\n"   \
+   "\n"\
+   "Append '%' to set values to percentiles.\n"  \
    "\n"   \
    "Left click locks ranges\n"   \
    "from automatic resetting.\n"   \
@@ -956,7 +1232,7 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
    "Same as 'ctrl+j' or\n"   \
    "an entry in the 'Node' cell\n"   \
    "under Xhair Info block."
-
+  
 #define SUMA_SurfContHelp_RangeTbl_r1  \
    "Range of values in intensity (I) column."
 
@@ -966,6 +1242,60 @@ SUMA_NIDO *SUMA_NodeLabelToTextNIDO (char *lbls, SUMA_ALL_DO *ado,
 #define SUMA_SurfContHelp_RangeTbl_r3  \
    "Range of values in brightness (B) column."
 
+#define  SUMA_SurfContHelp_MaskTypeTbl_c0 \
+   "Properties of current mask."
+           
+#define SUMA_SurfContHelp_MaskTypeTbl_c1 \
+   "Label."
+
+#define SUMA_SurfContHelp_MaskTypeTbl_c05 \
+   "Variable symbol."
+
+#define SUMA_SurfContHelp_MaskTypeTbl_c2 \
+   "Type string (box or sphere allowed)."
+
+#define SUMA_SurfContHelp_MaskTypeTbl_c3 \
+   "Coordinate in mm RAI of the center of the mask \n"   \
+   "You can enter new coordinates or right click in cell to\n"   \
+   "get back to the original center.\n"
+   
+#define SUMA_SurfContHelp_MaskTypeTbl_c4 \
+   "Size along three dimensions of mask. Enter new values or right\n"\
+   "click in cell to get back to the original center.\n"
+
+#define SUMA_SurfContHelp_MaskTypeTbl_c5  \
+   "Color of mask.\n" 
+
+#define SUMA_SurfContHelp_MaskTypeTbl_c6  \
+   "Alpha of mask color.\n" 
+
+#define SUMA_SurfContHelp_MaskTypeTbl_c7  \
+   "Transparency of mask\n" 
+
+#define SUMA_SurfContHelp_MaskTypeTbl_c8  \
+   "Dimming factor of color\n" 
+
+#define SUMA_SurfContHelp_MaskTypeTbl_r1  \
+   "Range of values in intensity (I) column."
+
+#define SUMA_SurfContHelp_EvalMaskExpr_r0 \
+   "Masking expression."
+
+#define SUMA_SurfContHelp_DistMask_r0 \
+   "Set Min Max length for tract masking"
+   
+#define SUMA_SurfContHelp_GDSET_ViewBundles \
+   "Show bundles instead of edges between nodes if \n"\
+   "the graph dataset contains such information. For\n"\
+   "the moment, only 3dProbTrackID creates such data."
+
+#define SUMA_SurfContHelp_GDSET_ViewUncon \
+   "Show graph nodes even if unconnected to other nodes.\n"
+
+#define  SUMA_SurfContHelp_Mask  \
+   "Opens controller for masks.\n" \
+   "Creates a default mask if none exists already."
+
 /* this one's based on AFNI's func->thr_pval_label help */
 #define SUMA_SurfContHelp_ThreshStats  \
    "Shows the estimated significance\n"   \
