@@ -62,7 +62,7 @@ typedef struct {
 
       Boolean read_dsets ;    /* 17 Mar 2000 */
 
-      char * layout_fname ;   /* 23 Sep 2000 */
+      char *layout_fname ;    /* 23 Sep 2000 */
 
       int enable_suma ;       /* 29 Aug 2001 */
 
@@ -74,7 +74,7 @@ typedef struct {
                /* port_niml no longer in use. ZSS June 2011 */
                /* int port_niml ;         10 Dec 2002 */
 
-      char * script_fname ;   /* 21 Jan 2003 */
+      char *script_fname ;    /* 21 Jan 2003 */
 } AF_options ;
 
 #ifdef MAIN
@@ -299,6 +299,9 @@ typedef struct {
       float func_pval ;  /* 06 Feb 2004 */
       float func_qval ;  /* 23 Jan 2008 */
 
+      int   fix_pval   , fix_qval   ;  /* 27 Feb 2014 */
+      float fixed_pval , fixed_qval ;
+
       int stats_anat_ok,     /* 29 Mar 2005: set in AFNI_range_label() */
           stats_func_ok,     /*   to indicate if the sub-brick range  */
           stats_thresh_ok ;  /*   statistics are loaded properly     */
@@ -444,7 +447,8 @@ typedef struct {
   Widget clust3d_pb, savetable_pb, index_lab, prefix_tf, done_pb ;
   Widget savemask_pb ;      /* 01 May 2008 */
   Widget whermask_pb ;      /* 04 Aug 2010 */
-
+  Widget linkrbrain_pb;     /* 21 Jan 2014 */
+  MCW_arrowval *linkrbrain_av;     /* 31 Mar 2014 */
   Widget dataset_pb ;       /* second row of controls */
   MCW_arrowval *from_av, *to_av, *aver_av ;
 
@@ -661,14 +665,17 @@ typedef struct {
       Widget frame , rowcol ;
 
       Widget thr_rowcol , thr_label , thr_scale , thr_pval_label ;
-      MCW_arrowval * thr_top_av ;
+      MCW_arrowval *thr_top_av ;
 
       Widget thr_menu ;
       MCW_bbox *thr_onoff_bbox ;
       MCW_bbox *thr_olayx_bbox ;
       Widget thr_autothresh_pb ;
+      Widget thr_setpval_pb ;      /* 03 Dec 2013 */
+      Widget thr_setqval_pb ;      /* 26 Feb 2014 */
       MCW_arrowval *thr_sign_av ;  /* 08 Aug 2007 */
       Widget thr_fdr_pb ;          /* 29 Jan 2008 */
+      Widget thr_pvalue_pb ;       /* 06 Mar 2014 */
 
       Widget inten_rowcol , inten_label ;
       MCW_pbar     *inten_pbar ;
@@ -737,9 +744,14 @@ typedef struct {
 } AFNI_function_widgets ;
 
 extern void AFNI_func_autothresh_CB(Widget,XtPointer,XtPointer) ; /* 25 Jul 2007 */
+extern void AFNI_func_setpval_CB   (Widget,XtPointer,XtPointer) ; /* 03 Dec 2013 */
+extern void AFNI_func_setqval_CB   (Widget,XtPointer,XtPointer) ; /* 03 Dec 2013 */
 extern void AFNI_func_thrsign_CB( MCW_arrowval * , XtPointer ) ;  /* 08 Aug 2007 */
 extern void AFNI_func_fdr_CB    (Widget,XtPointer,XtPointer) ;    /* 29 Jan 2008 */
 
+extern void AFNI_set_pval( struct Three_D_View * , float ) ;      /* 27 Feb 2014 */
+extern void AFNI_set_qval( struct Three_D_View * , float ) ;      /* 27 Feb 2014 */
+
 #define PBAR_MODEBUT  0
 #define PBAR_MODEPOS  (1 << PBAR_MODEBUT)
 
@@ -820,7 +832,9 @@ typedef struct {
       Widget         misc_1dchain_pb ;  /* 07 Aug 2001 */
 #endif
 
-      MCW_bbox     * thr_lock_bbox;     /* 16 Nov 2011 */
+      MCW_bbox     * thr_lock_bbox ;    /* 16 Nov 2011 */
+      MCW_bbox     * rng_lock_bbox ;    /* 04 Jun 2014 */
+      MCW_bbox     * pbar_lock_bbox;    /* 04 Jun 2014 */
       MCW_bbox     * ijk_lock_bbox ;    /* 11 Sep 2000 */
 
       Widget         misc_savelayout_pb ; /* 23 Sep 2000 */
@@ -830,6 +844,7 @@ typedef struct {
       Widget         misc_runscript_pb ;  /* 22 Jan 2003 */
 
       Widget         misc_readme_env_pb ; /* 05 Aug 2004 */
+      Widget         misc_pvalue_pb ;     /* 06 Mar 2014 */
 
       Widget         misc_motd_pb ;       /* 29 Nov 2005 */
       Widget         misc_hist_pb ;       /* 05 Mar 2008 */
@@ -880,6 +895,8 @@ typedef struct {
    Widget hidden_uscon_pb   ;  /* 30 Dec 2010 */
    Widget hidden_usdecl_pb  ;  /* 06 Jan 2011 */
    Widget hidden_melter_pb  ;  /* 18 Feb 2011 */
+   Widget hidden_pvalue_pb  ;  /* 06 Mar 2014 */
+   Widget hidden_papers_pb  ;  /* 02 May 2014 */
 
 #endif  /* USE_HIDDEN */
 
@@ -1117,14 +1134,50 @@ typedef struct Three_D_View {
       int cont_pbar_index, int_pbar_index;
       int first_integral;
       int cont_perc_thr;       /* ZSS percentile thresholding. April 26 2012 */
+
+      /* record current threshold and fim statistics as thresholded [12 Jun 2014] */
+
+      float fim_thrbot ;
+      float fim_thrtop ;
+      float fim_thresh_min ;
+      float fim_thresh_max ;
 } Three_D_View ;
 
+#define IM3D_CLEAR_THRSTAT(iq)                                                    \
+  do{ (iq)->fim_thrbot     = 666.0f; (iq)->fim_thrtop     = -666.0f;              \
+      (iq)->fim_thresh_min = 666.0f; (iq)->fim_thresh_max = -666.0f; } while(0)
+
+#define IM3D_ULAY_COHERENT(iq)                                                    \
+ (( (iq)->b123_ulay == (iq)->b123_anat || (iq)->b123_ulay == (iq)->b123_fim ) &&  \
+  ( (iq)->b231_ulay == (iq)->b231_anat || (iq)->b231_ulay == (iq)->b231_fim ) &&  \
+  ( (iq)->b312_ulay == (iq)->b312_anat || (iq)->b312_ulay == (iq)->b312_fim ))
+
 #define IM3D_CLEAR_TMASK(iq)                                                                   \
  do{ CLEAR_TMASK((iq)->b123_anat); CLEAR_TMASK((iq)->b231_anat); CLEAR_TMASK((iq)->b312_anat); \
      CLEAR_TMASK((iq)->b123_fim) ; CLEAR_TMASK((iq)->b231_fim) ; CLEAR_TMASK((iq)->b312_fim) ; \
      CLEAR_TMASK((iq)->b123_ulay); CLEAR_TMASK((iq)->b231_ulay); CLEAR_TMASK((iq)->b312_ulay); \
  } while(0)
 
+#define STATUS_IM3D_TMASK(iq)                           \
+ do{ STATUSp     ("b123_anat"      ,(iq)->b123_anat) ;  \
+     STATUS_TMASK("b123_anat tmask",(iq)->b123_anat) ;  \
+     STATUSp     ("b231_anat"      ,(iq)->b231_anat) ;  \
+     STATUS_TMASK("b231_anat tmask",(iq)->b231_anat) ;  \
+     STATUSp     ("b312_anat"      ,(iq)->b312_anat) ;  \
+     STATUS_TMASK("b312_anat tmask",(iq)->b312_anat) ;  \
+     STATUSp     ("b123_fim "      ,(iq)->b123_fim ) ;  \
+     STATUS_TMASK("b123_fim  tmask",(iq)->b123_fim ) ;  \
+     STATUSp     ("b231_fim "      ,(iq)->b231_fim ) ;  \
+     STATUS_TMASK("b231_fim  tmask",(iq)->b231_fim ) ;  \
+     STATUSp     ("b312_fim "      ,(iq)->b312_fim ) ;  \
+     STATUS_TMASK("b312_fim  tmask",(iq)->b312_fim ) ;  \
+     STATUSp     ("b123_ulay"      ,(iq)->b123_ulay) ;  \
+     STATUS_TMASK("b123_ulay tmask",(iq)->b123_ulay) ;  \
+     STATUSp     ("b231_ulay"      ,(iq)->b231_ulay) ;  \
+     STATUS_TMASK("b231_ulay tmask",(iq)->b231_ulay) ;  \
+     STATUSp     ("b312_ulay"      ,(iq)->b312_ulay) ;  \
+     STATUS_TMASK("b312_ulay tmask",(iq)->b312_ulay) ; } while(0)
+
 /*! Force re-volume-editing when this viewer is redisplayed */
 
 #define IM3D_VEDIT_FORCE(iq) (iq)->vedset.flags=1
@@ -1148,6 +1201,10 @@ typedef struct Three_D_View {
        (iq)->vwid->func->clu_tabNN3 = NULL ;                               \
      (iq)->vednomask = 0 ;                                                 \
      if( (iq)->vedset.code ) redis++ ;                                     \
+     if( redis ){                                                          \
+       (iq)->fim_thrbot     = 666.0f; (iq)->fim_thrtop     = -666.0f;      \
+       (iq)->fim_thresh_min = 666.0f; (iq)->fim_thresh_max = -666.0f;      \
+     }                                                                     \
      (iq)->vedset.flags = (iq)->vedset.code = 0; AFNI_set_thr_pval((iq));  \
      if( (iq)->vinfo->func_visible && redis ) AFNI_redisplay_func((iq)) ;  \
  } while(0) ;
@@ -1420,6 +1477,9 @@ extern "C" {
    extern PLUGIN_interface * GICOR_init(char *);         /* 22 Dec 2009 */
 #endif
 
+extern void ENV_globalrange_view( char *vname );
+extern void THD_set_image_globalrange_env(int ig);
+
 extern void GICOR_setup_func(NI_stream, NI_element *) ;        /* 22 Dec 2009 */
 extern void GICOR_process_dataset( NI_element *nel, int ct ) ; /* 23 Dec 2009 */
 extern void GICOR_process_message( NI_element *nel ) ;            /* Apr 2013 */
@@ -1503,6 +1563,12 @@ typedef struct {
    RT_status *realtime_status ;                  /* 01 Jun 2009 */
    gen_func  *realtime_callback ;
 
+   int dont_tell_suma ;            /* 1 = won't send SUMA info  */
+   int dont_overlay_suma ;  /* 1 = won't send SUMA func overlay */
+   int dont_hear_suma ;     /* 1 = I can't hear you SUMA        */
+
+   int pbar_fullrange ;                          /* 03 Jun 2014 */
+
 } AFNI_library_type ;
 
 #define BROWN_COLOR "#553319"
@@ -1523,6 +1589,21 @@ extern void AFNI_display_hist( Widget w ) ;       /* 05 Mar 2008 */
 #define FIM_THR          (0.01*GLOBAL_library.fim_bkthr_perc)  /* 02 Jun 1999 */
 #define SET_FIM_bkthr(v) (GLOBAL_library.fim_bkthr_perc = (v))
 
+#define NOT_TELLING_SUMA (GLOBAL_library.dont_tell_suma==1)
+#define     TELLING_SUMA (GLOBAL_library.dont_tell_suma==0)
+#define DONT_TELL_SUMA  (GLOBAL_library.dont_tell_suma=1)
+#define      TELL_SUMA  (GLOBAL_library.dont_tell_suma=0)
+
+#define NOT_OVERLAYING_SUMA (GLOBAL_library.dont_overlay_suma==1)
+#define     OVERLAYING_SUMA (GLOBAL_library.dont_overlay_suma==0)
+#define DONT_OVERLAY_SUMA  (GLOBAL_library.dont_overlay_suma=1)
+#define      OVERLAY_SUMA  (GLOBAL_library.dont_overlay_suma=0)
+
+#define NOT_HEARING_SUMA (GLOBAL_library.dont_hear_suma==1)
+#define     HEARING_SUMA (GLOBAL_library.dont_hear_suma==0)
+#define DONT_HEAR_SUMA  (GLOBAL_library.dont_hear_suma=1)
+#define      HEAR_SUMA  (GLOBAL_library.dont_hear_suma=0)
+
 #define DISABLE_LOCK    (GLOBAL_library.ignore_lock=1)
 #define ENABLE_LOCK     (GLOBAL_library.ignore_lock=0)
 #define BEEPIT          XBell(GLOBAL_library.dc->display,100)
@@ -1541,6 +1622,8 @@ extern void AFNI_display_hist( Widget w ) ;       /* 05 Mar 2008 */
 
 #define DOING_REALTIME_WORK (GLOBAL_library.interruptables.windows != NULL)
 
+#define PBAR_FULLRANGE  GLOBAL_library.pbar_fullrange
+
 #define UNDUMMYIZE                                                              \
  do { GLOBAL_library.have_dummy_dataset = 0 ;                                   \
       XtSetSensitive(GLOBAL_library.controllers[0]->vwid->prog->clone_pb,True); \
@@ -1608,6 +1691,14 @@ extern int AFNI_thresh_lock_env_val( void );
 extern void AFNI_func_thrlock_change_CB( Widget , XtPointer , XtPointer );
 extern void AFNI_set_all_thrlock_bboxes(Three_D_View *im3d, int bval) ;
 
+extern void AFNI_func_rnglock_change_CB( Widget , XtPointer , XtPointer );
+extern void AFNI_set_all_rnglock_bboxes(Three_D_View *im3d, int bval) ;
+extern int  AFNI_check_range_lock() ;
+
+extern void AFNI_func_pbarlock_change_CB( Widget , XtPointer , XtPointer );
+extern void AFNI_set_all_pbarlock_bboxes(Three_D_View *im3d, int bval) ;
+extern int  AFNI_check_pbar_lock() ;
+
 extern XtPointer AFNI_brick_to_mri( int n , int type , FD_brick * br );
 
 extern THD_3dim_dataset * AFNI_read_images( int nf , char * fname[] );
@@ -1639,6 +1730,9 @@ extern void AFNI_clus_popdown( Three_D_View *im3d ) ;
 extern int AFNI_clus_find_xyz( Three_D_View *im3d , float x,float y,float z ) ;
 extern void AFNI_clus_action_CB( Widget w , XtPointer cd , XtPointer cbs ) ;
 
+extern void AFNI_time_index_EV( Widget, XtPointer, XEvent *, Boolean * ) ; /* 24 Feb 2014 */
+extern void AFNI_time_index_set_fstep( Three_D_View *im3d , int istep ) ;
+
 extern void AFNI_update_dataset_viewing( THD_3dim_dataset * ); /* 21 Jul 2009 */
 extern void AFNI_alter_wami_text(Three_D_View *im3d, char *utlab);
 
@@ -1687,6 +1781,7 @@ extern void AFNI_write_many_dataset_CB( Widget , XtPointer , XtPointer ) ; /* 23
 extern void AFNI_anatmode_CB          ( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_funcmode_CB          ( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_raiseup_CB           ( Widget , XtPointer , XtPointer ) ;
+extern void AFNI_assign_ulay_bricks   ( Three_D_View *im3d ) ;             /* 10 Jun 2014 */
 
 extern void AFNI_saveas_dataset_CB   ( Widget , XtPointer , XtPointer ) ;  /* 18 Oct 2010 */
 extern void AFNI_saveas_finalize_CB  ( Widget , XtPointer , MCW_choose_cbs * ) ;
@@ -1695,6 +1790,7 @@ extern void AFNI_do_many_writes      ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_finalize_dataset_CB ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_jumpto_CB           ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern int  AFNI_jumpto_dicom        ( Three_D_View * , float, float, float  ) ;
+extern int  AFNI_jump_and_seed       ( Three_D_View * , float, float, float  ) ;
 extern int  AFNI_creepto_dicom       ( Three_D_View * , float, float, float  ) ;
 extern int  AFNI_jumpto_ijk          ( Three_D_View * , int, int, int  ) ;
 extern void AFNI_jumpto_ijk_CB       ( Widget , XtPointer , MCW_choose_cbs * ) ;
@@ -1807,11 +1903,12 @@ extern Boolean AFNI_refashion_dataset( Three_D_View * ,
 extern void AFNI_set_viewpoint( Three_D_View * , int,int,int , int ) ;
 extern void AFNI_set_index_viewpoint( Three_D_View *, int, int );
                                                             /*ZSS July 2010 */
-extern void AFNI_redisplay_func( Three_D_View * ) ; /* 05 Mar 2002 */
+extern void AFNI_redisplay_func( Three_D_View * ) ;          /* 05 Mar 2002 */
 extern void AFNI_view_setter( Three_D_View *, MCW_imseq *) ; /* 26 Feb 2003 */
 extern void AFNI_range_setter( Three_D_View *, MCW_imseq *); /* 04 Nov 2003 */
+extern void AFNI_redisplay_func_ignore( int ) ;              /* 03 Jun 2014 */
 
-extern void AFNI_coord_filer_setup( Three_D_View *im3d ) ; /* 07 May 2010 */
+extern void AFNI_coord_filer_setup( Three_D_View *im3d ) ;   /* 07 May 2010 */
 
 extern XmString AFNI_crosshair_label( Three_D_View * ) ;
 extern XmString AFNI_range_label( Three_D_View * ) ;
@@ -1829,11 +1926,19 @@ extern void AFNI_see_ttatlas_CB( Widget, XtPointer, XtPointer ) ; /* 25 Jul 2001
 extern void AFNI_range_rotate_av_CB( MCW_arrowval *, XtPointer ); /* 30 Mar 2001 */
 extern void AFNI_hintize_pbar( MCW_pbar * ,  float ) ;            /* 30 Mar 2001 */
 
-#define HINTIZE_pbar(iq)                            \
-  AFNI_hintize_pbar( (iq)->vwid->func->inten_pbar , \
-                    ((iq)->vinfo->fim_range != 0.0) \
-                     ? (iq)->vinfo->fim_range       \
-                     : (iq)->vinfo->fim_autorange )    /* 15 Aug 2001 */
+#define AFNI_pbar_topset(iq,val)           \
+ do{ MCW_choose_cbs cb ;                   \
+     cb.fval = val ;                       \
+     AFNI_set_pbar_top_CB(NULL,(iq),&cb) ; \
+ } while(0)
+
+#define FIM_RANGE(iq)              \
+ ( ((iq)->vinfo->fim_range != 0.0) \
+   ? (iq)->vinfo->fim_range        \
+   : (iq)->vinfo->fim_autorange )
+
+#define HINTIZE_pbar(iq)  \
+  AFNI_hintize_pbar( (iq)->vwid->func->inten_pbar , FIM_RANGE(iq) )
 
 extern void AFNI_reset_func_range( Three_D_View * ) ;
 
@@ -1945,6 +2050,10 @@ extern void AFNI_lock_button( Three_D_View * ) ;
 extern void AFNI_misc_button( Three_D_View * ) ;
 extern void AFNI_misc_CB    ( Widget , XtPointer , XtPointer );
 extern void AFNI_editenv_CB ( Widget , XtPointer , XtPointer );
+extern void AFNI_pvalue_CB  ( Widget , XtPointer , XtPointer );
+
+extern void AFNI_papers_CB  ( Widget , XtPointer , XtPointer );
+extern void AFNI_list_papers( Widget w ) ; /* 02 May 2014 */
 
 extern void AFNI_add_timeseries( MRI_IMAGE * ) ;
 extern void AFNI_replace_timeseries( MRI_IMAGE * ) ; /* 10 May 2009 */
