@@ -49,6 +49,87 @@
 
 
 afni_history_struct dglen_history[] = {
+{ 23 , MAY, 2014 , DRG , "whereami linkrbrain" , 
+    MINOR , TYPE_BUG_FIX, 
+    "Fixed linkrbrain coordinates and malloc/free error",
+    "Coordinates were not transformed properly from TLRC to MNI\n"
+    "space. Crashes from clusterize in afni GUI caused by mismatched\n"
+    "AFNI-friendly malloc, free in strdup function"
+} ,
+
+{ 19 , MAY, 2014 , DRG , "align_epi_anat.py" , 
+    MICRO , TYPE_BUG_FIX, 
+    "NIFTI dataset input and save_skullstrip",
+    "Fixed bug with both NIFTI input and save_skullstrip"
+} ,
+
+{ 2 , MAY, 2014 , DRG , "align_epi_anat.py" , 
+    MINOR , TYPE_GENERAL, 
+    "NIFTI dataset input",
+    "align_epi_anat.py modified to allow NIFTI input for anat,epi,\n"
+    "child_epi datasets. Output continues to be AFNI format."
+} ,
+
+{ 24 , APR, 2014 , DRG , "edge detect button in afni viewer" , 
+    MICRO , TYPE_GENERAL, 
+    "Toggle edge detection with 'e' key in viewer",
+    "Can toggle with keypress from viewer allowing plugout to drive\n"
+    "edge detection."
+} ,
+
+{ 04 , APR, 2014 , DRG , "whereami -linkrbrain" , 
+    MINOR , TYPE_NEW_OPT, 
+    "New option for getting task or gene correlation with coordinate list",
+    "Linkrbrain.org support is available in alpha form from whereami and\n"
+    "the Clusterize GUI. AFNI_LINKRBRAIN needs to be set to YES to use new\n"
+    "features."
+} ,
+
+{ 27 , MAR, 2014 , DRG , "align_epi_anat.py" , 
+    MICRO , TYPE_BUG_FIX, 
+    "h_view help added"
+} ,
+{ 17 , Feb, 2014 , DRG , "@MakeLabelTable" , 
+    MICRO , TYPE_BUG_FIX, 
+    "-centers option incorrect for some atlases",
+    "incorrect centers of mass for datasets with missing ROI values\n"
+},
+
+{ 31 , JAN, 2014 , DRG , "AFNI_IMAGE_GLOBALRANGE" , 
+    MINOR , TYPE_NEW_ENV, 
+    "New options for scaling display in afni GUI",
+    "AFNI_IMAGE_GLOBALRANGE can be set to SLICE (default), VOLUME (SUBBRICK),\n"
+    "or DSET. The GUI applies the lookup table to color the underlay with the\n"
+    "range determined from the slice, sub-brick or the whole multi-sub-brick\n"
+    "dataset. This environment variable may be set in a user's .afnirc file,\n"
+    "Additionally, the GUI allows changes from the environment plugin menu or\n"
+    "in two new places - the right-click menu on the image viewer colorbar.\n"
+    "Typing Control-m in an image viewer cycles among the global range types.\n"
+    "Previous YES/NO definitions for this variable correspond to VOLUME and\n"
+    "SLICE respectively and will continue to work as before. The lower right\n"
+    "corner of the image viewer shows the current range setting:\n"
+    "(2%-98%/Min2Max, Vol, Dset)"
+} ,
+
+{ 9 , JAN, 2014 , DRG , "3dDWItoDT" , 
+    MINOR , TYPE_BUG_FIX, 
+    "bmatrix options",
+    "bmatrix_Z and bmatrix_NZ options replace old -bmatrix option\n"
+    "The new options allow for a B=0 and no B=0 row in an optional\n"
+    "b-matrix input. The b-matrix may be used as input instead of the\n"
+    "gradient vector direction file. The former -bmatrix option\n"
+    "resulted in an error or incorrect results. (Paul Taylor is mostly\n"
+    "responsible for recognizing and fixing this. Thanks, Paul!)"
+} ,
+
+{ 21 , OCT, 2013 , DRG , "whereami GUI" , 
+    MINOR , TYPE_NEW_ENV, 
+    "NeuroSynth.org link",
+    "Link out from whereami GUI in afni to neurosynth.org\n"
+    "with new environment variable AFNI_NEUROSYNTH (YES/NO).\n"
+    "Must also set AFNI_WEBBY_WAMI to YES."
+} ,
+
 { 14 , MAY, 2013 , DRG , "align_epi_anat.py" , 
     MINOR , TYPE_BUG_FIX, 
     "motion file renamed",
@@ -76,7 +157,7 @@ afni_history_struct dglen_history[] = {
 } ,
 { 29 , MAR, 2013 , DRG , "general compression" , 
     MINOR , TYPE_NEW_ENV, 
-    "pigz for faster gzip-like compression"
+    "pigz for faster gzip-like compression",
     "pigz is a gzip utility that makes use of multiple CPU's.\n"
     "It produces and uses .gz files. To use, set AFNI_COMPRESSOR to PIGZ."
 } ,
@@ -275,7 +356,7 @@ afni_history_struct dglen_history[] = {
     "Added trivial ability to invert identity transformation"
 } ,
 
-{ 06 , SEP, 2011 , DRG , "align_epi_anat.py" , 
+{ 6 , SEP, 2011 , DRG , "align_epi_anat.py" , 
     MINOR , TYPE_BUG_FIX, 
     "save_skullstrip option saved obliqued and skullstripped data",
     "Fixed to save only skullstripped"
