@@ -29,6 +29,8 @@ PLUGIN_interface * GICOR_init(char *lab)
 
 static int ncall=0 ;
 
+static unsigned int called_before[26] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} ;
+
 /*--------------------- string to 'help' the user --------------------*/
 
 static char i_helpstring[] =
@@ -242,7 +244,8 @@ PLUGIN_interface * ICOR_init( char *lab )
      PLUTO_add_string( plint , "Method" ,
                        (un != NULL &&
                         (strstr(un,"cox")  != NULL ||
-                         strstr(un,"ziad") != NULL)  ) ? 10 : 4 ,
+                         strstr(un,"ziad") != NULL)||
+                         AFNI_yesenv("AFNI_ICORR_UBER_USER")  ) ? 10 : 4 ,
                        meth_string , 0 ) ;
    }
 
@@ -471,6 +474,7 @@ static char * ICOR_main( PLUGIN_interface *plint )
    iset->cmeth    = cmeth ;   /* 01 Mar 2010 */
    iset->prefix   = (char *)malloc(sizeof(char)*16) ;
    iset->change   = 2; /* 07 May 2012 ZSS */
+
    cpt = AFNI_controller_label(im3d); sprintf(iset->prefix,"%c_ICOR",cpt[1]);
 
    etim = PLUTO_elapsed_time() ;
@@ -537,7 +541,7 @@ ENTRY("AFNI_icor_setref_anatijk") ;
 
 int AFNI_icor_setref_xyz( Three_D_View *im3d , float xx,float yy,float zz )
 {
-   MRI_IMAGE *iim; float *iar, rng; THD_fvec3 iv,jv; THD_ivec3 kv; int ijk;
+   MRI_IMAGE *iim; float *iar, rng; THD_fvec3 iv,jv; THD_ivec3 kv; int ijk,ic ;
    THD_3dim_dataset *icoset ; THD_slist_find slf ; int nds=0 ;
    double etim ;
 
@@ -545,6 +549,8 @@ ENTRY("AFNI_icor_setref_xyz") ;
 
    if( !IM3D_OPEN(im3d) ) RETURN(-1) ;
 
+   ic = AFNI_controller_index(im3d) ; if( ic < 0 || ic > 25 ) RETURN(-1) ;
+
    /**** divert to Group InstaCorr? ****/
 
    if( im3d->giset != NULL && im3d->giset->ready ){
@@ -708,21 +714,35 @@ ENTRY("AFNI_icor_setref_xyz") ;
    }
 
    /* redisplay overlay */
+
+   if( called_before[ic] ) AFNI_ignore_pbar_top(1) ;  /* 03 Jun 2014 */
    if( im3d->fim_now != icoset ||
        im3d->iset->change ){  /* switch to this dataset */
      MCW_choose_cbs cbs ; char cmd[32] , *cpt=AFNI_controller_label(im3d) ;
      cbs.ival = nds ;
+   
      AFNI_finalize_dataset_CB( im3d->vwid->view->choose_func_pb ,
                                (XtPointer)im3d ,  &cbs           ) ;
      AFNI_set_fim_index(im3d,0) ;
      AFNI_set_thr_index(im3d,0) ;
 
-     sprintf(cmd,"SET_FUNC_RANGE %c.%.2f",cpt[1], rng) ;
-     AFNI_driver(cmd) ;
+     if( !called_before[ic] ){
+       sprintf(cmd,"SET_FUNC_RANGE %c.%.2f",cpt[1], rng) ;
+       AFNI_driver(cmd) ;
+     }
    }
-   AFNI_reset_func_range(im3d) ;
+   if( MCW_val_bbox(im3d->vwid->func->range_bbox) ){
+     char cmd[32] , *cpt=AFNI_controller_label(im3d) ;
+     AFNI_ignore_pbar_top(0) ;
+     sprintf(cmd,"SET_FUNC_RANGE %c.%f",cpt[1],im3d->vinfo->fim_autorange) ;
+     AFNI_driver(cmd) ;
+     AFNI_ignore_pbar_top(1) ;
+   }
+   AFNI_reset_func_range(im3d) ; called_before[ic]++ ;
+   AFNI_ignore_pbar_top(0) ;
 
    IM3D_CLEAR_TMASK(im3d) ;      /* Mar 2013 */
+   IM3D_CLEAR_THRSTAT(im3d) ; /* 12 Jun 2014 */
    if( MCW_val_bbox(im3d->vwid->view->see_func_bbox) == 0 ){ /* overlay is off */
      char cmd[32] , *cpt=AFNI_controller_label(im3d) ;
      sprintf(cmd,"SEE_OVERLAY %c.+",cpt[1]) ;
@@ -732,7 +752,7 @@ ENTRY("AFNI_icor_setref_xyz") ;
    }
    AFNI_set_thr_pval(im3d) ; AFNI_process_drawnotice(im3d) ;
 
-   im3d->iset->change = 0;    /* resset change flag */
+   im3d->iset->change = 0;    /* reset change flag */
 
    if( ncall <= 1 )
      ININFO_message(" InstaCorr elapsed time = %.2f sec: redisplay" ,
@@ -1035,6 +1055,7 @@ ENTRY("GICOR_setup_func") ;
    ININFO_message("----- AFNI is connnected to 3dGroupInCorr -----") ;
 
    IM3D_CLEAR_TMASK(im3d) ;      /* Mar 2013 */
+   IM3D_CLEAR_THRSTAT(im3d) ; /* 12 Jun 2014 */
    GICOR_refit_stat_menus() ;  /* 14 May 2010 */
 
    EXRETURN ;
@@ -1061,7 +1082,7 @@ void GICOR_process_dataset( NI_element *nel , int ct_start )
    Three_D_View *im3d = A_CONTROLLER , *qq3d ;
    GICOR_setup *giset = im3d->giset ;
    float *nelar , *dsdar ;
-   int nvec,nn,vv , vmul ; float thr ;
+   int nvec,nn,vv , vmul , ic ; float thr ;
 
 ENTRY("GICOR_process_dataset") ;
 
@@ -1070,6 +1091,8 @@ ENTRY("GICOR_process_dataset") ;
      EXRETURN ;
    }
 
+   ic = AFNI_controller_index(im3d) ; if( ic < 0 || ic > 25 ) EXRETURN ;
+
    if( giset->toplabel != NULL )
      PLUTO_set_toplabel( GICOR_plint , giset->toplabel ) ;
 
@@ -1173,7 +1196,9 @@ INFO_message("AFNI received %d vectors, length=%d",nel->vec_num,nvec) ;
 
    /* redisplay overlay */
 
+   if( called_before[ic] ) AFNI_ignore_pbar_top(1) ;  /* 03 Jun 2014 */
    IM3D_CLEAR_TMASK(im3d) ;      /* Mar 2013 */
+   IM3D_CLEAR_THRSTAT(im3d) ; /* 12 Jun 2014 */
    if( MCW_val_bbox(im3d->vwid->view->see_func_bbox) == 0 ){ /* overlay = off */
      char cmd[32] , *cpt=AFNI_controller_label(im3d) ;
      sprintf(cmd,"SEE_OVERLAY %c.+",cpt[1]) ;
@@ -1181,7 +1206,7 @@ INFO_message("AFNI received %d vectors, length=%d",nel->vec_num,nvec) ;
    } else {                                                  /* overlay = on */
      AFNI_redisplay_func(im3d) ;
    }
-   AFNI_set_thr_pval(im3d) ; AFNI_process_drawnotice(im3d) ;
+   AFNI_set_thr_pval(im3d) ; AFNI_process_drawnotice(im3d) ; called_before[ic]++ ;
 
    for( vv=1 ; vv < MAX_CONTROLLERS ; vv++ ){  /* other controllers need redisplay? */
      qq3d = GLOBAL_library.controllers[vv] ;
@@ -1190,6 +1215,7 @@ INFO_message("AFNI received %d vectors, length=%d",nel->vec_num,nvec) ;
        AFNI_reset_func_range(qq3d) ; AFNI_redisplay_func(qq3d) ;
      }
    }
+   AFNI_ignore_pbar_top(0) ;
 
    giset->busy = 0 ; /* Not busy waiting anymore [18 Mar 2010] */
    GRPINCORR_LABEL_ON(im3d) ;                  /* 07 Apr 2010 */
