@@ -523,7 +523,7 @@ ENTRY("load_PBAR_palette_array") ;
    }
 
    if( nn > 0 && !pbar->bigmode ){
-      Three_D_View * im3d = (Three_D_View *) pbar->parent ;
+      Three_D_View * im3d = (Three_D_View *)pbar->parent ;
       if( fixim && IM3D_OPEN(im3d) ){ HIDE_SCALE(im3d) ; }
       alter_MCW_pbar( pbar , 0 , NULL ) ;
       if( fixim && IM3D_OPEN(im3d) ){ FIX_SCALE_SIZE(im3d) ; }
@@ -563,9 +563,14 @@ ENTRY("AFNI_pbar_EV") ;
          XButtonEvent *event = (XButtonEvent *) ev ;
          im3d->vwid->butx = event->x_root ;  /* 17 May 2005 */
          im3d->vwid->buty = event->y_root ;
-         event->button    = Button3 ;                            /* fakeout */
-         XmMenuPosition( im3d->vwid->func->pbar_menu , event ) ; /* where */
-         XtManageChild ( im3d->vwid->func->pbar_menu ) ;         /* popup */
+         if( event->button == Button4 || event->button == Button5 ){ /* flip */
+           PBAR_flip(im3d->vwid->func->inten_pbar) ;          /* 19 Feb 2014 */
+           AFNI_inten_pbar_CB(im3d->vwid->func->inten_pbar,im3d,pbCR_COLOR) ;
+         } else {
+           event->button = Button3 ;                             /* fakeout */
+           XmMenuPosition( im3d->vwid->func->pbar_menu , event ) ; /* where */
+           XtManageChild ( im3d->vwid->func->pbar_menu ) ;         /* popup */
+        }
       }
       break ;
    }
@@ -616,7 +621,7 @@ ENTRY("AFNI_pbar_CB") ;
 
    else if( w == im3d->vwid->func->pbar_settop_pb ){
       MCW_choose_integer( im3d->vwid->func->options_label ,
-                          "Pbar Top" , 0 , 99999 , 1 ,
+                          "Pbar Top" , 0 , 999999 , 1 ,
                           AFNI_set_pbar_top_CB , cd   ) ;
    }
 
@@ -625,7 +630,7 @@ ENTRY("AFNI_pbar_CB") ;
    else if( w == im3d->vwid->func->pbar_flip_pb ||
             w == im3d->vwid->func->pbar_flip_pb2  ){
      PBAR_flip(pbar) ;
-     AFNI_pbar_lock_carryout(im3d) ;
+     AFNI_inten_pbar_CB(pbar,im3d,pbCR_COLOR) ;
    }
 
    /*--- Read in a palette file ---*/
@@ -835,20 +840,35 @@ ENTRY("AFNI_finalize_read_palette_CB") ;
 }
 
 /*----------------------------------------------------------------------------*/
+static int ignore_pbar_top = 0 ;
+void AFNI_ignore_pbar_top(int ig){ ignore_pbar_top = ig ; }
 
-void AFNI_set_pbar_top_CB( Widget wcaller , XtPointer cd , MCW_choose_cbs * cbs )
+/*----------------------------------------------------------------------------*/
+/* Also see macro AFNI_pbar_topset() */
+/*----------------------------------------------------------------------------*/
+
+void AFNI_set_pbar_top_CB( Widget wcaller , XtPointer cd , MCW_choose_cbs *cbs )
 {
-   Three_D_View * im3d = (Three_D_View *) cd ;
-   MCW_pbar * pbar ;
+   Three_D_View *im3d = (Three_D_View *)cd ;
+   MCW_pbar *pbar ;
    float pval[NPANE_MAX+1] ;
    float pmin,pmax , fac ;
    int ii ;
+   static int busy=0 ;  /* prevent recursion */
 
 ENTRY("AFNI_set_pbar_top_CB") ;
 
-   if( ! IM3D_OPEN(im3d) ) EXRETURN ;
+   if( ignore_pbar_top || busy || !IM3D_OPEN(im3d) ) EXRETURN ;
 
    pmax = cbs->fval ; if( pmax <= 0.0f ) EXRETURN ;
+
+   if( !IM3D_ULAY_COHERENT(im3d) ){           /* 10 Jun 2014 */
+     STATUS("incoherent ulay -- patching") ;
+     ERROR_message("AFNI_set_pbar_top_CB: incoherent ulay -- patching") ;
+     AFNI_assign_ulay_bricks(im3d) ;
+   }
+
+   busy = 1 ;
    pbar = im3d->vwid->func->inten_pbar ;
 
    HIDE_SCALE(im3d) ;
@@ -867,6 +887,7 @@ ENTRY("AFNI_set_pbar_top_CB") ;
    }
    FIX_SCALE_SIZE(im3d) ;
 
+   busy = 0 ;
    EXRETURN ;
 }
 
