@@ -5,7 +5,8 @@ function [err, ErrMessage, Info] = WriteBrik (M, Info, Opt)
 %Purpose:
 %
 %    Writes a data vector or matrix in AFNI's  format
-%    Also write data in AFNI's 1D file format
+%    Also write data in AFNI's 1D file format, 
+%    and some support for NIML format (Thanks to NNO's efforts)
 %     
 %Input Parameters:
 %   M is the brick data (in vector or matrix format)
@@ -18,6 +19,9 @@ function [err, ErrMessage, Info] = WriteBrik (M, Info, Opt)
 %      Info can be empty for 1D files but if you have one from
 %      BrikLoad you should use it.
 %
+%      To write 1D, or NIML format output, set the field 'FileFormat' 
+%      in Info to '1D' or 'NIML', respectively.
+%
 %   Opt an options structure with the following fields, [default]
 %   Most of the options are irrelevant for 1D formats. 
 %     .Scale: ([0]/1) scales values to a |maximum| of 32700. 
@@ -37,7 +41,8 @@ function [err, ErrMessage, Info] = WriteBrik (M, Info, Opt)
 %
 %               Regardless of .NoCheck, the following is done:
 %                  + The fields Info.BRICK_STATS and Info.BRICK_FLOAT_FACS
-%                    are cleared (and set if .Scale is used).
+%                    are recomputed. 
+%                    BRICK_FLOAT_FACS are set to zero if .Scale=0
 %                  + The field Info.IDCODE_STRING is cleared in WriteBrikHEAD
 %                  + The field Info.IDCODE_DATE is set in WriteBrikHEAD
 %      
@@ -55,6 +60,16 @@ function [err, ErrMessage, Info] = WriteBrik (M, Info, Opt)
 %      is set to 0 (no scaling).
 %      .OverWrite: if 'y' then overwrite existing dataset. 
 %                  Default is to not overwrite.
+%      .AdjustHeader: if 'y', then reset some header fields that conflict
+%                     with M. This option is 'y' by default. 
+%                     To find out what was done to the header, you can 
+%                     compare the returned info structure to the one
+%                     passed to the function.
+%                     With this option you can now do the following:
+%                [e,v,i] = BrikLoad('SOMETHING+orig');
+%                vout = rand(size(v,1), size(v,2), size(v,3), 20);
+%                Opt.Prefix = 'test';
+%                [e,e, io] = WriteBrik(vout, i, Opt);    
 %Output Parameters:
 %   err : 0 No Problem
 %       : 1 Mucho Problems
@@ -100,6 +115,7 @@ if (nargin < 3),
 end
 
 %first check on Prefix
+if (~isfield(Opt, 'AdjustHeader')) Opt.AdjustHeader = 'y'; end
 if (isfield(Opt,'prefix')), Opt.Prefix = Opt.prefix; end %comes from New_HEAD
 if (isfield(Opt,'scale')), Opt.Scale =  Opt.scale; end %comes from New_HEAD
 if (isfield(Opt,'overwrite')), Opt.OverWrite =  Opt.overwrite; end 
@@ -136,10 +152,17 @@ elseif (strcmp(Info.FileFormat,'1D')),
       ErrMessage = sprintf('Error %s: M is not one or two dimensional.', FuncName);  
       errordlg(ErrMessage); return;
    end
+elseif (strcmp(Info.FileFormat,'NIML')),
+   is1D = 2;
+   if (size(M,4) ~= 1 | size(M,3) ~= 1),
+      err = 1; 
+      ErrMessage = sprintf('Error %s: M is not one or two dimensional.', FuncName);  
+      errordlg(ErrMessage); return;
+   end
 end
 
 
-if (is1D),
+if (is1D == 1),
    if (isempty(Info)), [err,Info] = Info_1D(M); end
    if (OverW == 0) Opt1D.OverWrite = 'n'; % default mode
    else Opt1D.OverWrite = 'y';
@@ -173,6 +196,18 @@ if (is1D),
    Info.Extension_1D = sprintf('%s', Ext); 
    Info.RootName = sprintf('%s', Name);
    return;
+elseif (is1D == 2), %NIML
+   if (isfield(Opt,'Slices') & ~isempty(Opt.Slices)),
+      err = 1; 
+      ErrMessage = sprintf('Error %s: .Slices cannot be used for NIML files', FuncName);  
+      errordlg(ErrMessage); return;
+   end
+   [Name, Ext] = RemoveNIMLExtension(Opt.Prefix);
+   FullName = sprintf('%s.niml.dset', Name);
+   S = BrikInfo_2_niml_writesimple(Info);
+   S.data = M;
+   afni_niml_writesimple(S,FullName);
+   return;
 end
        
 %check on options
@@ -200,7 +235,19 @@ end
             fprintf(2,'\nNote %s:\n Adopting view (%s) from supplied prefix \n', FuncName, Opt.View);
          end
       end
-      if (isempty(Opt.View)), Opt.View = '+orig'; end
+      if (isempty(Opt.View)), 
+         Opt.View = '+orig'; 
+         if (isfield(Info,'SCENE_DATA') & ~isempty(Info.SCENE_DATA)),
+            switch Info.SCENE_DATA(1),
+               case 0
+                  Opt.View = '+orig';
+               case 1
+                  Opt.View = '+acpc';
+               case 2
+                  Opt.View = '+tlrc';
+            end      
+         end
+      end
    if (~isempty(findstr('orig', lower(Opt.View)))),
       Opt.View = '+orig';
    elseif (~isempty(findstr('acpc', lower(Opt.View)))),
@@ -213,7 +260,6 @@ end
    if (~isfield(Opt, 'AppendHistory') | isempty (Opt.AppendHistory)), Opt.AppendHistory = 1; end
 
 
-
 %form the flename based on the stuff in Opt.Prefix, just use the option
    Fname = sprintf('%s%s', Opt.Prefix, Opt.View);
    FnameHEAD = sprintf('%s%s.HEAD', Opt.Prefix, Opt.View);
@@ -243,6 +289,30 @@ end
       isVect = 0;
    end
 
+   if (Opt.AdjustHeader == 'y'),
+      if (isfield(Info,'BRICK_STATS')), 
+         Info = rmfield(Info,'BRICK_STATS');  end
+      if (isfield(Info,'BRICK_FLOAT_FACS')), 
+         Info = rmfield(Info,'BRICK_FLOAT_FACS');  end
+      if (isfield(Info,'DATASET_DIMENSIONS')), 
+         Info = rmfield(Info,'DATASET_DIMENSIONS');  end
+      if (isfield(Info,'DATASET_RANK')), 
+         if (Info.DATASET_RANK(2) ~= N(4)),
+            Info = rmfield(Info,'DATASET_RANK'); 
+            Info = rmfield(Info,'TAXIS_NUMS');
+            Info = rmfield(Info,'TAXIS_FLOATS');
+            Info = rmfield(Info,'TAXIS_OFFSETS');
+         end
+      end
+      if (isfield(Info,'IDCODE_STRING')), 
+         Info = rmfield(Info,'IDCODE_STRING');  end
+      if (isfield(Info,'BRICK_TYPES')),
+         if (length(Info.BRICK_TYPES) ~= N(4) & length(Info.BRICK_TYPES) >= 1),
+            Info.BRICK_TYPES = Info.BRICK_TYPES(1)*ones(1,N(4));
+         end
+      end
+   end
+   
    if (isfield(Info, 'DATASET_DIMENSIONS') & length(Info.DATASET_DIMENSIONS) < 3 & length(Info.DATASET_DIMENSIONS) > 0),
       err = 1; ErrMessage = sprintf('Error %s: If you specify DATASET_DIMENSIONS it must be a vector of three elements', FuncName); errordlg(ErrMessage); return;
    end
@@ -290,8 +360,8 @@ end
 %end
 
    %Delete the Brick_Stats, let afni take care of them
-   if (isfield(Info,'BRICK_STATS')), rmfield(Info,'BRICK_STATS');  end
-   if (isfield(Info,'BRICK_FLOAT_FACS')), rmfield(Info,'BRICK_FLOAT_FACS');  end
+   if (isfield(Info,'BRICK_STATS')), Info = rmfield(Info,'BRICK_STATS');  end
+   if (isfield(Info,'BRICK_FLOAT_FACS')), Info = rmfield(Info,'BRICK_FLOAT_FACS');  end
    
 %figure out the ouput format
 if (~isfield(Info, 'BRICK_TYPES') | isempty(Info.BRICK_TYPES)),
@@ -330,11 +400,15 @@ itype = unique(B_type);
 if (length(itype) > 1),
    err =  1; ErrMessage = sprintf('Error %s: Not set up to write sub-bricks of multiple sub-types', FuncName); errordlg(ErrMessage); return;
 end
+
+scaleval = 1;
 switch itype,
    case 0
       typestr = 'ubit8';
+      scaleval = 255;
    case 1
       typestr = 'short';
+      scaleval = 32767;
    case 2
       typestr = 'int';
    case 3
@@ -358,25 +432,34 @@ isallframes=all(ismember(allframes,Opt.Frames));
 
 Info.BRICK_FLOAT_FACS = zeros(1,Info.DATASET_RANK(2));
 
-if (Opt.Scale & isallslices & isallframes),
-      if (Opt.verbose), fprintf(1,'Scaling ...'); end
-      NperBrik = Info.DATASET_DIMENSIONS(1) .* Info.DATASET_DIMENSIONS(2) .* Info.DATASET_DIMENSIONS(3);
-      for (j=1:1:Info.DATASET_RANK(2)),
-         istrt = 1+ (j-1).*NperBrik;
-         istp = istrt + NperBrik - 1;
-         [max1, imax1] = max(abs(M(istrt:istp)));
-         Info.BRICK_STATS(2.*(j-1)+1)= min(M(istrt:istp));
-         [max2, imax2] = max(M(istrt:istp));
-         Info.BRICK_STATS(2.*j)= max2;
-         Info.BRICK_FLOAT_FACS(j) = max1 ./ 32700; % a bit lower than 32000
-         if (Info.BRICK_FLOAT_FACS(j) == 0)
-            Info.BRICK_FLOAT_FACS(j) = 1;
-         else
+%get min max and scale
+if (isallslices & isallframes),
+   if (Opt.verbose), fprintf(1,'Range computation ...'); end
+   NperBrik = Info.DATASET_DIMENSIONS(1) .* Info.DATASET_DIMENSIONS(2) .* Info.DATASET_DIMENSIONS(3);
+   for (j=1:1:Info.DATASET_RANK(2)),
+      istrt = 1+ (j-1).*NperBrik;
+      istp = istrt + NperBrik - 1;
+      [max1, imax1] = max(abs(M(istrt:istp)));
+      Info.BRICK_STATS(2.*(j-1)+1)= min(M(istrt:istp));
+      [max2, imax2] = max(M(istrt:istp));
+      Info.BRICK_STATS(2.*j)= max2;
+      Info.BRICK_FLOAT_FACS(j) = max1 ./ scaleval; 
+      if (Info.BRICK_FLOAT_FACS(j) == 0)
+         Info.BRICK_FLOAT_FACS(j) = 1;
+      else
+         if (Opt.Scale), % apply scale 
             M(istrt:istp) = M(istrt:istp) ./ Info.BRICK_FLOAT_FACS(j);
          end
       end
+   end
+end
+if (Opt.Scale & isallslices & isallframes),
+   %Nothing left to do, parameters computed above
 elseif (Opt.Scale),
    err = 1; ErrMessage = sprintf('Error %s: Cannot scale data when not writing all frames and all slices.\n', FuncName); errordlg(ErrMessage); return;
+else
+   %kill the float factor
+   Info.BRICK_FLOAT_FACS = zeros(1,Info.DATASET_RANK(2));
 end
 
 numpix=Info.DATASET_DIMENSIONS(1)*Info.DATASET_DIMENSIONS(2);
@@ -405,7 +488,9 @@ else
 end
 
 %write the file
-if (Opt.verbose), fprintf(1,'Writing %s to disk ...', FnameBRIK); end
+if (Opt.verbose), 
+   fprintf(1,'Writing %s to disk ...', FnameBRIK); 
+end
 if isallslices & isallframes
    cnt = fwrite(fid, M, typestr);
 else
