@@ -520,6 +520,12 @@ typedef struct MRI_IMARR {
 
 extern MRI_IMARR * mri_to_imarr( MRI_IMAGE *imin ) ; /* 06 Dec 2007 */
 
+/*----------------- Misc other types -------------------------------------*/
+
+typedef struct { int i,j;   } int_pair ;    /* 12 Aug 2002 */
+typedef struct { int i,j,k; } int_triple ;
+typedef struct { int i; float a; } intfloat ; /* 02 Jun 2014 */
+
 /******* macros for complex arithmetic, using comma operator *******/
 
 static float   MRI_fla ;                      /* float temporaries   */
@@ -834,6 +840,9 @@ extern double mri_maxabs( MRI_IMAGE * ) ;
 extern double_pair mri_minmax   ( MRI_IMAGE *im ) ;  /* Apr 2013 */
 extern double_pair mri_minmax_nz( MRI_IMAGE *im ) ;  /* Apr 2013 */
 
+intfloat mri_indmax_nz( MRI_IMAGE * ) ;  /* 02 Jun 2014 */
+intfloat mri_indmin_nz( MRI_IMAGE * ) ;
+
 extern MRI_IMAGE * mri_cut_2D( MRI_IMAGE * , int,int,int,int ) ;
 extern int mri_cut_many_2D(MRI_IMARR *,  int,int,int,int );
 
@@ -880,6 +889,8 @@ extern MRI_IMAGE * mri_read_ascii_ragged_complex(char *,float); /* 08 Mar 2007 *
 extern MRI_IMAGE * mri_read_ascii_ragged_fvect( char *, float, int ) ;
 extern MRI_IMARR * mri_fvect_to_imarr( MRI_IMAGE *inim ) ;
 extern MRI_IMAGE * mri_imarr_to_fvect( MRI_IMARR *imar ) ;
+extern MRI_IMAGE * mri_float_arrays_to_image(float **vecs, int vec_len, 
+                                             int vec_num); /* 16 Apr 2014 */
 extern MRI_IMAGE * mri_pair_to_fvect( MRI_IMAGE *, MRI_IMAGE * ) ;
 extern MRI_IMAGE * mri_triple_to_fvect( MRI_IMAGE *, MRI_IMAGE *, MRI_IMAGE *) ;
 extern MRI_IMAGE * mri_fvect_subimage( MRI_IMAGE *inim , int kk ) ;
@@ -986,6 +997,8 @@ extern MRI_IMAGE *mri_scale_to_float( float , MRI_IMAGE * ) ;
 extern void mri_threshold( double , double , MRI_IMAGE * , MRI_IMAGE * ) ;
 extern MRI_IMAGE * mri_mult_to_float( float * , MRI_IMAGE * ) ;
 
+extern float_pair mri_threshold_minmax( double,double, MRI_IMAGE *, MRI_IMAGE * ) ;
+
 extern void mri_maskify( MRI_IMAGE *im , byte *mask ) ; /* Jul 2010 */
 
 extern MRI_IMAGE * mri_scalize( MRI_IMAGE *, int, float * ) ; /* 20 Oct 2003 */
@@ -1314,11 +1327,6 @@ extern double poisson_p2t ( double qq , double lambda ) ;
 }
 #endif
 
-/*----------------- Misc other types -------------------------------------*/
-
-typedef struct { int i,j;   } int_pair ;    /* 12 Aug 2002 */
-typedef struct { int i,j,k; } int_triple ;
-
 /*----------*/
 
 typedef struct { int nar ; float  *ar , dx,x0 ; } floatvec ;
@@ -1913,6 +1921,8 @@ extern void GA_interp_varp1  ( MRI_IMAGE *fim , int npp,
                                float *ip, float *jp, float *kp, float *vv ) ;
 extern void GA_interp_wsinc5 ( MRI_IMAGE *fim , int npp,
                                float *ip, float *jp, float *kp, float *vv ) ;
+extern void GA_interp_wsinc5_2D( MRI_IMAGE *fim ,
+                                 int npp, float *ip, float *jp, float *vv ) ;
 extern int GA_gcd(int,int) ;
 extern int GA_find_relprime_fixed(int) ;
 extern MRI_IMAGE * GA_smooth( MRI_IMAGE *im, int meth, float rad ) ;
@@ -2142,7 +2152,10 @@ extern int mri_principal_vectors( MRI_IMARR *imar, int nvec, float *sval, float
 typedef struct {
   int    nx ,  ny ,  nz ;
   float *xd , *yd , *zd , *hv , *je , *se ;
-  mat33 emat ; int use_emat ;
+  int   use_es ;
+  float es_xd_xp, es_xd_xm, es_xd_yp, es_xd_ym, es_xd_zp, es_xd_zm,
+        es_yd_xp, es_yd_xm, es_yd_yp, es_yd_ym, es_yd_zp, es_yd_zm,
+        es_zd_xp, es_zd_xm, es_zd_yp, es_zd_ym, es_zd_zp, es_zd_zm ;
    /* stuff below here is for conversion to/from 3D dataset format */
   mat44 cmat , imat ;      /* cmat: i->x ; imat: x->i */
   char *geomstring ;
