@@ -4,6 +4,7 @@
 #include <suma_utils.h>
 #include <suma_datasets.h> 
 #include <TrackIO.h>
+#include "suma_suma.h"
 
 static int NI_tract_type = -1;
 int get_NI_tract_type(void) {
@@ -34,7 +35,7 @@ TAYLOR_BUNDLE *AppCreateBundle(TAYLOR_BUNDLE *tbu, int N_tractsbuf,
                                TAYLOR_TRACT *tracts_buff)
 {
    TAYLOR_BUNDLE *tb=NULL;
-   int nn;
+   int nn, tinb;
    TAYLOR_TRACT *tt=NULL;
    
    ENTRY("AppCreateBundle");
@@ -43,18 +44,25 @@ TAYLOR_BUNDLE *AppCreateBundle(TAYLOR_BUNDLE *tbu, int N_tractsbuf,
       tb = (TAYLOR_BUNDLE *)calloc(1,sizeof(TAYLOR_BUNDLE));
       tb->N_allocated = 0;
       tb->N_tracts = 0;
+      tb->N_points_private = -1;
+      tb->tract_P0_offset_private = NULL;
    } else {
       tb = tbu;
+      tb->N_points_private = -1; /* reset so that this will get recomputed
+                            when Bundle_N_points gets called */
    }
    while (N_tractsbuf > tb->N_allocated - tb->N_tracts) {
       tb->N_allocated += 1000;
       tb->tracts = (TAYLOR_TRACT*)realloc(tb->tracts,
                                           tb->N_allocated*sizeof(TAYLOR_TRACT));
+      tb->tract_P0_offset_private = (int *)realloc(tb->tract_P0_offset_private,
+                                          tb->N_allocated*sizeof(int));;
    }
    
    if (tracts_buff && N_tractsbuf > 0) {
       for (nn=0; nn<N_tractsbuf; ++nn) {
-         tt = tb->tracts+nn+tb->N_tracts; 
+         tinb = nn+tb->N_tracts;
+         tt = tb->tracts+tinb; 
          tt->id = tracts_buff[nn].id;
          tt->N_pts3 = tracts_buff[nn].N_pts3;
          tt->pts = (float *)calloc(tt->N_pts3, sizeof(float));
@@ -63,6 +71,9 @@ TAYLOR_BUNDLE *AppCreateBundle(TAYLOR_BUNDLE *tbu, int N_tractsbuf,
 						  nn, tt->id, TRACT_NPTS(tt), tracts_buff[nn].pts);
          }
          memcpy(tt->pts, tracts_buff[nn].pts, tt->N_pts3*sizeof(float));
+         if (tinb == 0) tb->tract_P0_offset_private[tinb] = 0;
+         else tb->tract_P0_offset_private[tinb] = 
+                  tb->tract_P0_offset_private[tinb-1]+tt->N_pts3/3;
       }
       tb->N_tracts += N_tractsbuf;
    } 
@@ -70,9 +81,72 @@ TAYLOR_BUNDLE *AppCreateBundle(TAYLOR_BUNDLE *tbu, int N_tractsbuf,
    RETURN(tb);
 }
 
+
 /*!
   grid (THD_3dim_dataset *) Without the grid structure, coordinates
   will not be in RAI, but in UHU, unholy units.
+
+  this works a bit differently than originally: tracts are not
+  delivered in half anymore but instead in full strings; length of
+  tract is not just given but the start and ending locations, A and B
+  (respectively and inclusively) in the array, because NOT masks might
+  trim away part of string for *some* nets but not all; finally,
+  tracts themselves will be included a bit differently-- instead of
+  having `test-ends' awkwardly separate, they will now be included in
+  the array even though they aren't full locations.
+
+*/
+TAYLOR_TRACT *Create_Tract_NEW(int ptA, int ptB, float **pts_buff, 
+									int id, THD_3dim_dataset *grid)
+{
+   TAYLOR_TRACT *tt=NULL;
+   int kk = 0, ii=0;
+   static int nwarn=0;
+   
+   ENTRY("Create_Tract");
+   
+   if (grid) {
+      if (ORIENT_typestr[grid->daxes->xxorient][0] != 'R' ||
+          ORIENT_typestr[grid->daxes->yyorient][0] != 'A' ||
+          ORIENT_typestr[grid->daxes->zzorient][0] != 'I' ) {
+         ERROR_message("Only expecting RAI grids");
+         RETURN(NULL);
+      }  
+   } else {
+      if (!nwarn) {
+         WARNING_message("No grid, coordinates in UHU\n"
+                         "Further messages muted\n");
+         ++nwarn;
+      }
+   }
+   tt = (TAYLOR_TRACT *)calloc(1, sizeof(TAYLOR_TRACT));
+   if (tt == NULL) {
+      ERROR_message("Failed to allocate tract");
+      RETURN(NULL);
+   }
+   tt->id = id; tt->N_pts3 = (ptB-ptA+1)*3;
+   if (!(tt->pts = (float *)calloc(tt->N_pts3, sizeof(float)))) {
+      ERROR_message("Failed to allocate pts vector");
+      Free_Tracts(tt,1); RETURN(NULL);
+   }
+   kk=0;
+   if (pts_buff) { // A and B are inclusive vals
+      for (ii=ptA; ii<=ptB; ++ii) { 
+       tt->pts[kk] = pts_buff[ii][0]+DSET_XORG(grid);++kk;
+       tt->pts[kk] = pts_buff[ii][1]+DSET_YORG(grid);++kk;
+       tt->pts[kk] = pts_buff[ii][2]+DSET_ZORG(grid);++kk;
+      }
+   }
+
+   RETURN(tt);
+}
+
+
+/*!  THIS VERSION JUST KEPT AROUND FOR 3dTrackID, UNTIL THAT IS FULLY
+     REMOVED.
+
+     grid (THD_3dim_dataset *) Without the grid structure, coordinates
+     will not be in RAI, but in UHU, unholy units.
 */
 TAYLOR_TRACT *Create_Tract(int N_ptsB, float **pts_buffB, 
 									int N_ptsF, float **pts_buffF,
@@ -145,6 +219,7 @@ TAYLOR_BUNDLE *Free_Bundle(TAYLOR_BUNDLE *tb)
    
    if (!tb) RETURN(NULL);
    tb->tracts = Free_Tracts(tb->tracts, tb->N_tracts);
+   if (tb->tract_P0_offset_private) free(tb->tract_P0_offset_private);
    free(tb);
    RETURN(NULL);
 }
@@ -231,8 +306,8 @@ void Show_Taylor_Network(TAYLOR_NETWORK *net, FILE *out,
    
    ENTRY("Show_Taylor_Network");
    if (!out) out = stderr;
-   if (!tb) {
-      fprintf(out,"NULL tb"); 
+   if (!net) {
+      fprintf(out,"NULL net"); 
       EXRETURN;
    }
    fprintf(out,"  Network has %d bundles\n", net->N_tbv);
@@ -246,7 +321,274 @@ void Show_Taylor_Network(TAYLOR_NETWORK *net, FILE *out,
    EXRETURN;
 }
 
+float Tract_Length(TAYLOR_TRACT *tt) 
+{
+   float l = -1.0, dx, dy, dz;
+   int i, N, i13, i03;
+   if (!tt) return(l);
+   N = tt->N_pts3/3;
+   l = 0.0;
+   for (i=1; i<N; ++i) {
+      i13 = 3*i; i03 = i13-3;
+      dx = tt->pts[i13  ]-tt->pts[i03  ];
+      dy = tt->pts[i13+1]-tt->pts[i03+1];
+      dz = tt->pts[i13+2]-tt->pts[i03+2];
+      l += sqrt(dx*dx+dy*dy+dz*dz);
+   }
+   return(l);
+}
 
+int Bundle_N_points(TAYLOR_BUNDLE *bun, byte recalc)
+{
+   int it, nn;
+   
+   if (!bun) return(-1);
+   if (!recalc && bun->N_points_private > 0) return(bun->N_points_private);
+   for (it=0, nn=0; it<bun->N_tracts; ++it) {
+      nn += bun->tracts[it].N_pts3;
+   }
+   nn /= 3;
+   bun->N_points_private = nn;
+   return(nn);
+}
+
+int Network_N_points(TAYLOR_NETWORK *net, byte recalc)
+{
+   int nb, ib=0, nn=-1, it;
+   
+   if (!net) return(-1);
+   if (!recalc && net->N_points_private > 0) return(net->N_points_private);
+   nn = 0;
+   for (ib=0; ib<net->N_tbv; ++ib) {
+      if (net->tbv[ib]) {
+         nb = 0;
+         for (it=0; it<net->tbv[ib]->N_tracts; ++it) {
+            nb += net->tbv[ib]->tracts[it].N_pts3;
+         }
+         net->tbv[ib]->N_points_private = nb/3;
+         nn += nb;
+      } 
+   }
+   nn /= 3;
+   net->N_points_private = nn;
+   return(nn);
+}
+
+int Network_N_tracts(TAYLOR_NETWORK *net, byte recalc)
+{
+   int nb, ib=0, nn=-1, it;
+   
+   if (!net) return(-1);
+   if (!recalc && net->N_tracts_private > 0) return(net->N_tracts_private);
+   nn = 0;
+   for (ib=0; ib<net->N_tbv; ++ib) {
+      if (net->tbv[ib]) {
+         nn += net->tbv[ib]->N_tracts;
+      } 
+   }
+   net->N_tracts_private = nn;
+   return(nn);
+}
+
+int Network_Max_tract_length(TAYLOR_NETWORK *net, byte recalc,
+                             int *t, int *b)
+{
+   int ib, it;
+   
+   if (!net) return(-1);
+   if (!recalc && net->Longest_tract_length_private > 0) {
+      if (t) *t = net->Longest_tract_index_in_bundle_private;
+      if (b) *b = net->Longest_tract_bundle_index_in_network_private;
+      return(net->Longest_tract_length_private);
+   }
+   net->Longest_tract_length_private = 0;
+   for (ib=0; ib<net->N_tbv; ++ib) {
+      for (it=0; it<net->tbv[ib]->N_tracts; ++it) {
+         if (net->tbv[ib]->tracts[it].N_pts3 > 
+             net->Longest_tract_length_private) {
+            net->Longest_tract_length_private = net->tbv[ib]->tracts[it].N_pts3;
+            net->Longest_tract_index_in_bundle_private = it;
+            net->Longest_tract_bundle_index_in_network_private = ib;
+         }
+      }
+   }
+   net->Longest_tract_length_private /= 3;
+   
+   if (t) *t = net->Longest_tract_index_in_bundle_private;
+   if (b) *b = net->Longest_tract_bundle_index_in_network_private;
+   return(net->Longest_tract_length_private);
+}
+
+int Network_N_bundles(TAYLOR_NETWORK *net)
+{
+   if (!net) return(-1);
+   return(net->N_tbv);
+}
+
+int Network_PTB_to_1P(TAYLOR_NETWORK *net, int p, int t, int b)
+{
+   int PP, it, ip, ib;
+   
+   ENTRY("Network_PTB_to_1P");
+   
+   if (!net || p<0 || t<0 || b<0) RETURN(-1);
+   
+   #if 0
+   fprintf(stderr,"P %d, T %d, B %d, \n"
+                  "N_bundlesNet=%d, N_tractsBundle=%d, N_pointsTract %d\n",
+                  p, t, b, b<net->N_tbv ? net->N_tbv:-1,
+           (b<net->N_tbv) ? net->tbv[b]->N_tracts:-1,
+           (b<net->N_tbv && t<net->tbv[b]->N_tracts) ? 
+                                          net->tbv[b]->tracts[t].N_pts3/3 : -1);
+   #endif
+   
+   if (b>=net->N_tbv) RETURN(-1);
+   
+   if (t>=net->tbv[b]->N_tracts) RETURN(-1);
+   
+   if ((3*p)>=net->tbv[b]->tracts[t].N_pts3) RETURN(-1);
+   
+   PP = 0;
+   for (ib=0; ib<b; ++ib) {
+      PP += Bundle_N_points(net->tbv[ib], 0);
+   }
+   if (!net->tbv[b]->tract_P0_offset_private) {
+      for (it=0; it<t; ++it) {
+         PP += (net->tbv[b]->tracts[it].N_pts3/3);
+      }
+   } else {
+      /* use the precomputed offsets */
+      if (t > 0) PP += net->tbv[b]->tract_P0_offset_private[t-1];
+   }
+   PP += p;
+   
+   RETURN(PP);
+}
+
+int Network_TB_to_1T(TAYLOR_NETWORK *net, int t, int b)
+{
+   int it, ib, l1=0;
+   
+   ENTRY("Network_TB_to_1T");
+   
+   if (!net || b<0 || t<0) RETURN(-1);
+   
+   #if 0
+   fprintf(stderr,"T %d, B %d, \n"
+                  "N_bundlesNet=%d, N_tractsBundle=%d, N_pointsTract %d\n",
+                  t, b, b<net->N_tbv ? net->N_tbv:-1,
+           (b<net->N_tbv) ? net->tbv[b]->N_tracts:-1,
+           (b<net->N_tbv && t<net->tbv[b]->N_tracts) ? 
+                                          net->tbv[b]->tracts[t].N_pts3/3 : -1);
+   #endif
+   
+   if (b>=net->N_tbv) RETURN(-1);
+   
+   if (t>=net->tbv[b]->N_tracts) RETURN(-1);
+      
+   l1 = 0;
+   for (ib=0; ib<b; ++ib) {
+      l1 += net->tbv[ib]->N_tracts;
+   }
+   l1 += t;
+   
+   RETURN(l1);
+}
+
+int Network_1T_to_TB(TAYLOR_NETWORK *net, int TT, int *t, int *b, 
+                     int *PP0, int *PP1)
+{
+   int ib;
+   
+   ENTRY("Network_1T_to_TB");
+   
+   if (!net || TT < 0) RETURN(-1);
+   
+   ib = 0;
+   while (ib < net->N_tbv && net->tbv[ib]->N_tracts <= TT) {
+      TT -= net->tbv[ib]->N_tracts;
+      ++ib;
+   }
+   if (ib >= net->N_tbv) RETURN(-1);
+   
+   /* We are in bundle ib, tract TT within ib */
+   if (b) *b = ib; if (t) *t = TT;
+   
+   /* what is the 1st and last point (indexed into the network) of that tract? */
+   if (PP0) {
+      #if 1
+      *PP0 = Network_PTB_to_1P(net, 0, TT, ib);    
+      #else /* No speed up, don't bother */
+      int kb;
+      for (kb = 0, *PP0 =0; kb< ib; ++kb) 
+         *PP0 += Bundle_N_points(net->tbv[ib], 0);
+      for (kb=0; kb<TT; ++kb) *PP0 += net->tbv[ib]->tracts[kb].N_pts3/3;
+      #endif
+      if (PP1) *PP1 = *PP0+(net->tbv[ib]->tracts[TT].N_pts3/3 - 1);
+   }
+   RETURN(1);
+}
+
+int Network_1B_to_1P(TAYLOR_NETWORK *net, int BB, int *PP1)
+{
+   int ib, PP0;
+   
+   ENTRY("Network_1B_to_Prange");
+   
+   if (!net || BB < 0 || BB >= net->N_tbv) RETURN(-1);
+   
+   ib = PP0 = 0;
+   while (ib < BB) {
+      PP0 += Bundle_N_points(net->tbv[ib], 0);
+      ++ib;
+   }
+   
+   if (PP1) *PP1 = PP0 + Bundle_N_points(net->tbv[BB], 0)-1;
+   
+   RETURN(PP0);
+}
+
+/* Go from point index into the whole network to 
+ (bundle in network, tract in bundle, point in tract)
+ and if l1u is not NULL, return the index into the whole
+ network of the tract containing the point index.
+ The latter index is useful for retrieving data stored
+ per tract, rather than per point (SUMA_LEV1_DAT). */
+int Network_1P_to_PTB(TAYLOR_NETWORK *net, int PP, 
+                      int *p, int *t, int *b, int *l1u)
+{
+   int ib, bnp, it, tnp, l1;
+   
+   ENTRY("Network_1P_to_PTB");
+   
+   if (!net || PP<0) RETURN(-1);
+
+   ib = l1 = 0;
+   while (ib < net->N_tbv && ((bnp = Bundle_N_points(net->tbv[ib], 0)) <= PP)) {
+      PP -= bnp;
+      l1 += net->tbv[ib]->N_tracts;
+      ++ib;
+   }
+   if (ib >= net->N_tbv) RETURN(-1);
+   
+   /* We're in bundle ib, get tract in question */
+   it = 0;
+   while (it < net->tbv[ib]->N_tracts && 
+          ((tnp = net->tbv[ib]->tracts[it].N_pts3/3) <= PP)) {
+      PP -= tnp;
+      ++it;
+   } 
+   if (it >= net->tbv[ib]->N_tracts) RETURN(-1);
+   l1 += it; /* Tract index (into whole network) */
+   
+   *p = PP;
+   *t = it;
+   *b = ib;
+   
+   if (l1u) *l1u= l1;
+   
+   RETURN(1);   
+}
 
 NI_element *Tract_2_NIel(TAYLOR_TRACT *tt)
 {
@@ -349,7 +691,7 @@ NI_group *Network_2_NIgr(TAYLOR_NETWORK *net, int mode)
    TAYLOR_BUNDLE *tb=NULL;
    int ii=0, N_All_tracts, ei, ei_alt, bb;
    
-   ENTRY("Bundle_2_NIgr");
+   ENTRY("Network_2_NIgr");
    
    if ( !net || !net->tbv || net->N_tbv < 1) RETURN(ngr);
    
@@ -400,6 +742,23 @@ NI_group *Network_2_NIgr(TAYLOR_NETWORK *net, int mode)
    RETURN(ngr);
 }
 
+NI_group *Network_link(char *filename)
+{
+   NI_group *ngr=NULL;
+   char *fext = NULL;
+   
+   ENTRY("Network_link");
+   
+   if ( !filename) RETURN(ngr);
+   fext = SUMA_Extension(filename, ".niml.tract", NOPE);
+   ngr = NI_new_group_element(); NI_rename_group(ngr,"network_link");
+   NI_set_attribute(ngr, "network_file", fext);
+   SUMA_free(fext);
+   
+   RETURN(ngr);
+}
+
+
 TAYLOR_NETWORK *NIgr_2_Network(NI_group *ngr) 
 {
    TAYLOR_NETWORK *net=NULL;
@@ -415,7 +774,10 @@ TAYLOR_NETWORK *NIgr_2_Network(NI_group *ngr)
    if (!strcmp(ngr->name,"bundle") ||/* old style */
        !strcmp(ngr->name,"network") ) { 
       net = (TAYLOR_NETWORK *)calloc(1,sizeof(TAYLOR_NETWORK));
+      net->N_points_private = -1;
+      net->N_tracts_private = -1;
       tbb = (TAYLOR_BUNDLE *)calloc(1,sizeof(TAYLOR_BUNDLE));
+      tbb->N_points_private = -1;
       for( ip=0 ; ip < ngr->part_num ; ip++ ){ 
          switch( ngr->part_typ[ip] ){
 			case NI_GROUP_TYPE:
@@ -476,6 +838,7 @@ TAYLOR_NETWORK *AppAddBundleToNetwork(TAYLOR_NETWORK *network,
    if (!network) {
       net = (TAYLOR_NETWORK *)calloc(1,sizeof(TAYLOR_NETWORK));
       net->N_allocated = -1;
+      net->N_points_private = -1;
       if (grid) {
          snprintf(net->atlas_space,64*sizeof(char),"%s", grid->atlas_space);
       } else {
@@ -483,6 +846,7 @@ TAYLOR_NETWORK *AppAddBundleToNetwork(TAYLOR_NETWORK *network,
       }      
    } else { 
       net = network;
+      net->N_points_private = -1; /* uninitialize so that it gets reset later */
    }
    
    if (net->N_allocated <= 0 || net->N_tbv >= net->N_allocated) {
@@ -519,7 +883,7 @@ int Write_NI_Network(NI_group *ngr, char *name, char *mode)
       RETURN(0);
    }
    if (!name) name = "no_name";
-   
+
    nameout = (char *)calloc(strlen(name)+35, sizeof(char));
    strcpy(nameout, "file:");
    strcat(nameout,name);
@@ -545,7 +909,7 @@ int Write_NI_Network(NI_group *ngr, char *name, char *mode)
    }
    NI_stream_close(ns); ns=NULL;
    free(nameout);
-   
+
    RETURN(1);
 }
 
@@ -674,9 +1038,8 @@ TAYLOR_NETWORK * Read_Network(char *name)
    RETURN(net);
 }
 
-
-int NI_getTractAlgOpts(NI_element *nel, float *MinFA, float *MaxAngDeg, 
-                       float *MinL, int *SeedPerV, int *M, int *bval)
+int NI_getTractAlgOpts_M(NI_element *nel, float *MinFA, float *MaxAngDeg, 
+                         float *MinL, int *SeedPerV)
 {
    char *atr=NULL;
    
@@ -684,7 +1047,7 @@ int NI_getTractAlgOpts(NI_element *nel, float *MinFA, float *MaxAngDeg,
    if (!nel) RETURN(1);
    
    if (MinFA && (atr=NI_get_attribute(nel,"Thresh_FA"))) {
-      *MinFA = (float)strtod(atr,NULL);
+      *MinFA = (float)strtod(atr,NULL); 
    }
    if (MaxAngDeg && (atr=NI_get_attribute(nel,"Thresh_ANG"))) {
       *MaxAngDeg = (float)strtod(atr,NULL);
@@ -701,18 +1064,13 @@ int NI_getTractAlgOpts(NI_element *nel, float *MinFA, float *MaxAngDeg,
    if (SeedPerV && (atr=NI_get_attribute(nel,"Nseed_Z"))) {
       SeedPerV[2] = (int)strtod(atr,NULL);
    }
-   if (M && (atr=NI_get_attribute(nel,"Ngrads"))) {
-      *M = (int)strtod(atr,NULL);
-   }
-   if (bval && (atr=NI_get_attribute(nel,"Bval"))) {
-      *bval = (int)strtod(atr,NULL);
-   }
+   
    RETURN(0);
 }
 
-NI_element * NI_setTractAlgOpts(NI_element *nel, float *MinFA, 
+NI_element * NI_setTractAlgOpts_M(NI_element *nel, float *MinFA, 
 										  float *MaxAngDeg, float *MinL, 
-										  int *SeedPerV, int *M, int *bval)
+                                  int *SeedPerV)
 {   
    ENTRY("NI_setTractAlgOpts");
    
@@ -732,22 +1090,17 @@ NI_element * NI_setTractAlgOpts(NI_element *nel, float *MinFA,
       NI_SETA_INT(nel,"Nseed_Y",SeedPerV[1]);
       NI_SETA_INT(nel,"Nseed_Z",SeedPerV[2]);
    }
-   if (M) {
-      NI_SETA_INT(nel,"Ngrads",*M);
-   }
-   if (bval) {
-      NI_SETA_INT(nel,"Bval",*bval);
-   }
    
    RETURN(nel);
 }
+      
 
-NI_element * ReadTractAlgOpts(char *fname) 
+NI_element * ReadTractAlgOpts_M(char *fname) 
 {
    NI_stream ns=NULL;
    NI_element *nel=NULL;
    float MinFA, MaxAngDeg, MinL;
-   int SeedPerV[3], M, bval;
+   int SeedPerV[3];
    char *strm=NULL;
    FILE *fin4=NULL;
    
@@ -773,13 +1126,13 @@ NI_element * ReadTractAlgOpts(char *fname)
 			fprintf(stderr, "Error opening file %s.",fname);
 			RETURN(NULL);
       }
-      fscanf(fin4, "%f %f %f %d %d %d %d %d",
+      fscanf(fin4, "%f %f %f %d %d %d",
 				 &MinFA,&MaxAngDeg,&MinL,&SeedPerV[0],&SeedPerV[1],
-				 &SeedPerV[2],&M,&bval);
+				 &SeedPerV[2]);
       fclose(fin4);
       if (!(nel = 
-            NI_setTractAlgOpts(NULL, &MinFA, &MaxAngDeg, &MinL, 
-                               SeedPerV, &M, &bval))){
+            NI_setTractAlgOpts_M(NULL, &MinFA, &MaxAngDeg, &MinL, 
+                               SeedPerV))){
          ERROR_message("Failed to get options");
          RETURN(NULL);
       }
@@ -788,6 +1141,7 @@ NI_element * ReadTractAlgOpts(char *fname)
    RETURN(nel);
 }      
 
+// THIS one doesn't need to change with new format for MULTI/HARDI update
 int WriteTractAlgOpts(char *fname, NI_element *nel) 
 {
    char *strm=NULL;
@@ -795,13 +1149,20 @@ int WriteTractAlgOpts(char *fname, NI_element *nel)
    
    ENTRY("WriteTractAlgOpts");
    
-   if (!nel || !fname) RETURN(1);
-   
-   strm = (char *)calloc(strlen(fname)+20, sizeof(char));
-   if (STRING_HAS_SUFFIX(fname,".niml.opts")) {
-      sprintf(strm,"file:%s",fname);
+   if (!nel) {
+      fprintf(stderr, "NULL nel\n");
+      RETURN(1);
+   }
+   if (fname) {
+      strm = (char *)calloc(strlen(fname)+20, sizeof(char));
+      if (STRING_HAS_SUFFIX(fname,".niml.opts")) {
+         sprintf(strm,"file:%s",fname);
+      } else {
+         sprintf(strm,"file:%s.niml.opts",fname);
+      }
    } else {
-      sprintf(strm,"file:%s.niml.opts",fname);
+      strm = (char *)calloc(20, sizeof(char));
+      sprintf(strm,"fd:1");
    }
    if (!(ns = NI_stream_open( strm , "w" ))) {
       ERROR_message("Failed to open %s\n", strm);
@@ -812,50 +1173,48 @@ int WriteTractAlgOpts(char *fname, NI_element *nel)
    RETURN(0);
 } 
 
-// **********************************
-
-int NI_getProbTractAlgOpts(NI_element *nel, float *MinFA, float *MaxAngDeg, 
+int NI_getProbTractAlgOpts_M(NI_element *nel, float *MinFA, float *MaxAngDeg, 
 									float *MinL, float *NmNsFr, int *Nseed, 
-									int *Nmonte, int *M, int *bval)
+									int *Nmonte)
 {
    char *atr=NULL;
    
    ENTRY("NI_getProbTractAlgOpts");
    if (!nel) RETURN(1);
    
-   if (MinFA && (atr=NI_get_attribute(nel,"Thresh_FA"))) {
+   if (MinFA && ( (atr=NI_get_attribute(nel,"Thresh_FA")) ||
+                  (atr=NI_get_attribute(nel,"MinFA"))) ) {
       *MinFA = (float)strtod(atr,NULL);
    }
-   if (MaxAngDeg && (atr=NI_get_attribute(nel,"Thresh_ANG"))) {
+   if (MaxAngDeg && ( (atr=NI_get_attribute(nel,"Thresh_ANG")) ||
+                      (atr=NI_get_attribute(nel,"MaxAng"))) ) {
       *MaxAngDeg = (float)strtod(atr,NULL);
    }
-   if (MinL && (atr=NI_get_attribute(nel,"Thresh_Len"))) {
+   if (MinL && ( (atr=NI_get_attribute(nel,"Thresh_Len")) ||
+                 (atr=NI_get_attribute(nel,"MinL"))) ) {
       *MinL = (float)strtod(atr,NULL);
    }
-   if (NmNsFr && (atr=NI_get_attribute(nel,"Thresh_Frac"))) {
+
+   if (NmNsFr && ( (atr=NI_get_attribute(nel,"Thresh_Frac")) ||
+                   (atr=NI_get_attribute(nel,"MinHitFr"))) ) {
       *NmNsFr = (float)strtod(atr,NULL);
    }
-   if (Nseed && (atr=NI_get_attribute(nel,"Nseed_Vox"))) {
+   if (Nseed && ( (atr=NI_get_attribute(nel,"Nseed_Vox")) ||
+                  (atr=NI_get_attribute(nel,"Nseed"))) ) {
 	   *Nseed = (int)strtod(atr,NULL);
    }
    if (Nmonte && (atr=NI_get_attribute(nel,"Nmonte"))) {
       *Nmonte = (int)strtod(atr,NULL);
    }
-   if (M && (atr=NI_get_attribute(nel,"Ngrads"))) {
-      *M = (int)strtod(atr,NULL);
-   }
-   if (bval && (atr=NI_get_attribute(nel,"Bval"))) {
-      *bval = (int)strtod(atr,NULL);
-   }
    RETURN(0);
 }
 
-NI_element * NI_setProbTractAlgOpts(NI_element *nel, float *MinFA, 
+NI_element * NI_setProbTractAlgOpts_M(NI_element *nel, float *MinFA, 
 												float *MaxAngDeg, float *MinL,
 												float *NmNsFr, int *Nseed, 
-												int *Nmonte, int *M, int *bval)
+												int *Nmonte)
 {   
-   ENTRY("NI_setProbTractAlgOpts");
+   ENTRY("NI_setProbTractAlgOpts_M");
    
    if (!nel) nel = NI_new_data_element ("PROBTRACK_opts",0);
    
@@ -877,22 +1236,16 @@ NI_element * NI_setProbTractAlgOpts(NI_element *nel, float *MinFA,
 	if (Nmonte) {
       NI_SETA_INT(nel,"Nmonte",*Nmonte);
 	}
-   if (M) {
-      NI_SETA_INT(nel,"Ngrads",*M);
-   }
-   if (bval) {
-      NI_SETA_INT(nel,"Bval",*bval);
-   }
    
    RETURN(nel);
 }
 
-NI_element * ReadProbTractAlgOpts(char *fname) 
+NI_element * ReadProbTractAlgOpts_M(char *fname) 
 {
    NI_stream ns=NULL;
    NI_element *nel=NULL;
    float MinFA, MaxAngDeg, MinL,NmNsFr;
-   int Nseed, Nmonte, M, bval;
+   int Nseed, Nmonte;
    char *strm=NULL;
    FILE *fin4=NULL;
    
@@ -918,12 +1271,15 @@ NI_element * ReadProbTractAlgOpts(char *fname)
 			fprintf(stderr, "Error opening file %s.",fname);
 			RETURN(NULL);
       }
-      fscanf(fin4, "%f %f %f %f %d %d %d %d",
-             &MinFA,&MaxAngDeg,&MinL,&NmNsFr,&Nseed,&Nmonte,&M,&bval);
+      fscanf(fin4, "%f %f %f %f %d %d",
+             &MinFA,&MaxAngDeg,&MinL,&NmNsFr,&Nseed,&Nmonte);
       fclose(fin4);
+      //printf("%f %f %f %f %d %d",
+      //     MinFA,MaxAngDeg,MinL,NmNsFr,Nseed,Nmonte); 
+
       if (!(nel = 
-            NI_setProbTractAlgOpts(NULL, &MinFA, &MaxAngDeg, &MinL, 
-											  &NmNsFr,&Nseed,&Nmonte,&M,&bval))){
+            NI_setProbTractAlgOpts_M(NULL, &MinFA, &MaxAngDeg, &MinL, 
+											  &NmNsFr,&Nseed,&Nmonte))){
          ERROR_message("Failed to get options");
          RETURN(NULL);
       }
@@ -932,15 +1288,18 @@ NI_element * ReadProbTractAlgOpts(char *fname)
    RETURN(nel);
 }      
 
-int SimpleWriteDetNetTr(FILE *file, int ***idx, THD_3dim_dataset *FA,
-                        THD_3dim_dataset *MD, THD_3dim_dataset *L1,
-                        float **loc, int **locI, int len,
-                        int *TV, int *Dim, float *Ledge)
+int SimpleWriteDetNetTr_M(int N_HAR, FILE *file, int ***idx, 
+                           THD_3dim_dataset **PARS,
+                           int PAR_BOT, int PAR_TOP,
+                           float **loc, int **locI, int len,
+                           int *TV, int *Dim, float *Ledge)
 { // for trackvis
-  int m,aa,bb;
+   int m,aa,bb;
   int READS_in;
   float READS_fl;
 
+  ENTRY("SimpleWriteDetNetTr");
+  
   // first write len of tr
   READS_in = len;
   fwrite(&READS_in,sizeof(READS_in),1,file);
@@ -955,13 +1314,142 @@ int SimpleWriteDetNetTr(FILE *file, int ***idx, THD_3dim_dataset *FA,
       fwrite(&READS_fl,sizeof(READS_fl),1,file);
     }
     bb=idx[locI[m][0]][locI[m][1]][locI[m][2]];
-    READS_fl = 1; //THD_get_voxel(FA, bb, 0); 
-    fwrite(&READS_fl,sizeof(READS_fl),1,file);
-    READS_fl = 1; //THD_get_voxel(MD, bb, 0); 
-    fwrite(&READS_fl,sizeof(READS_fl),1,file);
-    READS_fl = 1; //THD_get_voxel(L1, bb, 0); 
-    fwrite(&READS_fl,sizeof(READS_fl),1,file);
+    if(N_HAR) { // hardi, single param
+       READS_fl = THD_get_voxel(PARS[PAR_BOT], bb, 0); 
+       fwrite(&READS_fl,sizeof(READS_fl),1,file);
+    }
+    else 
+       for( aa=1 ; aa<4 ; aa++ ) {
+          READS_fl = THD_get_voxel(PARS[aa], bb, 0); 
+          fwrite(&READS_fl,sizeof(READS_fl),1,file);
+       }
   }
-
+  
   RETURN(1);
 }
+
+int Free_Insta_Tract_Setup(INSTA_TRACT_SETUP *ITS)
+{
+   ENTRY("Free_Insta_Tract_Setup");
+
+   if (!ITS) RETURN(0);
+   
+   if (ITS->grid) DSET_delete(ITS->grid);
+   ITS->grid = NULL;
+   
+   /* Do not delte ITS , leave it to calling function */
+  
+   RETURN(1);
+}
+
+/* Create brandnew, or wipe clean existing ITS */
+INSTA_TRACT_SETUP *New_Insta_Tract_Setup(INSTA_TRACT_SETUP *ITS)
+{
+   ENTRY("New_Insta_Tract_Setup");
+
+   if (!ITS) ITS = (INSTA_TRACT_SETUP *)calloc(1,sizeof(INSTA_TRACT_SETUP));
+   else Free_Insta_Tract_Setup(ITS);
+   
+   /* Put any initialization here ... */
+   
+   RETURN(ITS);
+}
+
+
+// *****************-> NIMLly reading in DTI input  <-*********************
+
+NI_element * ReadDTI_inputs(char *fname) 
+{
+   NI_stream ns=NULL;
+   NI_element *nel=NULL;
+   char *strm=NULL;
+   FILE *fin4=NULL;
+   
+   ENTRY("ReadDTI_inputs");  
+       
+   if (!fname || !THD_is_file(fname)) RETURN(NULL);
+   
+   if (STRING_HAS_SUFFIX(fname,".niml.opts")) {
+      strm = (char *)calloc(strlen(fname)+20, sizeof(char));
+      sprintf(strm,"file:%s",fname);
+      if (!(ns = NI_stream_open( strm , "r" ))) {
+         ERROR_message("Failed to open %s\n", strm);
+         free(strm); RETURN(NULL);
+      }
+      if (!(nel = NI_read_element( ns , 2 ))) {
+         ERROR_message("Failed to read element from \n", strm);
+         free(strm); RETURN(NULL);
+      }
+      NI_stream_close(ns); free(strm); strm = NULL;
+   } else {
+      ERROR_message("Failed to get DTI inputs from %s",fname);
+      RETURN(NULL);
+   }
+   
+   RETURN(nel);
+}      
+
+int NI_getDTI_inputs( NI_element *nel, 
+                      char **NameVEC,
+                      char *NameXF, 
+                      char **NameSCAL, 
+                      char **NameP,
+                      int *extrafile, int *pars_top)
+{
+   char *atr=NULL;
+   int ct_ex = 0;
+
+   ENTRY("NI_getDTI_inputs");
+   if (!nel) RETURN(1);
+   
+   if (NameVEC[0] && (atr=NI_get_attribute(nel,"dti_V1"))) {
+      snprintf(NameVEC[0],100,"%s", atr);
+   }
+   if (NameVEC[1] && (atr=NI_get_attribute(nel,"dti_V2"))) {
+      snprintf(NameVEC[1],100,"%s", atr);
+   }
+   if (NameVEC[2] && (atr=NI_get_attribute(nel,"dti_V3"))) {
+      snprintf(NameVEC[2],100,"%s", atr);
+   }
+   if (NameXF && (atr=NI_get_attribute(nel,"dti_XF"))) {
+      snprintf(NameXF,100,"%s", atr); *extrafile = 1;
+   }
+   else
+      NameXF = NULL;
+   if (NameSCAL[0] && (atr=NI_get_attribute(nel,"dti_FA"))) {
+      snprintf(NameSCAL[0],100,"%s", atr);
+   }
+   if (NameSCAL[1] && (atr=NI_get_attribute(nel,"dti_MD"))) {
+      snprintf(NameSCAL[1],100,"%s", atr);
+   }
+   if (NameSCAL[2] && (atr=NI_get_attribute(nel,"dti_L1"))) {
+      snprintf(NameSCAL[2],100,"%s", atr);
+   }
+   // allow up to four extra files
+   if (NameP[0] && (atr=NI_get_attribute(nel,"dti_P1"))) {
+      snprintf(NameP[0],100,"%s", atr);  ct_ex++;
+   }
+   else snprintf(NameP[0],100,"%s", "\0");
+   if (NameP[1] && (atr=NI_get_attribute(nel,"dti_P2"))) {
+      snprintf(NameP[1],100,"%s", atr);  ct_ex++;
+   }
+   else snprintf(NameP[1],100,"%s", "\0");
+   if (NameP[2] && (atr=NI_get_attribute(nel,"dti_P3"))) {
+      snprintf(NameP[2],100,"%s", atr);  ct_ex++;
+   }
+   else snprintf(NameP[2],100,"%s", "\0");
+   if (NameP[3] && (atr=NI_get_attribute(nel,"dti_P4"))) {
+      snprintf(NameP[3],100,"%s", atr);  ct_ex++;
+      //printf("\nHERE! %d\n",ct_ex);
+   }
+   else snprintf(NameP[3],100,"%s", "\0");
+   
+   *pars_top = 2 + 3 + ct_ex; // RD and extra; FA,MD;extras
+
+   //printf("\nLISTED: PARS_TOP=%d with ct_ex=%d\n", *pars_top, ct_ex);
+   //   printf("\nLISTED: NameSCAL[0]=%s\n", NameSCAL[0]);
+
+
+   RETURN(0);
+}
+
