@@ -76,7 +76,6 @@ gen_ss_review_scripts.py - generate single subject analysis review scripts
       motion_dset          dfile_rall.1D
       volreg_dset          pb02.FT.r01.volreg+tlrc.HEAD
       xmat_regress         X.xmat.1D
-      stats_dset           stats.FT+tlrc.HEAD
       final_anat           FT_anat+tlrc.HEAD
 
    optional files/datasets (censor files are required if censoring was done):
@@ -84,8 +83,10 @@ gen_ss_review_scripts.py - generate single subject analysis review scripts
       mask_dset            full_mask.FT+tlrc.HEAD
       censor_dset          motion_FT_censor.1D
       sum_ideal            sum_ideal.1D
+      stats_dset           stats.FT+tlrc.HEAD
       xmat_uncensored      X.nocensor.xmat.1D
       tsnr_dset            TSNR.ft+tlrc.HEAD
+      gcor_dset            out.gcor.1D
 
 ------------------------------------------
 
@@ -261,51 +262,76 @@ echo "num TRs above out limit   : $mcount"
 echo ""
 
 # ------------------------------------------------------------
-# get tcat files and count TRs
-set tcat_files = ( `find . -maxdepth 1 -name "pb00*$subj*.HEAD" -print` )
-if ( $#tcat_files == 0 ) then
-    echo "** missing tcat files, skipping num runs and TRs per run..."
-    echo ""
-else
-    set trs = ( )
-    foreach file ( $tcat_files )
-        set trs = ( $trs `3dnvals $file` )
-    end
-    echo "num runs found            : $#tcat_files"
-    echo "num TRs per run           : $trs"
+# note number of runs, TRs per run, and possibly censored TRs per run
+set nruns = ( `1d_tool.py -infile X.xmat.1D -show_num_runs` )
+set trs   = ( `1d_tool.py -infile X.xmat.1D -show_tr_run_counts trs_no_cen` )
+echo "num runs found            : $nruns"
+echo "num TRs per run           : $trs"
+
+if ( $was_censored ) then
+   set tra = ( `1d_tool.py -infile $xmat_regress -show_tr_run_counts trs`)
+   set trc = ( `1d_tool.py -infile $xmat_regress -show_tr_run_counts trs_cen`)
+   set trf = ( `1d_tool.py -infile $xmat_regress -show_tr_run_counts frac_cen`)
+   echo "num TRs per run (applied) : $tra"
+   echo "num TRs per run (censored): $trc"
+   echo "fraction censored per run : $trf"
 endif
 
-# count total (uncensored) TRs from X-matrix (as a check against sum of TRs)
+# ------------------------------------------------------------
+# count total TRs (uncensored and censored) from X-matrix
+
+# note X-matrix dimensions
+set rows_cols = ( `1d_tool.py -infile $xmat_regress -show_rows_cols -verb 0` )
+set num_trs = $rows_cols[1]
+set total_trs = $num_trs        # total might change if censoring
+
 if ( $?xmat_uncensored ) then
    set xmat = $xmat_uncensored
-else
-   set xmat = $xmat_regress
+   set urc = ( `1d_tool.py -infile $xmat -show_rows_cols -verb 0` )
+   set total_trs = $urc[1]
+   echo "TRs total (uncensored)    : $total_trs"
 endif
-set rows_cols = ( `1d_tool.py -infile $xmat -show_rows_cols -verb 0` )
-set num_trs = $rows_cols[1]
-echo "TRs total (uncensored)    : $num_trs"
-echo ""
 
+echo "TRs total                 : $num_trs"
+
+@ dof_rem = $rows_cols[1] - $rows_cols[2]
+echo "degress of freedom used   : $rows_cols[2]"
+echo "degress of freedom left   : $dof_rem"
+echo ""
 """
 
-g_censor_results_str = """
+
+g_reg_counts_str = """
 # ----------------------------------------------------------------------
-# report censoring results (if applicable)
+# report TR counts per stim, and possibly censor counts
+
+# note number of regressors of interest
+set rc = ( `1d_tool.py -infile $xstim -show_rows_cols -verb 0` )
+set nint = $rc[2]
+@ nm1 = $nint - 1
+
 if ( $was_censored ) then
     set ntr_censor = `cat $censor_dset | grep 0 | wc -l`
     echo "TRs censored              : $ntr_censor"
-    echo "censor fraction           : `ccalc $ntr_censor/$num_trs`"
-
-    # note number of regressors of interest
-    set rc = ( `1d_tool.py -infile $xstim -show_rows_cols -verb 0` )
-    set nint = $rc[2]
+    echo "censor fraction           : `ccalc $ntr_censor/$total_trs`"
     echo "num regs of interest      : $nint"
+else
+    # no censoring - just compute num TRs per regressor
+    set stim_trs = ()
+    foreach index ( `count -digits 1 0 $nm1` )
+        set st = `1deval -a $xstim"[$index]" -expr 'bool(a)' | grep 1 | wc -l`
+        set stim_trs = ( $stim_trs $st )
+    end
+    echo "num regs of interest      : $nint"
+    echo "num TRs per stim          : $stim_trs"
+endif
 
+# report per-stim censoring
+if ( $was_censored && $num_stim > 0 ) then
     # compute fractions of stimulus TRs censored in each
     set stim_trs = ()
     set stim_trs_censor = ()
     set stim_frac_censor = ()
-    @ nm1 = $nint - 1
     foreach index ( `count -digits 1 0 $nm1` )
         # count response TRs, with and without censoring
         # (their difference is the number of TRs lost to censoring)
@@ -322,14 +348,15 @@ if ( $was_censored ) then
         set stim_frac_censor = ( $stim_frac_censor $ff )
     end
 
-    echo "num TRs per stim          : $stim_trs"
+    echo "num TRs per stim (orig)   : $stim_trs"
     echo "num TRs censored per stim : $stim_trs_censor"
     echo "fraction TRs censored     : $stim_frac_censor"
     echo ""
-endif  # $was_censored
+endif
 
 """
 
+
 # rcr - do not include this...
 g_basic_count_sfiles = """
 # ------------------------------------------------------------
@@ -348,14 +375,34 @@ if ( -f $tsnr_dset && -f $mask_dset ) then
 endif
 """
 
-g_basic_finish_str = """
+g_basic_gcor_str = """
+# ------------------------------------------------------------
+# get global correlation
+if ( -f $gcor_dset ) then
+    set gcor_val = `cat $gcor_dset`
+    echo "global correlation (GCOR) : $gcor_val"
+endif
+"""
+
+g_basic_fstat_str = """
 # ------------------------------------------------------------
 # note maximum F-stat
-if ( -f $stats_dset ) then
-    set fmax = `3dBrickStat -slow -max $stats_dset"[Full_Fstat]"`
-    echo "maximum F-stat            : $fmax"
+if ( -f "$stats_dset" ) then
+  set fmax = `3dBrickStat -slow -max $stats_dset"[Full_Fstat]"`
+  echo "maximum F-stat            : $fmax"
 endif
+"""
 
+g_basic_fstat_mask_str = """
+# ------------------------------------------------------------
+# note maximum masked F-stat
+if ( -f "$stats_dset" && -f $mask_dset ) then
+  set fmax = `3dBrickStat -slow -max -mask $mask_dset $stats_dset"[Full_Fstat]"`
+  echo "maximum F-stat (masked)   : $fmax"
+endif
+"""
+
+g_basic_finish_str = """
 # ------------------------------------------------------------'
 # note blur estimates
 set blur_file = blur_est.$subj.1D
@@ -419,6 +466,7 @@ g_eg_uvar.enorm_dset      = 'motion_FT_enorm.1D'
 g_eg_uvar.censor_dset     = 'motion_FT_censor.1D'
 g_eg_uvar.motion_dset     = 'dfile_rall.1D'
 g_eg_uvar.outlier_dset    = 'outcount_rall.1D'
+g_eg_uvar.gcor_dset       = 'out.gcor.1D'
 g_eg_uvar.mot_limit       = 0.3
 g_eg_uvar.out_limit       = 0.1
 g_eg_uvar.xmat_regress    = 'X.xmat.1D'
@@ -441,6 +489,7 @@ g_uvar_dict = {
  'enorm_dset'       :'set motion_enorm file',
  'motion_dset'      :'set motion parameter file',
  'outlier_dset'     :'set outcount_rall file',
+ 'gcor_dset'        :'set gcor_dset file',
  'mot_limit'        :'set motion limit (maybe for censoring)',
  'out_limit'        :'set outlier limit (maybe for censoring)',
  'xmat_regress'     :'set X-matrix file used in regression',
@@ -462,6 +511,7 @@ g_cvars_defs.scr_basic  = '@ss_review_basic'
 g_cvars_defs.scr_drive  = '@ss_review_driver'
 g_cvars_defs.cmds_drive = '@ss_review_driver_commands'
 g_cvars_defs.xstim      = 'X.stim.xmat.1D'
+g_cvars_defs.basic_command = 'yes' # if set, include command in review_basic
 g_cvars_defs.out_prefix = ''    # if set, use as prefix to cvar files
 g_cvars_defs.exit0      = 0     # if set, return 0 even on errors
 
@@ -510,21 +560,36 @@ g_history = """
         - use 3 decimal places for TR censor fractions
    0.18 Apr 12, 2012: replace enumerate(), for backport to python 2.2
    0.19 May 01, 2012: added -prefix option; added censoring to 1dplot commands
-   0.20 May 09, 2012: accomodate more than 99 runs
+   0.20 May 09, 2012: accommodate more than 99 runs
    0.21 May 11, 2012: also output average censored motion (per TR)
    0.22 Jun 14, 2012: use afni -com instead of plugout_drive
                       (avoids issue on systems with multiple users)
                       Thanks to V Razdan and N Adleman for noting the issue.
-   0.23 Jun 25, 2012: ick, fixed uninitilaized cpad1,2 (if no censoring)
+   0.23 Jun 25, 2012: ick, fixed uninitialized cpad1,2 (if no censoring)
    0.24 Jul 17, 2012:
         - add checks for volreg and uncensored X-mat
         - probably init view from volreg
    0.25 Aug 23, 2012: allow passing of -censor_dset
    0.26 Sep 06, 2012: print missing xmat message w/out debug as it is fatal
    0.27 Apr 29, 2013: set AFNI_NO_OBLIQUE_WARNING in scripts
+   0.28 Oct 24, 2013: output global correlation, and DoF info from review_basic
+   0.29 Dec 16, 2013: fixed use of num_trs with censoring
+   0.30 Dec 26, 2013: max F (and for cluster jump) are masked, if possible
+   0.31 Dec 27, 2013: also output censored TRs per run, and fractions
+   0.32 Feb 10, 2014: show TRs per run, applied and censored
+   0.33 Mar 06, 2014:
+        - if censoring, create X.stim.xmat.1D from uncensored matrix
+        - if no censor, still report num regs of interest and TRs per stim
+        - report per-stim censoring only with stim classes
+   0.34 Mar 11, 2014: added GSSRS command comment at bottom of _basic script
+   0.35 Mar 21, 2014: removed -e option from "tcsh -ef @ss_review_basic"
+        - grep commands are killing it when not finding matches
+        - linux systems are not terminating there, while macs do
+   0.36 Apr 09, 2014: for GCOR files, give priority to having 'out' in name
+   0.37 May 13, 2014: allow no stats, in case of rest and 3dTproject
 """
 
-g_version = "gen_ss_review_scripts.py version 0.27, April 29, 2013"
+g_version = "gen_ss_review_scripts.py version 0.37, May 13, 2014"
 
 g_todo_str = """
    - figure out template_space
@@ -778,8 +843,8 @@ class MyInterface:
       if self.guess_rm_trs():      return 1
       if self.guess_stats_dset():  return 1
       if self.guess_sum_ideal():   return 1
-      if self.guess_final_view():  return 1
       if self.guess_volreg_dset(): return 1
+      if self.guess_final_view():  return 1
       if self.guess_enorm_dset():  return 1
       if self.guess_motion_dset(): return 1
       if self.guess_outlier_dset():return 1
@@ -787,6 +852,7 @@ class MyInterface:
       if self.guess_align_anat():  return 1
       if self.guess_mask_dset():   return 1
       if self.guess_tsnr_dset():   return 1
+      if self.guess_gcor_dset():   return 1
 
       if self.find_censor_file():  return 1
 
@@ -1082,13 +1148,14 @@ class MyInterface:
             print '-- already set: final_view = %s' % self.uvars.final_view
          return 0
 
-      if not self.dsets.is_empty('volreg_dset'):
+      if self.dsets.is_not_empty('volreg_dset'):
          view = self.dsets.volreg_dset.view
-      elif not self.dsets.is_empty('stats_dset'):
+      elif self.dsets.is_not_empty('stats_dset'):
          view = self.dsets.stats_dset.view
       else:
          print '** no stats_dset to get final_view from'
          view = ''
+         print '== rcr - vr = %s' % self.dsets.val('volreg_dset')
 
       if len(view) != 5: # maybe surface, go after volreg explicitly for now
          vv = "+orig"
@@ -1407,6 +1474,38 @@ class MyInterface:
 
       return 0
 
+   def guess_gcor_dset(self):
+      """set uvars.gcor_dset"""
+
+      # check if already set
+      if self.uvars.is_not_empty('gcor_dset'):
+         if self.cvars.verb > 3:
+            print '-- already set: gcor_dset = %s' % self.uvars.gcor_dset
+
+      gstr = 'out.gcor.1D'
+
+      if os.path.isfile(gstr):
+         self.uvars.gcor_dset = gstr
+         return 0
+
+      # otherwise, try a glob pattern
+      gstr = '*[gG][cC][oO][rR]*.1D'
+      glist = glob.glob(gstr)
+      if len(glist) == 0:
+         print '** failed to find gcor dset, continuing...'
+         return 0 # failure is not terminal
+
+      # search for files containing 'out', but default to first
+      guse = glist[0]
+      for gfile in glist:
+         if gfile.find('out') >= 0:
+            guse = gfile
+            break
+
+      self.uvars.gcor_dset = guse
+
+      return 0
+
    def guess_volreg_dset(self):
       """set uvars.volreg_dset"""
 
@@ -1511,9 +1610,10 @@ class MyInterface:
       dlist = [d for d in dlist if d.find('_REMLvar+') < 0]
 
       if len(dlist) < 1:
-         print '** failed to guess at any stats dset, globstr = "%s"' % gform
-         print '   (X-matrix file "%s" may not apply)' % ax.fname
-         return 1
+         print '** failed to guess at any stats dset, resting state?'
+         print '   (else X-matrix file "%s" may not apply)' % ax.fname
+         self.uvars.stats_dset = 'NO_STATS'
+         return 0
       if len(dlist) == 1:
          sset = dlist[0]
       else:     # must pare down the list
@@ -1538,8 +1638,7 @@ class MyInterface:
 
       sb = BASE.afni_name(sname)
       if not sb.exist():
-         print '** stats dataset not found: %s' % xname
-         return 1
+         print "** warning: stats dataset not found: '%s'" % sname
 
       # set corresponding dset items and then go after uncensored
       self.dsets.stats_dset = sb
@@ -1634,12 +1733,24 @@ class MyInterface:
       if self.basic_overview(): return 1
 
       # most of script is just raw text
-      self.text_basic += g_censor_results_str
+      self.text_basic += g_reg_counts_str
       if self.uvars.is_not_empty('mask_dset') and \
          self.uvars.is_not_empty('tsnr_dset'):
          self.text_basic += g_basic_tsnr_str
+
+      if self.uvars.is_not_empty('gcor_dset'):
+         self.text_basic += g_basic_gcor_str
+
+      # maybe use mask for max F-stat
+      if self.uvars.is_not_empty('mask_dset'):
+         self.text_basic += g_basic_fstat_mask_str
+      else:
+         self.text_basic += g_basic_fstat_str
+
       self.text_basic += g_basic_finish_str
 
+      self.basic_add_command_string()
+
       return 0
 
    def basic_overview(self):
@@ -1648,8 +1759,7 @@ class MyInterface:
          astr = 'echo "final anatomy dset        : $final_anat"\n'
       else: astr = ''
 
-      sset = self.uvars.val('stats_dset')
-      if sset != None:
+      if self.uvars.is_not_empty('stats_dset'):
          sstr = 'echo "final stats dset          : $stats_dset"\n' \
                 'echo "final voxel resolution    : '               \
                                         '`3dinfo -ad3 $stats_dset`"\n'
@@ -1660,6 +1770,12 @@ class MyInterface:
       txt += \
         '# ------------------------------------------------------------\n'   \
         '# make non-basline X-matrix, if one is not already here\n'          \
+        'if ( $?xmat_uncensored ) then\n'                                    \
+        '    set xmat = $xmat_uncensored\n'                                  \
+        'else\n'                                                             \
+        '    set xmat = $xmat_regress\n'                                     \
+        'endif\n'                                                            \
+        '\n'                                                                 \
         'set xstim = %s\n'                                                   \
         'if ( ! -f $xstim ) then\n'                                          \
         '   set reg_cols = `1d_tool.py -infile $xmat -show_indices_interest`\n'\
@@ -1676,7 +1792,7 @@ class MyInterface:
       txt  = '# ' + '-'*70 + '\n' + \
              '# main variables regarding this single subject analysis\n'
 
-      format = 'set %-16s = %s\n'
+      form = 'set %-16s = %s\n'
       uvars  = self.uvars
       errs   = 0
       # some cases with matching var names
@@ -1684,7 +1800,7 @@ class MyInterface:
       for var in ['subj', 'rm_trs', 'num_stim', 'mot_limit', 'out_limit',
                   'final_view']:
          if uvars.valid(var):
-            txt += format % (var,self.uvars.val(var))
+            txt += form % (var,self.uvars.val(var))
          elif var in ['rm_trs']: # non-fatal
             print '** warning: basic script, missing variable %s' % var
          else:
@@ -1692,72 +1808,73 @@ class MyInterface:
             errs += 1
 
       if self.dsets.is_not_empty('censor_dset'):
-         txt += format % ('was_censored', '1')
+         txt += form % ('was_censored', '1')
       else:
-         txt += format % ('was_censored', '0')
+         txt += form % ('was_censored', '0')
 
       txt += '\n'
 
       var = 'tcat_dset'
       if self.uvars.is_not_empty(var):
-         txt += format % (var, self.uvars.val(var))
-      else: # non-fatal
-         print '** warning: basic script: missing variable %s' % var
+         txt += form % (var, self.uvars.val(var))
+
+      var = 'gcor_dset'
+      if self.uvars.is_not_empty(var):
+         txt += form % (var, self.uvars.val(var))
 
       var = 'enorm_dset'
       if uvars.is_not_empty(var):
-         txt += format % ('enorm_dset', uvars.val(var))
+         txt += form % ('enorm_dset', uvars.val(var))
       else:
          print '** basic script: missing variable %s' % var
          errs += 1
 
       var = 'motion_dset'
       if uvars.is_not_empty(var):
-         txt += format % ('motion_dset', uvars.val(var))
+         txt += form % ('motion_dset', uvars.val(var))
 
       var = 'outlier_dset'
       if uvars.is_not_empty(var):
-         txt += format % ('outlier_dset', uvars.val(var))
+         txt += form % ('outlier_dset', uvars.val(var))
       else:
          print '** basic script: missing variable %s' % var
          errs += 1
 
       var = 'xmat_regress'
       if uvars.is_not_empty(var):
-         txt += format % ('xmat_regress', uvars.val(var))
+         txt += form % ('xmat_regress', uvars.val(var))
       else:
          print '** basic script: missing variable %s' % var
          errs += 1
 
       var = 'stats_dset'
-      if self.uvars.is_not_empty(var):
-         txt += format % (var, self.uvars.val(var))
-      else:
-         print '** basic script: missing variable %s' % var
-         errs += 1
+      val = self.uvars.val(var)
+      if val != None:
+         txt += form % (var, self.uvars.val(var))
+      # else: okay - not required (resting state)
 
       var = 'censor_dset'
       if self.uvars.is_not_empty(var):
-         txt += format % (var, self.uvars.val(var))
+         txt += form % (var, self.uvars.val(var))
 
       var = 'xmat_uncensored'
       if self.uvars.is_not_empty(var):
-         txt += format % (var, self.uvars.val(var))
+         txt += form % (var, self.uvars.val(var))
       # else: okay - not required
 
       var = 'final_anat'
       if self.uvars.is_not_empty(var):
-         txt += format % (var, self.uvars.val(var))
+         txt += form % (var, self.uvars.val(var))
       # else: okay - not required
 
       var = 'mask_dset'
       if self.uvars.is_not_empty(var):
-         txt += format % (var, self.uvars.val(var))
+         txt += form % (var, self.uvars.val(var))
       # not required
 
       var = 'tsnr_dset'
       if self.uvars.is_not_empty(var):
-         txt += format % (var, self.uvars.val(var))
+         txt += form % (var, self.uvars.val(var))
       # not required
 
       if errs > 0: return 1
@@ -1766,10 +1883,15 @@ class MyInterface:
 
       return 0
 
+   def basic_add_command_string(self):
+      if self.cvars.val('basic_command') != 'yes': return
+
+      self.text_basic += '\n' + UTIL.get_command_str(comment=1) + '\n'
+
    def make_drive_script(self):
       # init script and commands-only text
       self.text_drive = ''
-      self.commands_drive = 'tcsh -ef %s\n' % self.cvars.val('scr_basic')
+      self.commands_drive = 'tcsh -f %s\n' % self.cvars.val('scr_basic')
 
       if self.drive_init(): return 1
       if self.drive_motion(): return 1
@@ -1792,7 +1914,7 @@ class MyInterface:
        '\n'                                                                 \
        'if ( -f %s ) then\n'                                                \
        '   echo ------------------- %s --------------------\n'              \
-       '   tcsh -ef %s\n'                                                   \
+       '   tcsh -f %s\n'                                                    \
        '   echo ---------------------------------------------------------\n'\
        '\n'                                                                 \
        '   prompt_user -pause "                      \\\n'                  \
@@ -1842,11 +1964,13 @@ class MyInterface:
       else: s3 = ''
 
       s4  = \
-          '# locate peak coords of biggest cluster and jump there\n'    \
-          'set maxcoords = ( `3dclust -1thresh $thresh -dxyz=1 1 2 \\\n'\
-          '    %s"[0]" | & awk \'/^ / {print $14, $15, $16}\' | head -n 1` )\n'\
-          'echo -- jumping to max coords: $maxcoords\n'                 \
-          % sset.pv()
+       '# locate peak coords of biggest masked cluster and jump there\n'  \
+       '3dcalc -a %s"[0]" -b %s -expr "a*b" \\\n'                         \
+       '       -overwrite -prefix .tmp.F\n'  \
+       'set maxcoords = ( `3dclust -1thresh $thresh -dxyz=1 1 2 .tmp.F+%s \\\n'\
+       '       | & awk \'/^ / {print $14, $15, $16}\' | head -n 1` )\n'\
+       'echo -- jumping to max coords: $maxcoords\n'                      \
+       % (sset.pv(), mset.pv(), self.uvars.final_view)
 
       txt += '# get 90 percentile for thresholding in afni GUI\n'       \
              '%s'                                                       \
@@ -2020,7 +2144,7 @@ class MyInterface:
          if mesg: print '** no %s dset, %s' % (dname, mesg)
          return 1
       if not dset.exist():
-         if mesg: print '** missing %s %s, %s' % (dname, dset, mesg)
+         if mesg: print '** missing %s %s, %s' % (dname, dset.prefix, mesg)
          return 1
       return 0
 
