@@ -624,11 +624,12 @@ def valid_as_identifier(text, name, warn=0, wparent=None, empty_ok=1):
       valid = 0
    else:
       # check for valid characters
-      # replace '_' with alpha, then check s[0].isalpha and rest isalphanum
+      # replace [._] with alpha, then check s[0].isalpha and rest isalphanum
       # scopy = copy.deepcopy(text)
       scopy = str(text)
       if scopy[0].isalpha():                    # first character is good
          scopy = scopy.replace('_', 'x')        # swap out '_' to use isalnum()
+         scopy = scopy.replace('.', 'x')        # swap out '.' to use isalnum()
          if scopy.isalnum(): return 1           # then VALID
       
       if ' ' in text or '\t' in text: extext = '     <contains whitespace>'
@@ -1427,22 +1428,19 @@ class button_list_widget(object):
          self.bdict[label] = b
          layout.addWidget(b)
 
-   def get_button_text(self, index=-1, button=None):
+   def get_button_text(self, button):
       """return text for button
-            index  : if apppropriate, return text for this button index
-            button : else, locate this button and return text
+            locate this button and return text
 
-        rcr - fix this, no blist, probably just
-              return button.text().toAscii().data()
+         verify that the button is in this widget, then
+         return button.text().toAscii().data()
 
         if no button is found, return 'NO_SUCH_BUTTON'"""
 
-      if index >= 0 and index < len(self.blist):
-         return blist[index].text().toAscii().data()
-      elif button != None:
-         for b in self.blist:
-            if b == button:
-               return b.text().toAscii().data()
+      keys = self.bdict.keys()
+      for key in keys:
+         if self.bdict[key] == button:
+            return button.text().toAscii().data()
  
       return 'NO_SUCH_BUTTON'
 
