@@ -2,7 +2,7 @@
 
 # system libraries
 import sys
-import gc, math
+import gc, math, copy
 
 # AFNI libraries
 import option_list as OL
@@ -54,52 +54,52 @@ timing_tool.py    - for manipulating and evaluating stimulus timing files
 --------------------------------------------------------------------------
 examples:
 
-   0. Basic commands:
+   Example 0. Basic commands:
 
          timing_tool.py -help
          timing_tool.py -hist
          timing_tool.py -show_valid_opts
          timing_tool.py -ver
 
-   1. Combine the timing of 2 files (extend one timing by another and sort).
-      Write to a new timing file.
+   Example 1. Combine the timing of 2 files (extend one timing by another and
+      sort).  Write to a new timing file.
 
          timing_tool.py -timing stimesB_01_houses.1D         \\
                         -extend stimesB_02_faces.1D          \\
                         -sort                                \\
                         -write_timing stimesB_extended.1D
 
-   2. Subtract 12 seconds from each stimulus time (to offset TRs dropped
-      prior to the magnetization steady state).
+   Example 2. Subtract 12 seconds from each stimulus time (to offset TRs
+      dropped prior to the magnetization steady state).
 
          timing_tool.py -timing stimesB_01_houses.1D         \\
                         -add_offset -12.0                    \\
                         -write_timing stimesB1_offset12.1D
 
-   2b.Similar to 2, but scale times (multiply) by 0.975, perhaps to account
-      for a different TR or stimulus duration.
+   Example 2b. Similar to 2, but scale times (multiply) by 0.975, perhaps to
+      account for a different TR or stimulus duration.
 
          timing_tool.py -timing stimesB_01_houses.1D         \\
                         -scale_data 0.975                    \\
                         -write_timing stimesB1_scaled.1D
 
-   2c.Similar to 2, but shift run times so that the first time occurs at the
-      beginning of the run.
+   Example 2c. Similar to 2, but shift run times so that the first time occurs
+      at the beginning of the run.
 
          timing_tool.py -timing stimesB_01_houses.1D         \\
                         -shift_to_run_offset 0               \\
                         -write_timing stimesB1_offset0.1D
 
-   3. Show timing statistics for the 3 timing files generated by example 3
-      from "make_random_timing -help".  To be accurate, specify the run
-      and stimulus durations.
+   Example 3. Show timing statistics for the 3 timing files generated by
+      example 3 from "make_random_timing -help".  To be accurate, specify the
+      run and stimulus durations.
 
          timing_tool.py -multi_timing stimesC_*.1D           \\
                         -run_len 200 -multi_stim_dur 3.5     \\
                         -multi_show_isi_stats
 
-   4. Show timing statistics for the timing files generated by example 6
-      from "make_random_timing -help".  Since both the run and stimulus
+   Example 4. Show timing statistics for the timing files generated by example
+      6 from "make_random_timing -help".  Since both the run and stimulus
       durations vary, 4 run lengths and 3 stimulus durations are given.
 
          timing_tool.py -multi_timing stimesF_*.1D           \\
@@ -107,26 +107,28 @@ examples:
                         -multi_stim_dur 3.5 4.5 3            \\
                         -multi_show_isi_stats
 
-   5. Partition the stimulus timing file 'response_times.1D' into multiple
-      timing files based on the labels in a partition file, partitions.1D.
-      If partitions.txt contains (0, correct, incorrect), there will be 2
-      output timing files, new_times_correct.1D and new_times_incorrect.1D.
+   Example 5. Partition the stimulus timing file 'response_times.1D' into
+      multiple timing files based on the labels in a partition file,
+      partitions.1D.  If partitions.txt contains (0, correct, incorrect),
+      there will be 2 output timing files, new_times_correct.1D and
+      new_times_incorrect.1D.
       Times where the partition label is '0' will be skipped.
 
          timing_tool.py -timing response_times.1D       \\
                         -partition partitions.txt new_times
 
-   6a.Convert a stim_times timing file to 0/1 stim_file format.  Suppose the
-      timing is random where each event lasts 2.5 seconds and runs are of 
-      lengths 360, 360 and 400 seconds.  Convert timing.txt to sfile.1D on a TR
-      grid of 0.5 seconds (oversampling), where a TR gets an event if at least        30% of the TR is is occupied by stimulus.
+   Example 6a. Convert a stim_times timing file to 0/1 stim_file format.
+      Suppose the timing is random where each event lasts 2.5 seconds and runs
+      are of lengths 360, 360 and 400 seconds.  Convert timing.txt to sfile.1D
+      on a TR grid of 0.5 seconds (oversampling), where a TR gets an event if
+      at least 30% of the TR is is occupied by stimulus.
 
           timing_tool.py -timing timing.txt -timing_to_1D sfile.1D      \\
                          -tr 0.5 -stim_dur 2.5 -min_frac 0.3            \\
                          -run_len 360 360 400
 
-    6b.Evaluate the results.  Use waver to convolve sfile.1D with GAM and use
-       3dDeconvolve to convolve the timing file with BLOCK(2.5).
+    Example 6b. Evaluate the results.  Use waver to convolve sfile.1D with GAM
+       and use 3dDeconvolve to convolve the timing file with BLOCK(2.5).
 
           waver -GAM -TR 0.5 -peak 1 -input sfile.1D > waver.1D
 
@@ -137,7 +139,7 @@ examples:
 
           1dplot -sepscl sfile.1D waver.1D X.xmat.1D
 
-   7. Truncate stimulus times to the beginning of respective TRs.
+   Example 7a. Truncate stimulus times to the beginning of respective TRs.
 
       Given a TR of 2.5 seconds and random stimulus times, truncate those times
       to multiples of the TR (2.5).
@@ -148,10 +150,10 @@ examples:
       Here, 11.83 would get truncated down to 10, the largest multiple of 2.5
       less than or equal to the original time.
 
-   7b.Instead of just truncating the times, round them to the nearest TR, based
-      on some TR fraction.  In this example, round up to the next TR when a
-      stimulus occurs at least 70% into a TR, otherwise round down to the
-      beginning.
+    Example 7b. Instead of just truncating the times, round them to the nearest
+      TR, based on some TR fraction.  In this example, round up to the next TR
+      when a stimulus occurs at least 70% into a TR, otherwise round down to
+      the beginning.
 
           timing_tool.py -timing timing.txt -tr 2.5 -round_times 0.7    \\
                          -write_timing round_times.txt
@@ -162,18 +164,19 @@ examples:
 
       Here, 11.83 would get rounded up to 12.5.
 
-   8. Create an event list from stimulus timing files.  The TR is 1.25s, events
-      are ~1 TR long, and require them to occupy at least half of the given TR.
-      Specify that rows should be per run and the run durations are all 370.
+   Example 8a. Create an event list from stimulus timing files.  The TR is
+      1.25s, events are ~1 TR long, and require them to occupy at least half of
+      the given TR.  Specify that rows should be per run and the run durations
+      are all 370.
 
           timing_tool.py -multi_timing stimes.*.txt        \\
                -multi_timing_to_events all.events.txt      \\
                -tr 1.25 -multi_stim_dur 1 -min_frac 0.5    \\
                -per_run -run_len 370 
 
-   8b.Break the event file into 2, one for a sequence of changing event types,
-      one for a sequence of ISIs (TRs from one event to the next, including
-      the TR of the event).  So if the event file from #8 shows:
+   Example 8b. Break the event list into 2, one for a sequence of changing
+      event types, one for a sequence of ISIs (TRs from one event to the next,
+      including the TR of the event).  So if the event file from #8 shows:
         0 0 3 0 0 0 0 1 0 2 2 0 0 0 ...
       The resulting event/ISI files would read:
         event: 0 3 1 2 2 ...
@@ -184,7 +187,7 @@ examples:
                -tr 1.25 -multi_stim_dur 1 -min_frac 0.5        \\
                -per_run -run_len 370 
 
-   9a. Convert from global stim times to local.
+   Example 9a. Convert from global stim times to local.
        This requires knowing the run lengths, say 4 runs of 200 seconds here.
        The result will have 4 rows, each starting at time 0.
 
@@ -205,14 +208,14 @@ examples:
        It will complain about the 3 times after the last run ends (no run
        should have times above 200 sec).
 
-   9b. Convert from local timing back to global.
+   Example 9b. Convert from local timing back to global.
 
           timing_tool.py -timing local.1D                       \\
                 -local_to_global global.1D                      \\
                 -run_len 200 200 200
 
-   10. Display within-TR statistics of stimulus timing files, to show when
-       stimuli occur within TRs.  The -tr option must be specified.
+   Example 10. Display within-TR statistics of stimulus timing files, to show
+       when stimuli occur within TRs.  The -tr option must be specified.
 
        a. one file: show offset statistics (using -show_tr_stats)
 
@@ -226,11 +229,57 @@ examples:
 
              timing_tool.py -multi_timing stim*.txt -tr 2.0 -warn_tr_stats
 
+   Example 11. Test a timing file for timing issues, which currently means
+       having times that are intended to be local but might be read as global.
+
+          timing_tool.py -multi_timing stim*.txt -test_local_timing
+
+   Examples 12 and 13 : akin to Example 8...
+
+   Example 12. Create a timing style event list.
+
+      Create a simple horizontal event list (one row per run), where the event
+      class is the (1-based) index of the given input file.  This is very
+      similar to the first file output in example 8b, but no TR information is
+      required here.  Events are simply ordered.
+
+          timing_tool.py -multi_timing stimes.*.txt            \\
+               -multi_timing_to_event_list index elist12.txt
+
+   Example 13a. Create a vertical GE (global events) list, showing ALL fields.
+
+      timing_tool.py -multi_timing stim.* -multi_timing_to_event_list GE:ALL -
+
+   Example 13b. Like 13a, but restrict the output to:
+
+             event index (i), duration (d), offset from previous (o), 
+             start time (t), and stim file (f)
+
+       Also, write the output to elist13b.txt, rather than the screen.
+
+          timing_tool.py -multi_timing stimes.*.txt            \\
+               -multi_timing_to_event_list GE:idotf elist13b.txt
+
+   Example 14. Partition one stimulus class based on others.
+
+      Class '1' (from the first input) is partitioned based on the class that
+      precedes it.  If none precede an early class 1 event, event INIT is used
+      as the default (else consider '-part_init 2', for example).
+
+          timing_tool.py -multi_timing stimes.*.txt            \\
+               -multi_timing_to_event_list part part1.pred.txt
+
+      The result could be applied to actually partition the first timing file,
+      akin to Example 5:
+
+         timing_tool.py -timing stimes.1.txt                   \\
+                        -partition part1.pred.txt stimes.1.part
+
 --------------------------------------------------------------------------
 Notes:
 
-   1. Action options are performed in the order of the options.  If the -chrono
-      option is given, everything (but -chrono) is.
+   1. Action options are performed in the order of the options.
+      Note: -chrono has been removed.
 
    2. Either -timing or -multi_timing is required for processing.
 
@@ -329,7 +378,6 @@ options with both single and multi versions (all single first):
 action options (apply to single timing element, only):
 
    ** Note that these options are processed in the order they are read.
-      See '-chrono' for similar notions.
 
    -add_offset OFFSET           : add OFFSET to every time in main element
 
@@ -498,6 +546,16 @@ action options (apply to single timing element, only):
 
             Consider '-write_timing'.
 
+   -test_local_timing           : test for certain problems with local timing
+
+        The main purpose of this is to test for timing files that are intended
+        to be interpreted by 3dDeconvolve as being LOCAL TIMES, but might
+        actually be interpreted as being GLOBAL TIMES.
+
+        Note that as of 18 Feb, 2014, any '*' in a timing file will cause it
+        to be interpreted by 3dDeconvolve as LOCAL TIMES, even if the file is
+        only a single column.
+
    -timing_to_1D output.1D      : convert stim_times format to stim_file
 
         e.g. -timing_to_1D stim_file.1D
@@ -614,15 +672,46 @@ action options (apply to multi timing elements, only):
 
            Consider example 8b.
 
+   -multi_timing_to_event_list STYLE FILE : make an event list file
+
+        e.g. -multi_timing_to_event_list index events.txt
+        e.g. -multi_timing_to_event_list GE:itodf event.list.txt
+
+        Similar to -multi_timing_to_events, but make a more simple event list
+        that does not require knowing the TR or run lengths.
+
+        The output is written to FILE, where 'stdout' or '-' mean to write to
+        the terminal window.
+
+        The information and format is specified by the STYLE field:
+
+           index        : write event index classes, in order, one row per run
+
+           part         : partition the first class of events according to the
+                          predecessor classes - the output is a list of class
+                          indices for events the precede those of the first
+                          class
+                          (this STYLE is esoteric, written for W Tseng)
+
+           GE:TYPE      : write a vertical list of events, according to TYPE
+
+              TYPE is a list comprised of the following specifiers, where
+              column output is in order specified (e.g. if i comes first, then
+              the first column of output will be the class index).
+
+                 i : event class index
+                 p : previous event class index
+                 t : event onset time
+                 d : event duration
+                 o : offset from previous event (including previous duration)
+                 f : event class file name
+
 ------------------------------------------
 general options:
 
    -chrono                      : process options chronologically
 
-        While the action options are already processed in order, general and
-        -timing options are not, unless the chrono option is given.  This 
-        allows one to do things like scripting a sequence of operations
-        within a single command.
+        This option has been removed.
 
    -min_frac FRAC               : specify minimum TR fraction
 
@@ -644,6 +733,19 @@ general options:
 
             Consider -timing_to_1D.
 
+   -part_init NAME             : specify a default partition NAME
+
+        e.g.     -part_init 2
+        e.g.     -part_init frogs
+        default: -part_init INIT
+
+        This option applies to '-multi_timing_to_event_list part'.  In the
+        case of generating a partition based on the previous events, this
+        option allow the user to specify the partition class to be used when
+        the class in question comes first (i.e. there is no previous event).
+
+        The default class is the label INIT (the other classes will be
+        small integers, from 1 to #inputs).
 
    -nplaces NPLACES             : specify # decimal places used in printing
 
@@ -744,9 +846,20 @@ g_history = """
    2.01 Oct 28, 2011 - allow use of -show_isi_stats w/o stim duration
    2.02 Oct 31, 2011 - added -show_tr_stats and -warn_tr_stats options
    2.03 Oct 03, 2012 - some options do not allow dashed parameters
+   2.04 Feb 18, 2014 - added -test_local_timing, to check for local vs. global
+        - in some cases, promote married types to combine/compare them
+        - keep track of '*' entries from timing files
+   2.05 Apr 24, 2014 - added -multi_timing_to_event_list
+        - generates simple event lists, partitions, and detailed event lists
+   2.06 Apr 29, 2014 - micro change to text output
+   2.07 May  9, 2014
+        - added -part_init option for WL Tseng
+        - removed -chrono option
+          (action items are still processed chronologically)
+   2.08 May 12, 2014 - default -part_init to INIT (0 not valid for -partition)
 """
 
-g_version = "timing_tool.py version 2.03, Oct 3, 2012"
+g_version = "timing_tool.py version 2.08, May 12, 2014"
 
 
 
@@ -757,9 +870,9 @@ class ATInterface:
       self.status          = 0                       # exit value
       self.valid_opts      = None
       self.user_opts       = None
+      self.all_edtypes     = ['i', 'p', 't', 'o', 'd', 'f']
 
       # user options
-      self.chrono          = 0          # are options processed chronologically
       self.nplaces         = -1         # num decimal places for writing
       self.run_len         = [0]        # time per run (for single/multi)
       self.verb            = verb
@@ -767,6 +880,7 @@ class ATInterface:
       self.min_frac        = 0.3        # applies to timing_to_1D
       self.tr              = 0          # applies to some output
       self.per_run         = 0          # conversions done per run
+      self.part_init       = 'INIT'     # default for -part_init
 
       # user options - single var
       self.timing          = None       # main timing element
@@ -787,6 +901,7 @@ class ATInterface:
 
       self.status = 1 # init to failure
       timing = LT.AfniTiming(fname, dur=self.stim_dur, verb=self.verb)
+
       if not timing.ready:
          print "** failed to read timing from '%s'" % fname
          return 1
@@ -955,6 +1070,9 @@ class ATInterface:
       self.valid_opts.add_opt('-partition', 2, [], 
                          helpstr='partition the events into multiple files')
 
+      self.valid_opts.add_opt('-part_init', 1, [], 
+                         helpstr='initial index for event list part (def=0)')
+
       self.valid_opts.add_opt('-round_times', 1, [], 
                          helpstr='round times up if past FRAC of TR')
 
@@ -970,6 +1088,9 @@ class ATInterface:
       self.valid_opts.add_opt('-sort', 0, [], 
                          helpstr='sort the data, per row')
 
+      self.valid_opts.add_opt('-test_local_timing', 0, [], 
+                         helpstr='check timing files for local timing issues')
+
       self.valid_opts.add_opt('-timing_to_1D', 1, [], 
                          helpstr='convert stim_times to 0/1 stim_file')
 
@@ -1005,11 +1126,11 @@ class ATInterface:
                          helpstr='convert stim_times event file')
       self.valid_opts.add_opt('-multi_timing_to_event_pair', 2, [], 
                          helpstr='convert stim_times event/isi files')
+      self.valid_opts.add_opt('-multi_timing_to_event_list', 2, [], 
+                         helpstr='convert to event list (style, filename)')
 
 
       # general options (including multi)
-      self.valid_opts.add_opt('-chrono', 0, [], 
-                         helpstr='process options chronologically')
       self.valid_opts.add_opt('-min_frac', 1, [], 
                          helpstr='min tr fraction (in [0,1.0])')
       self.valid_opts.add_opt('-nplaces', 1, [], 
@@ -1057,21 +1178,26 @@ class ATInterface:
       # ============================================================
       # read options specified by the user
       self.user_opts = OL.read_options(sys.argv, self.valid_opts)
-      uopts = self.user_opts            # convenience variable
+
+      # for convenience and popping
+      uopts = copy.deepcopy(self.user_opts)
+
       if not uopts: return 1            # error condition
 
       # ------------------------------------------------------------
       # check general options, esp. chrono
 
-      if uopts.find_opt('-chrono'): self.chrono = 1
+      # ------------------------------------------------------------
+      # process general options first (so -show options are still in order)
 
-      # if options are not chronological, process general options now
-      # (so -show options are still in order)
-      if not self.chrono:
-
-         val, err = self.user_opts.get_type_opt(int, '-verb')
+      oind = uopts.find_opt_index('-verb')
+      if oind >= 0:
+         val, err = uopts.get_type_opt(int, '-verb')
          if val != None and not err: self.verb = val
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-min_frac')
+      if oind >= 0:
          val, err = uopts.get_type_opt(float, '-min_frac')
          if val and not err:
             self.min_frac = val
@@ -1079,45 +1205,80 @@ class ATInterface:
             print '** invalid -min_frac = %g' % self.min_frac
             print '   (should be in [0,1])'
             return 1
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-nplaces')
+      if oind >= 0:
          val, err = uopts.get_type_opt(int, '-nplaces')
          if val and not err:
             self.nplaces = val
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-part_init')
+      if oind >= 0:
+         val, err = uopts.get_string_opt('-part_init')
+         if val and not err:
+            self.part_init = val
+         uopts.olist.pop(oind)
+
+      oind = uopts.find_opt_index('-per_run')
+      if oind >= 0:
          if uopts.find_opt('-per_run'): self.per_run = 1
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-tr')
+      if oind >= 0:
          val, err = uopts.get_type_opt(float, '-tr')
          if val and not err:
             self.tr = val
             if self.tr <= 0.0:
                print '** invalid (non-positive) -tr = %g' % self.tr
                return 1
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-run_len')
+      if oind >= 0:
          val, err = uopts.get_type_list(float, '-run_len')
          if type(val) == type([]) and not err:
             self.run_len = val
+         uopts.olist.pop(oind)
 
-         # main timing options
+      # main timing options
 
+      oind = uopts.find_opt_index('-timing')
+      if oind >= 0:
          val, err = uopts.get_string_opt('-timing')
          if val and not err:
             if self.set_timing(val): return 1
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-stim_dur')
+      if oind >= 0:
          val, err = uopts.get_type_opt(float, '-stim_dur')
          if val and not err:
             self.set_stim_dur(val)
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-multi_timing')
+      if oind >= 0:
          val, err = uopts.get_string_list('-multi_timing')
          if type(val) == type([]) and not err:
             if self.multi_set_timing(val): return 1
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-multi_stim_dur')
+      if oind >= 0:
          val, err = uopts.get_type_list(float, '-multi_stim_dur')
          if type(val) == type([]) and not err:
             self.multi_set_stim_durs(val)
+         uopts.olist.pop(oind)
 
+      oind = uopts.find_opt_index('-write_all_rest_times')
+      if oind >= 0:
          val, err = uopts.get_string_opt('-write_all_rest_times')
          if val and not err:
             self.all_rest_file = val
+         uopts.olist.pop(oind)
 
       # ------------------------------------------------------------
       # selection and process options:
@@ -1125,80 +1286,6 @@ class ATInterface:
 
       for opt in uopts.olist:
 
-         # if all options are chronological, check load and general, too
-
-         if self.chrono:
-
-            # continue after any found option
-
-            if opt.name == '-chrono': continue
-
-            # main timing options
-            if opt.name == '-timing':
-               if self.set_timing(opt.parlist[0]): return 1
-               continue
-
-            elif opt.name == '-multi_timing':
-               if self.multi_set_timing(opt.parlist): return 1
-               continue
-
-            elif opt.name == '-stim_dur':
-               val, err = self.user_opts.get_type_opt(float, opt=opt)
-               if val != None and err: return 1
-               else: self.set_stim_dur(val)
-               continue
-
-            elif opt.name == '-multi_stim_dur':
-               val, err = self.user_opts.get_type_list(float, opt=opt)
-               if val != None and err: return 1
-               else: self.multi_set_stim_durs(val)
-               continue
-
-            # general options
-
-            val, err = uopts.get_type_opt(float, '-min_frac')
-            if val and not err:
-               self.min_frac = val
-            if self.min_frac < 0.0 or self.min_frac > 1.0:
-               print '** invalid -min_frac = %g' % self.min_frac
-               print '   (should be in [0,1])'
-               return 1
-
-            elif opt.name == '-nplaces':
-               val, err = self.user_opts.get_type_opt(int, '', opt=opt)
-               if val != None and err: return 1
-               else: self.nplaces = val
-               continue
-
-            elif opt.name == '-per_run':
-               self.per_run = 1
-               continue
-
-            elif opt.name == '-run_len':
-               val, err = self.user_opts.get_type_list(float, opt=opt)
-               if val != None and err: return 1
-               else: self.run_len = val
-               continue
-
-            val, err = uopts.get_type_opt(float, '-tr')
-            if val and not err:
-               self.tr = val
-               if self.tr <= 0.0:
-                  print '** invalid -tr = %g' % self.tr
-                  return 1
-
-            elif opt.name == '-verb':
-               val, err = self.user_opts.get_type_opt(int, '', opt=opt)
-               if val != None and err: return 1
-               else: self.verb = val
-               continue
-
-            elif opt.name == '-write_all_rest_times':
-               val, err = self.user_opts.get_string_opt('', opt=opt)
-               if val != None and err: return 1
-               else: self.all_rest_file = val
-               continue
-
          #---------- action options, always chronological ----------
 
          # check multi- options first, then require self.timing
@@ -1207,7 +1294,7 @@ class ATInterface:
             self.show_multi()
             continue
 
-         if opt.name == '-add_rows':
+         elif opt.name == '-add_rows':
             if not self.timing:
                print "** '%s' requires -timing" % opt.name
                return 1
@@ -1329,6 +1416,12 @@ class ATInterface:
                return 1
             if self.show_tr_stats(warn=1): return 1
 
+         elif opt.name == '-test_local_timing':
+            if not self.timing and len(self.m_timing) == 0:
+               print "** '%s' requires -timing or -multi_timing" % opt.name
+               return 1
+            self.test_local_timing()
+
          elif opt.name == '-timing_to_1D':
             if not self.timing:
                print "** '%s' requires -timing" % opt.name
@@ -1355,6 +1448,15 @@ class ATInterface:
             if val != None and err: return 1
             self.multi_timing_to_events(val)
 
+         # just pass the event filename
+         elif opt.name == '-multi_timing_to_event_list':
+            if not self.m_timing:
+               print "** '%s' requires -multi_timing" % opt.name
+               return 1
+            val, err = uopts.get_string_list('', opt=opt)
+            if val != None and err: return 1
+            self.multi_timing_to_event_list(val[1], style=val[0])
+
          elif opt.name == '-transpose':
             if not self.timing:
                print "** '%s' requires -timing" % opt.name
@@ -1394,8 +1496,190 @@ class ATInterface:
             if val != None and err: return 1
             self.write_timing(val)
 
+         else:
+            print '** unknown option: %s' % opt.name
+            return 1
+
       return 0
 
+   def multi_timing_to_event_list(self, fname='stdout', style='index'):
+      """convert multi-stim_times to 1..N sorted event list
+
+         style:
+            index       write as a per-run sorted event list 1..N
+            part        partition first class events according to predecessors
+                        (so 2 CLASS_1 elements in a row is an error)
+                        if first element, default to follow first other type
+            GE:TYPE     write as global events, comments between runs, with:
+                        TYPE = i        : event class index
+                               p        : previous event class index
+                               t        : event time
+                               o        : offset from previous
+                               d        : stim duration
+                               f        : event class file name
+                             ALL        : use all valid types
+                        e.g. itf        - for index, time and file name
+      """
+
+      tlist = self.m_timing     # convenience
+
+      if len(tlist) < 1:
+         print '** no multi_timing, cannot convert'
+         return 1
+      if style not in ['index', 'part'] and style[0:3] != 'GE:':
+         print '** invalid style %s' % style
+         return 1
+
+      # check for partition styles
+      if style == 'part' and len(tlist) < 2:
+         print '** event_list partition requires at least 2 timing inputs'
+         return 1
+
+      # check for GE:TYPE style
+      s1d_type = ''
+      if style[0:3] == 'GE:':
+         s1d_type = style[3:]
+         if s1d_type != 'ALL':
+            for st in s1d_type:
+               if st not in self.all_edtypes:
+                  print "** invalid GE: event style '%s' in %s" % (st, style)
+                  return 1
+
+      if fname in ['-', 'stdout']: fp = sys.stdout
+      else:
+         try: fp = open(fname, 'w')
+         except:
+            print '** failed to open %s for writing' % fname
+            return 1
+
+      nruns = len(tlist[0].mdata)
+
+      # test nruns
+      for index, timing in enumerate(tlist):
+         if len(timing.mdata) != nruns:
+            print '** have variable runs between timing files %s and %s' \
+                  % (tlist[0].fname, timing.fname)
+            return 1
+
+      # get to work
+      for rind in range(nruns):
+         allevents = [] # complete event list for current run
+         etlist = []   # if printing event types, accumulate first
+         for index, timing in enumerate(tlist):
+             md = copy.deepcopy(timing.mdata[rind])
+             for event in md:
+                 event.append(index+1)
+             # now events are: [time, [amp mods], dur, class_index]
+             allevents.extend(md)
+
+         allevents.sort() # sort by times, which are the first elements
+
+         if s1d_type and self.verb > 0:
+            fp.write('# have %d events in run %d\n' % (len(allevents),rind+1))
+            etlist.append(self.make_s1d_ehdr_list(s1d_type))
+
+         # and output
+         for eind, event in enumerate(allevents):
+            cind = event[3]
+            # track previous class and event times
+            if eind == 0:
+               cprev = self.part_init
+               tprev = 0
+            else:
+                cprev  = allevents[eind-1][3]
+                tprev = allevents[eind-1][0]
+
+            if s1d_type != '':
+               etlist.append(self.make_s1d_estr_list(s1d_type,cind,cprev=cprev,
+                                   etime=event[0], tprev=tprev, dur=event[2]))
+            elif style == 'index': fp.write('%d ' % cind)
+            elif style == 'part':
+               if cind != 1: continue # only write predecessors of class 1
+               if cprev == 1 and self.verb > 0:
+                  print '** run %d, event %d: class %d preceded by class %s' \
+                        % (rind, eind, cind, cprev)
+               fp.write('%s ' % cprev)
+            else:
+               print '** invalid style %s' % style
+               return 1
+
+         # done with current run, print global events, if we have them
+         # (could join() them, but they vary in width)
+         if len(etlist) > 0:
+            clens = self.get_et_col_widths(etlist)
+            for eline in etlist:
+               for eind, estr in enumerate(eline):
+                  if eind: fp.write('  ')
+                  fp.write('%-*s' % (clens[eind], estr))
+               fp.write('\n')
+
+         fp.write('\n')
+
+      if fp != sys.stdout: fp.close()
+
+      return 0
+
+   def get_et_col_widths(self, estr_list):
+      """for each column, return the maximum element length"""
+      if len(estr_list) < 1: return []
+      elen = len(estr_list[0])
+      if elen < 1: return []
+
+      wlist = [max([len(elist[ind]) for elist in estr_list]) \
+                                    for ind in range(elen)]
+
+      return wlist
+
+   def make_s1d_estr_list(self, stypes, cind=0, cprev=0,
+                             etime=0.0, tprev=0.0, dur=0.0, maxfilelen=10):
+      tlist = self.m_timing
+
+      # apply special case of ALL for types
+      if stypes == 'ALL': etypes = self.all_edtypes
+      else:               etypes = stypes
+
+      elist = []
+      for st in etypes:
+         if   st == 'i': astr = '%d' % cind
+         elif st == 'p': astr = '%s' % cprev
+         elif st == 'f': astr = '%-*s' % (maxfilelen,tlist[cind-1].fname)
+         elif st == 'd': astr = '%8.3f' % dur
+         elif st == 'o':
+            offset = etime-tprev-dur
+            if offset == 0.0:
+                         astr = '   0    '
+            else:        astr = '%8.3f' % offset
+         elif st == 't': astr = '%8.3f' % etime
+         else:
+            print '** invalid GE: style %s' % style
+            return []
+         elist.append(astr)
+
+      return elist
+                        
+   def make_s1d_ehdr_list(self, stypes):
+
+      # apply special case of ALL for types
+      if stypes == 'ALL': etypes = self.all_edtypes
+      else:               etypes = stypes
+
+      elist = []
+      for sind, st in enumerate(etypes):
+         if   st == 'i': astr = 'class'
+         elif st == 'p': astr = 'prev_class'
+         elif st == 'f': astr = 'stim_file'
+         elif st == 'd': astr = 'duration'
+         elif st == 'o': astr = 'timediff'
+         elif st == 't': astr = 'event_time'
+         else:
+            print '** invalid GE: style %s' % style
+            return []
+         if sind == 0: elist.append('# ' + astr)
+         else:         elist.append(astr)
+
+      return elist
+                        
+
    def multi_timing_to_events(self, fname, wname=None):
       """convert multi-stim_times to 0..N event list
          If wname is not set, write event list to fname.
@@ -1668,6 +1952,26 @@ class ATInterface:
 
       return 0
 
+   def test_local_timing(self):
+      """test timing files for known issues
+            - looks local, but might be treated as global
+      """
+
+      # just start with everything that might be a timing element
+      if self.timing: tlist = [self.timing]
+      else:           tlist = []
+      tlist.extend(self.m_timing)
+
+      if len(tlist) == 0:
+         print '** no timing/multi_timing, nothing to test'
+         return 1
+
+      errs = 0
+      for timing in tlist:
+         errs += timing.looks_local_but_3dD_global(warn=1)
+
+      return errs
+
    def test(self, verb=3):
       # init
       print '------------------------ initial reads -----------------------'
@@ -1687,6 +1991,11 @@ class ATInterface:
       # more tests
       return None
 
+def compare_events(e0, e1):
+   if   e0[0] < e1[0]: return -1
+   elif e0[0] > e1[0]: return 1
+   else:               return 0
+
 def main():
    ard = ATInterface()
    if not ard: return 1
