@@ -2,10 +2,12 @@
 #define SUMA_DATASETS_INCLUDED
 
 #include "matrix.h"
+#include "ptaylor/TrackIO.h"
 #include "suma_afni_surface.h"
 
 #define MAX_ERRLOG_MSG 1000
 #define MAX_ERRLOG_FUNCNAME 200
+#define TMP_NAME_SEP "_LDPID_"
 
 /*! macro to avoid typecasting warnings when going from 
     void * pointers (or XtPointer) to int and vice versa */
@@ -29,14 +31,19 @@
 
 
 #ifdef USE_TRACING
-#define SUMA_DUMP_TRACE(ihead) { /* taken from dbtrace.h */\
+#define SUMA_DUMP_TRACE( ... ) { /* taken from dbtrace.h */\
    int m_ii;  \
-   char *head=(char*)ihead; /* a trick to quiet compiler warnings about \
-               fixed length   strings addresses always evaluating to true */\
-   if (head) { \
-      SUMA_S_Note(head);\
-   } else {SUMA_S_Note("Dumping Trace:");\
-   }   \
+   SUMA_S_Note( __VA_ARGS__ );\
+   if( DBG_num >= 0 ){  \
+      for( m_ii=DBG_num-1; m_ii >= 0 ; m_ii-- ) \
+         fprintf(stderr,"%*.*s%s\n",m_ii+1,m_ii+1," ",DBG_rout[m_ii]) ; \
+   } else { \
+      fprintf(stderr,"[No debug tracing stack: DBG_num=%d]\n",DBG_num) ;   \
+   }  \
+}
+#define SUMA_EDUMP_TRACE( ... ) { /* taken from dbtrace.h */\
+   int m_ii;  \
+   SUMA_S_Err( __VA_ARGS__ ); \
    if( DBG_num >= 0 ){  \
       for( m_ii=DBG_num-1; m_ii >= 0 ; m_ii-- ) \
          fprintf(stderr,"%*.*s%s\n",m_ii+1,m_ii+1," ",DBG_rout[m_ii]) ; \
@@ -46,7 +53,9 @@
 }
 #else
 #define SUMA_DUMP_TRACE(x) /* nada */
+#define SUMA_EDUMP_TRACE( ... ) /* nada */
 #endif
+#define SUMA_T_Err SUMA_EDUMP_TRACE
 
 typedef struct {
     char macroname[100];
@@ -61,6 +70,7 @@ typedef enum { SUMA_NO_NUM_UNITS = 0,
                SUMA_MM_UNITS,
                SUMA_P_VALUE_UNITS,
                SUMA_Q_VALUE_UNITS,
+               SUMA_PERC_VALUE_UNITS,
                
                SUMA_N_NUMERICAL_UNITS
                } SUMA_NUMERICAL_UNITS;
@@ -88,6 +98,7 @@ typedef enum {
    SUMA_ERROR_DSET_TYPE = -1,
    SUMA_NO_DSET_TYPE,
    SUMA_NODE_BUCKET,
+   SUMA_VOXEL_BUCKET,
    SUMA_AFNI_NODE_BUCKET,
    SUMA_NODE_ROI, /*!< Col0: Node ID, Col1: ROI label (int) */
    SUMA_NODE_RGB,
@@ -108,6 +119,7 @@ typedef enum {
    SUMA_SEGMENT_OBJECT,
    SUMA_LABEL_TABLE_OBJECT,
    SUMA_GRAPH_BUCKET,
+   SUMA_TRACT_BUCKET,
    SUMA_N_DSET_TYPES
 } SUMA_DSET_TYPE; /*!<  Type of data set ( should be called Object, not DSET ) 
                         When you add a new element, modify functions
@@ -151,8 +163,10 @@ typedef enum {
    SUMA_NO_COL_TYPE,
    SUMA_NODE_INT,    /*!< Generic integer */
    SUMA_NODE_INDEX,  /*!< index of a node OR edge to locate it in its domain */
-   SUMA_NODE_ILABEL, /*!< An integer coding for a label */
-   SUMA_NODE_SLABEL, /*!< An integer coding for a string label */
+   SUMA_NODE_ILABEL, /*!< An integer coding for an integer label */
+   SUMA_NODE_SLABEL, /*!< An string label */
+   SUMA_GNODE_IGROUP, /*!< An integer coding for a group integer label of 
+                           a graph node*/
    SUMA_NODE_FLOAT,  /*!< Generic float */ 
    SUMA_NODE_CX,     /*!< Node convexity */
    SUMA_NODE_X,      /*!< Node X coordinate */
@@ -186,6 +200,9 @@ typedef enum {
                               need changing*/
    SUMA_EDGE_P1_INDEX,        /* First point (Graph Node) defining an edge */
    SUMA_EDGE_P2_INDEX,        /* Second point defining an edge */
+   
+   
+   SUMA_N_COL_TYPES           /* MAX number of col types */
 }  SUMA_COL_TYPE; /*!<  Column types.
                         When you add a new element, you need to modify
                         SUMA_AddColAttr
@@ -196,11 +213,23 @@ typedef enum {
 #define SUMA_IS_GNODE_IXYZ_COL(ctp) (((ctp)==SUMA_NODE_X || \
                                  (ctp)==SUMA_NODE_Y || \
                                  (ctp)==SUMA_NODE_Z || \
-                                 (ctp)==SUMA_GNODE_INDEX) ? 1:0)
+                                 (ctp)==SUMA_GNODE_INDEX || \
+                                 (ctp)==SUMA_GNODE_IGROUP || \
+                                 (ctp)==SUMA_NODE_R || \
+                                 (ctp)==SUMA_NODE_G || \
+                                 (ctp)==SUMA_NODE_B || \
+                                 (ctp)==SUMA_NODE_SLABEL) ? 1:0)
+                                 
 #define SUMA_GNODE_IXYZ_CTP2COL(ctp) ( (ctp)==SUMA_NODE_X ? 1 : \
                                        ( (ctp)==SUMA_NODE_Y ? 2: \
                                        ( (ctp)==SUMA_NODE_Z ? 3: \
-                                       ( (ctp)==SUMA_GNODE_INDEX ? 0: -1 ) ) )  )
+                                       ( (ctp)==SUMA_NODE_SLABEL ? 4: \
+                                       ( (ctp)==SUMA_GNODE_IGROUP ? 5: \
+                                       ( (ctp)==SUMA_NODE_R ? 6: \
+                                       ( (ctp)==SUMA_NODE_G ? 7: \
+                                       ( (ctp)==SUMA_NODE_B ? 8: \
+                                       ( (ctp)==SUMA_GNODE_INDEX ? 0: \
+                                                         -1 ) ) ) ) ) ) ) ) )
                                        
 #define SUMA_IS_DATUM_INDEX_COL(ctp) (((ctp)==SUMA_NODE_INDEX || \
                                  (ctp)==SUMA_EDGE_P1_INDEX || \
@@ -226,7 +255,7 @@ typedef enum {
                                                                                     SBT_type: Screen-based text
                                                                                     DBT_type: Dicom-based text
                                                                                     */
-typedef enum { type_not_set = -1,
+typedef enum { NOT_SET_type = -1,
                not_DO_type, SO_type, AO_type, ROIdO_type, ROIO_type, 
                GO_type, LS_type, NBLS_type, OLS_type, NBOLS_type,
                NBV_type, ONBV_type, SP_type,
@@ -234,7 +263,7 @@ typedef enum { type_not_set = -1,
                NBT_type, SBT_type, DBT_type, /*!< Those three will 
                                                    likely not be used */
                NIDO_type, SDSET_type, TRACT_type,
-               GRAPH_LINK_type,
+               GRAPH_LINK_type, MASK_type,
                N_DO_TYPES } SUMA_DO_Types;   
 
 #define iDO_isSO(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? 0: \
@@ -243,22 +272,52 @@ typedef enum { type_not_set = -1,
                       ( SUMAg_DOv[(i)].ObjectType == GRAPH_LINK_type ? 1:0) )
 #define iDO_isTDO(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? 0: \
                       ( SUMAg_DOv[(i)].ObjectType == TRACT_type ? 1:0) )
-#define iDO_type(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? type_not_set: \
+#define iDO_isMDO(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? 0: \
+                      ( SUMAg_DOv[(i)].ObjectType == MASK_type ? 1:0) )
+#define iDO_isVO(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? 0: \
+                      ( SUMAg_DOv[(i)].ObjectType == VO_type ? 1:0) )
+#define iDO_type(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? NOT_SET_type: \
                       ( SUMAg_DOv[(i)].ObjectType ) )
 #define iDO_typename(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? \
                               "NO OBJECT!": \
                       ( SUMA_ObjectTypeCode2ObjectTypeName(\
                                     SUMAg_DOv[(i)].ObjectType) ) )
+#define iDO_ADO(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? NULL : \
+                              (SUMA_ALL_DO *)SUMAg_DOv[(i)].OP)
+#define ADO_iDO(ado) ( (ado) ? SUMA_whichDO(SUMA_ADO_idcode((ado)), \
+                                            SUMAg_DOv, SUMAg_N_DOv):-1 )
+#define iDO_state SUMA_iDO_state
+#define iDO_group SUMA_iDO_group
+
 #define ADO_TNAME(ado) (!(ado) ? \
                               "NULL ADO!": \
                       ( SUMA_ObjectTypeCode2ObjectTypeName(\
                                     (ado)->do_type) ) )
+#define ADO_ID(ado) SUMA_ADO_idcode(ado)
+#define ADO_LABEL(ado) SUMA_ADO_sLabel(ado)
+#define ADO_STATE(ado) SUMA_iDO_state(ADO_iDO(ado))
+#define ADO_GROUP(ado) SUMA_iDO_group(ADO_iDO(ado))
 
 #define iDO_label(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? \
                               "NO OBJECT!": \
                       ( (SUMA_ADO_Label((SUMA_ALL_DO *)SUMAg_DOv[(i)].OP) ) ) )
+#define iDO_variant(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? \
+                              "NO OBJECT!": \
+                      ( (SUMA_ADO_variant((SUMA_ALL_DO *)SUMAg_DOv[(i)].OP) ) ) )
+
+#define iDO_GSaux(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? \
+                              NULL: \
+                      ( (SUMA_ADO_GSaux((SUMA_ALL_DO *)SUMAg_DOv[(i)].OP) ) ) )
+#define iDO_idcode(i) ( ((i)<0 || (i)>=SUMAg_N_DOv) ? \
+                              NULL: \
+                      ( (SUMA_ADO_idcode((SUMA_ALL_DO *)SUMAg_DOv[(i)].OP) ) ) )                      
+#define iDO_is_variant(i,var) ( (var) ? !strcmp(iDO_variant(i),(var)):0 )
 #define  DO_label(DO) ( !(DO) ? "NO OBJECT!": \
                         ((SUMA_ADO_Label((SUMA_ALL_DO *)(DO)) ) ) )
+
+#define SUMA_IS_GOOD_STATE(mm) ( ((mm)&&strncmp(mm,"TheShadow",9))?1:0 )
+#define SUMA_IS_REAL_VARIANT(mm) ( ((mm)&&strncmp(mm,"TheShadow",9))?1:0 )
+
 /*! 
 I do not think we can have both nodes and triangles in this struct.
 I guess I can make this be a Node Datum then create a similar struct
@@ -306,13 +365,26 @@ typedef enum { SUMA_NO_PTR_TYPE,
                SUMA_LINKED_MEMB_FACE_TYPE, 
                                  /*!< For pointers to SUMA_MEMBER_FACE_SETS*/
                SUMA_LINKED_SURFCONT_TYPE, /*!< For pointers to SUMA_X_SurfCont*/
+               SUMA_LINKED_COLORLIST_TYPE,/* pointers to SUMA_COLORLIST_STRUCT */
                SUMA_N_LINKED_PTR_TYPES } SUMA_LINKED_PTR_TYPES;
 
 typedef enum { MAT_UNKNOWN=-2, MAT_NA = -1, MAT_HEEHAW = 0 /* not set */, 
                MAT_FULL = 1, MAT_TRI, MAT_TRI_DIAG, MAT_SPARSE 
               } SUMA_SQ_MATRIX_SHAPES; 
 
+typedef enum { SURF_DSET, GRAPH_DSET, TRACT_DSET, VOLUME_DSET } 
+                                                   SUMA_DSET_FLAVORS;
+typedef enum { SUMA_ELEM_DAT=0, /* Nodes of surface, points of tracts, 
+                                 edges of graph */
+               SUMA_LEV1_DAT, /* data at the tract level*/
+               SUMA_LEV2_DAT, /* data at the bundle level */
+               SUMA_N_LEV_DAT} SUMA_DATUM_LEVEL;
+
 typedef struct { /* Something to hold auxiliary datasets structs */
+      /* The Saux kids are not always set, they should be
+         reached by accessor functions and macros, not
+         fished directly from here 
+         Where Saux lies depends on the flavor of dset at hand */
    void *Saux; /* A pointer to a structure for SUMA's use */
    void (*FreeSaux)(void *Saux); /* Function to free Saux */
    
@@ -326,8 +398,10 @@ typedef struct { /* Something to hold auxiliary datasets structs */
    long int range_edge_index[2]; /* min, max, edge index */
    long int range_node_index[2]; /* min, max, node index 
                                    (points defining edges)*/
-   long int N_uniq_nodes; /* Number of unique node indices */
-   byte isGraph;
+   long int N_seg_nodes; /* Number of node indices making up segments*/
+   long int N_all_nodes; /* Total number of nodes stored in nodelist of the
+                            graph dataset */
+   SUMA_DSET_FLAVORS isGraph;
 } SUMA_DSET_AUX;
 
 /*!   
@@ -715,7 +789,15 @@ typedef struct {
    #define SDSET_VECFILLED(dset) dset->nel->vec_filled
 #else
    #define SDSET_FILENAME(dset) NI_get_attribute(dset->ngr,"filename")
+      /* A safer version than SDSET_FILENAME */
+   #define SDSET_FILENAME_s(dset) SUMA_sdset_filename(dset)
+   
+   /* This macro can return NULL, use it wisely, or use the safer macro */
    #define SDSET_LABEL(dset) NI_get_attribute(dset->ngr,"label")
+   #define SDSET_LABEL_s(dset) SUMA_sdset_label(dset)
+   
+   /* This macro can return NULL, can't change behavior of SUMA_sdset_id() 
+   unless I check usage of SDSET_ID everywhere in the code... */
    #define SDSET_ID(dset) SUMA_sdset_id(dset) 
    #define SDSET_IDGDOM(dset) \
             NI_get_attribute(dset->ngr,"geometry_parent_idcode") 
@@ -729,14 +811,21 @@ typedef struct {
    #define SDSET_TYPE_NAME(dset) NI_get_attribute(dset->ngr,"dset_type")
    #define SDSET_TYPE(dset)   \
       SUMA_Dset_Type(NI_get_attribute(dset->ngr,"dset_type"))
+   #define SDSET_DAT_LEVEL(dset) \
+      SUMA_sdset_datum_level(dset);
    #define SDSET_COLCAST(dset, i)   \
       SUMA_ColType2TypeCast(SUMA_TypeOfDsetColNumb(dset, i))
    #define SDSET_COLTYPE(dset, i)   \
       SUMA_TypeOfDsetColNumb(dset, i)
    #define SDSET_VECLEN(dset) ( (!dset || !dset->dnel) ? -1:dset->dnel->vec_len)
+   #define SDSET_VEC(dset,iii) dset->dnel->vec[iii]
+   #define SDSET_NVOX SDSET_VECLEN
+   #define SDSET_IS_VOL SUMA_isVolDataset
+   #define SDSET_BRICK_FACTOR SUMA_GetBrickFactor
+   #define SDSET_BRICK_TYPE   SUMA_GetBrickType
+   #define SDSET_ARRAY SDSET_VEC
    #define SDSET_NODEINDLEN(dset) dset->inel->vec_len
    #define SDSET_VECNUM(dset) dset->dnel->vec_num
-   #define SDSET_VEC(dset,iii) dset->dnel->vec[iii]
    #define SDSET_NODEINDNUM(dset) dset->inel->vec_num
    #define SDSET_VECFILLED(dset) dset->dnel->vec_filled
    #define SDSET_NODEINDFILLED(dset) dset->inel->vec_filled
@@ -768,8 +857,10 @@ typedef struct {
                                           -1: ((dset)->Aux->matrix_size[0]) )
    #define SDSET_MATRIX_SZ1(dset) ( (!(dset) || !(dset)->Aux) ? \
                                           -1: ((dset)->Aux->matrix_size[1]) )
-   #define GDSET_MAX_POINTS(dset) ( (!(dset) || !(dset)->Aux) ? \
-                                          -1: ((dset)->Aux->N_uniq_nodes) )
+   #define GDSET_N_SEG_POINTS(dset) ( (!(dset) || !(dset)->Aux) ? \
+                                          -1: ((dset)->Aux->N_seg_nodes) )
+   #define GDSET_N_ALL_POINTS(dset) ( (!(dset) || !(dset)->Aux) ? \
+                                          -1: ((dset)->Aux->N_all_nodes) )
 #endif
 
 #define DSET_MAX_NODE_INDEX(dset, MM) {\
@@ -789,10 +880,6 @@ typedef struct {
    DO NOT USE COLP_NODEDEF macro inside a loop where the returned
    value is not to change because it involves a function call (SLOW)
 */
-/* Pre March 29 04. 
-#define COLP_NODEDEF(cop) SUMA_GetNodeDef(cop->dset_link)
-#define COLP_N_NODEDEF(cop) cop->dset_link->nel->vec_filled
-*/
 /* Post March 29 04. You can't go frugal and use dset's fields
 NodeDef might be dynamically changed in the overlay plane */
 #define COLP_NODEDEF(cop) cop->NodeDef
@@ -840,7 +927,7 @@ static byte NI_GOT;
    if (m_s) { NI_GOT = 1; val = atoi(m_s); } else { NI_GOT = 0; val = 0; }\
 }
 #define NI_SET_INTv(ngr, name, valv, n) {\
-   char m_stmp[400]; int m_i=0, m_s=0; m_stmp[0] = '\0';\
+   char m_stmp[400]={""}; int m_i=0, m_s=0; m_stmp[0] = '\0';\
    for (m_i=0; m_i<n && m_s < 350; ++m_i) { \
       sprintf(m_stmp+m_s, " %d", valv[m_i]);   \
       m_s = strlen(m_stmp);  \
@@ -863,7 +950,7 @@ static byte NI_GOT;
             else  if (m_nr > n) { \
                SUMA_S_Warn("More values in field\nProceeding..."); }  \
          }  \
-         for (m_i=0; m_i<SUMA_MIN_PAIR(n, m_nr);++m_i) valv[m_i] = m_iv[m_i];    \
+         for (m_i=0; m_i<SUMA_MIN_PAIR(n, m_nr);++m_i) valv[m_i] = m_iv[m_i];  \
          SUMA_free(m_iv);  \
       } else {    \
          NI_GOT = 1; \
@@ -1028,6 +1115,8 @@ static byte NI_GOT;
    }  \
 }
 
+#define DSET_FIND(id) (SUMA_FindDset_s(id, SUMAg_CF->DsetList))
+
 /*! Write an array to a text file, mcol consecutive values per line. 
 v is the array
 Nel is the total number of values
@@ -1340,16 +1429,6 @@ If ind is NULL, then the index will be the line number.
 }
 /*!
    replace a string positioned in column col, row row in NI_element * nel.
-   
-   This macro, when called from SUMA_AddColAtt_CompString ends up reporting
-   a NI_malloc post corruption, at least when starting the dot-product 
-   demo script "tcsh @bugy" under directory
-      /Volumes/Data/ziad/SUMA_test_dirs/suma/srf
-   I can't figure out how m_rc[(row)] is getting corrupted. For now, 
-   allocating for 10 chars more gets rid of the problem.
-   Set +10 to +1 if you want to reproduce the problem.
-   Search for comment tagged: TAG09102012 to find calling 
-   culprit. 
 */
 #define SUMA_NEL_REPLACE_STRING(nel, row, col, str) {\
    char **m_rc;   \
@@ -1357,7 +1436,7 @@ If ind is NULL, then the index will be the line number.
    if (m_rc[(row)]) NI_free(m_rc[(row)]); \
    m_rc[(row)] = NULL;\
    if (str) { \
-      m_rc[(row)] = (char*)NI_malloc(char, (strlen((str))+10)*sizeof(char));\
+      m_rc[(row)] = (char*)NI_malloc(char, (strlen((str))+1)*sizeof(char));\
       strcpy( m_rc[(row)], str );   \
    }  \
 }
@@ -1486,6 +1565,7 @@ NI_element *SUMA_FindDsetDataElement(SUMA_DSET *dset);
 NI_element *SUMA_FindGDsetNodeListElement(SUMA_DSET *dset);
 NI_element *SUMA_AddGDsetNodeListElement(SUMA_DSET *dset, 
                                          int *I, float *X, float *Y, float *Z, 
+                                         char **names, int *cln, float *cols,
                                          int N_Node);
 NI_element *SUMA_FindDsetDatumIndexElement(SUMA_DSET *dset);
 NI_element *SUMA_FindSDsetNodeIndexElement(SUMA_DSET *dset);
@@ -1541,6 +1621,7 @@ SUMA_Boolean SUMA_isDsetNelAttr(NI_element *nel);
 char * SUMA_CreateDsetColRangeCompString( SUMA_DSET *dset, int col_index, 
                                           SUMA_COL_TYPE ctp);
 int SUMA_UpdateDsetColRange(SUMA_DSET *dset, int icol);
+int SUMA_UpdateDsetColLabel(SUMA_DSET *dset, int icol, char *label);
 char * SUMA_GetDsetColStringAttr( SUMA_DSET *dset, int col_index, 
                                     char *attrname);
 char * SUMA_GetNgrColStringAttr( NI_group *ngr, int col_index, 
@@ -1552,9 +1633,12 @@ SUMA_Boolean SUMA_CopyDsetAttributes ( SUMA_DSET *src, SUMA_DSET *dest,
                                        int isrc, int idest );
                                        
 /* A quick way to check graphinity. Use SUMA_isGraphDset for safety */
-#define SUMA_isGraphDset_fast(dset) ( (dset)->Aux->isGraph ) 
+#define SUMA_isGraphDset_fast(dset) ( ((dset)->Aux->isGraph==GRAPH_DSET) ) 
+#define SUMA_isTractDset_fast(dset) ( ((dset)->Aux->isGraph==TRACT_DSET) ) 
 byte SUMA_isGraphDset(SUMA_DSET *dset);
 byte SUMA_isGraphDsetNgr(NI_group *ngr);
+byte SUMA_isTractDset(SUMA_DSET *dset);
+byte SUMA_isTractDsetNgr(NI_group *ngr);
 SUMA_Boolean SUMA_Add_Dset_Aux(SUMA_DSET *dset);
 SUMA_Boolean SUMA_NewDsetGrp (SUMA_DSET *dset, SUMA_DSET_TYPE dtp, 
                            char* MeshParent_idcode, 
@@ -1580,8 +1664,13 @@ DListElmt * SUMA_FindDsetEl_ns (char *idcode, DList *DsetList);
 SUMA_DSET * SUMA_FindDset_eng (char *idcode_str, DList *DsetList, 
                                  DListElmt **elp, char *itype);
 char *SUMA_DsetInfo (SUMA_DSET *dset, int detail);
+char * SUMA_Taylor_Network_Info(TAYLOR_NETWORK *net, 
+                                int show_maxu, int show_maxub);
+char *SUMA_Taylor_Bundle_Info(TAYLOR_BUNDLE *tb, int show_maxu); 
+char *SUMA_Taylor_Tract_Info(TAYLOR_TRACT *tt, int show_maxu);
 void SUMA_ShowDset (SUMA_DSET *dset, int detail, FILE *out);
-char *SUMA_ShowMeSome (void *dt, SUMA_VARTYPE tp, int N_dt, int mxshow, char *title);
+char *SUMA_ShowMeSome (void *dt, SUMA_VARTYPE tp, int N_dt, 
+                       int mxshow, char *title);
 SUMA_DSET * SUMA_NewDsetPointer(void);
 SUMA_DSET * SUMA_CreateDsetPointer (  
                               char *name, 
@@ -1630,6 +1719,13 @@ float * SUMA_Col2Float (NI_element *nel, int ind, int FilledOnly);
 SUMA_Boolean SUMA_SetUniqueValsAttr(SUMA_DSET *dset, int icol, byte replace);
 NI_element * SUMA_GetUniqueValsAttr(SUMA_DSET *dset, int icol);
 NI_element * SUMA_GetUniqueIndicesAttr(SUMA_DSET *dset, int iindex);
+int * SUMA_GetUniqueIndicesVec(SUMA_DSET *dset, int iindex);
+int * SUMA_GetDatasetDimensions(SUMA_DSET *dset);
+float * SUMA_GetDatasetFactors(SUMA_DSET *dset);
+MRI_TYPE SUMA_GetBrickType(SUMA_DSET *dset, int ii);
+float SUMA_GetBrickFactor(SUMA_DSET *dset, int ii);
+float * SUMA_GetDatasetI2X(SUMA_DSET *dset, float M[4][4]);
+int SUMA_isVolDataset(SUMA_DSET *dset);
 SUMA_Boolean SUMA_SetUniqueIndicesAttr(SUMA_DSET *dset, byte replace);
 int SUMA_GetDsetColRange(SUMA_DSET *dset, int col_index, 
                          double range[2], int loc[2]);
@@ -1674,6 +1770,7 @@ SUMA_DSET * SUMA_far2dset_ns( char *FullName, char *dset_id, char *dom_id,
 int SUMA_is_AllNumeric_dset(SUMA_DSET *dset);
 int SUMA_dset_to_Label_dset(SUMA_DSET *dset); 
 int SUMA_is_Label_dset(SUMA_DSET *dset, NI_group **NIcmap); 
+int SUMA_is_Label_dset_col(SUMA_DSET *dset, int icol);
 int SUMA_is_Phase_dset(SUMA_DSET *dset); 
 int SUMA_is_RetinoAngle_dset(SUMA_DSET *dset); 
 int SUMA_is_VFR_dset(SUMA_DSET *dset); 
@@ -1724,7 +1821,12 @@ SUMA_Boolean SUMA_Append_Copy_Part_Column(void *col, NI_rowtype *rt, int N_col,
 void *SUMA_Copy_Part_Column(void *col,  NI_rowtype *rt, int N_col, 
                             byte *rowmask, int masked_only, int *n_incopy);
 char* SUMA_sdset_id(SUMA_DSET *dset);
+char *SUMA_sdset_label(SUMA_DSET *dset);
+char *SUMA_sdset_filename(SUMA_DSET *dset);
 char* SUMA_sdset_idmdom(SUMA_DSET *dset);
+char *SUMA_Dset_orcode(SUMA_DSET *dset);
+SUMA_DATUM_LEVEL SUMA_sdset_datum_level(SUMA_DSET *dset);
+SUMA_Boolean SUMA_sdset_set_datum_level(SUMA_DSET *dset, SUMA_DATUM_LEVEL lvl);
 NI_group *SUMA_oDsetNel2nDsetNgr(NI_element *nel); 
 void SUMA_SetParent_DsetToLoad(char *parent);
 float *SUMA_Load1D_eng (char *oName, int *ncol, int *nrow, int RowMajor, int verb);
@@ -1774,7 +1876,7 @@ int SUMA_GetAddIndex_1D(void);
 THD_3dim_dataset *SUMA_sumadset2afnidset(SUMA_DSET **dsetp, int copy_data, 
                                          int cleardset);
 SUMA_DSET *SUMA_afnidset2sumadset(THD_3dim_dataset **dsetp, int copy_data, 
-                                  int cleardset);
+                                  int cleardset, int floatize);
 int SUMA_GetDsetColStatAttr(  SUMA_DSET *dset, int col_index, 
                               int *statcode,
                               float *p1, float *p2, float *p3);
@@ -1896,14 +1998,25 @@ byte SUMA_GDSET_PointsToSegIndex(SUMA_DSET *dset, int i1, int i2, int *si);
 byte SUMA_GDSET_SegRowToPoints(SUMA_DSET *dset, int ri, 
                                  int *i1, int *i2, int *index);
 byte SUMA_GDSET_PointsToSegRow(SUMA_DSET *dset, int i1, int i2, int *ri);
+byte SUMA_GDSET_PointToDiagSegRowIndex(SUMA_DSET *dset,int i1, int *ri, int *si);
+byte SUMA_GDSET_PointToDiagSegIndex(SUMA_DSET *dset, int i1, int *si);
+byte SUMA_GDSET_PointToDiagSegRow(SUMA_DSET *dset, int i1, int *ri);
 SUMA_SQ_MATRIX_SHAPES SUMA_matrix_shape_name_to_matrix_shape(char *name); 
 char * SUMA_matrix_shape_to_matrix_shape_name(SUMA_SQ_MATRIX_SHAPES sq);
 int *SUMA_GDSET_GetPointIndexColumn(SUMA_DSET *dset, int *N_vals, NI_element **);
+char **SUMA_GDSET_GetPointNamesColumn(SUMA_DSET *dset, int *N_vals, 
+                                    NI_element **nelxyzr);
+float *SUMA_GDSET_GetPointColumn_f(SUMA_DSET *dset, int *N_vals, 
+                                      NI_element **nelxyzr, char *label);
+int *SUMA_GDSET_GetPointGroupColumn(SUMA_DSET *dset, int *N_vals, 
+                                      NI_element **nelxyzr);
 int SUMA_GDSET_Index_To_NodeIndex(SUMA_DSET *dset, int cinode);
 int SUMA_GDSET_NodeIndex_To_Index(SUMA_DSET *dset, int node);
 int SUMA_GDSET_EdgeIndex_To_Row(SUMA_DSET *dset, int ei);
 int SUMA_GDSET_EdgeRow_To_Index(SUMA_DSET *dset, int ri);
 int SUMA_GDSET_Max_Edge_Index(SUMA_DSET *dset);
+char *SUMA_GDSET_Node_Label(SUMA_DSET *dset, int psel);
+char *SUMA_GDSET_Edge_Label(SUMA_DSET *dset, int isel, char *pref, char *sep);
 
 
 #endif
