@@ -1101,6 +1101,8 @@ void free_atlas(ATLAS *xa)
    if (xa->name) free(xa->name);
    if (xa->comment) free(xa->comment);
    if (xa->atlas_type) free(xa->atlas_type);
+   if (xa->supp_web_info) free(xa->supp_web_info);
+   if (xa->supp_web_type) free(xa->supp_web_type);
    if (xa->orient) free(xa->orient);
 
    if (xa->adh) free_adh(xa->adh);
@@ -1336,6 +1338,16 @@ void print_atlas_type(ATLAS *xa)
        INFO_message("%s", ATL_TYPE(xa));
 }
 
+/* print the atlas supplemental information for an atlas */
+void print_atlas_supp_web_info(ATLAS *xa)
+{
+    if((xa) && ATL_SUPP_WEB_INFO(xa)) {
+       INFO_message("%s/roiname%s", ATL_SUPP_WEB_INFO_S(xa),
+                    ATL_SUPP_WEB_TYPE_S(xa));
+    }
+
+}
+
 /* print the coordinate orientation for an atlas */
 void print_atlas_orient(ATLAS *xa)
 {
@@ -2474,6 +2486,8 @@ int atlas_read_atlas(NI_element *nel, ATLAS *atlas, char *parentdir)
    atlas->description = NULL;
    atlas->orient = NULL; /* coordinate order for web queries (Elsevier) */
    atlas->atlas_type =  NULL;  /* atlas can be "web"/ NULL */
+   atlas->supp_web_info =  NULL;  /* supplemental info available at website */
+   atlas->supp_web_type =  NULL;  /* atlas info may be .pdf, .html, ...*/
    atlas->atlas_found = 0; /* flag for dataset available, -1 not found, 
                               1 found, 0 init value */
 
@@ -2501,6 +2515,10 @@ int atlas_read_atlas(NI_element *nel, ATLAS *atlas, char *parentdir)
       atlas->orient = nifti_strdup(s);
    if ((s=NI_get_attribute(nel,"type"))) 
       atlas->atlas_type = nifti_strdup(s);
+   if ((s=NI_get_attribute(nel,"supp_web_info"))) 
+      atlas->supp_web_info = nifti_strdup(s);
+   if ((s=NI_get_attribute(nel,"supp_web_type"))) 
+      atlas->supp_web_type = nifti_strdup(s);
 
    if((atlas->dset_name == NULL) || (atlas->space == NULL)) {
       WARNING_message("bad atlas nel");
@@ -2525,6 +2543,8 @@ int atlas_dup_atlas(ATLAS *srcatlas, ATLAS *destatlas)
    destatlas->atlas_found = srcatlas->atlas_found;
    destatlas->orient = srcatlas->orient;
    destatlas->atlas_type = srcatlas->atlas_type;
+   destatlas->supp_web_info = srcatlas->supp_web_info;
+   destatlas->supp_web_type = srcatlas->supp_web_type;
 
    destatlas->adh = srcatlas->adh;
    
