@@ -164,18 +164,24 @@ ENTRY("THD_set_dataset_attributes") ;
 
    /*-- write matrix for (i,j,k) to DICOM real (x,y,z) conversion [18 May 2007] --*/
    /* to store obliquity information */
-   THD_check_oblique_field(dset);
-   /* if not oblique already,compute Tc (Cardinal transformation matrix) */
-   angle = THD_compute_oblique_angle(daxes->ijk_to_dicom_real, 0);
-   if(angle==0.0){
-      THD_dicom_card_xform(dset, &tmat, &tvec);
-      LOAD_MAT44(Tc,
-          tmat.mat[0][0], tmat.mat[0][1], tmat.mat[0][2], tvec.xyz[0],
-          tmat.mat[1][0], tmat.mat[1][1], tmat.mat[1][2], tvec.xyz[1],
-          tmat.mat[2][0], tmat.mat[2][1], tmat.mat[2][2], tvec.xyz[2]);
-      daxes->ijk_to_dicom_real = Tc;
-   }
+   if(!THD_update_obliquity_status()){ /* maybe update the obliquity unless refitting */
+      THD_check_oblique_field(dset);
 
+      if (ISVALID_MAT44(dset->daxes->ijk_to_dicom_real)){
+           /* if not oblique already,compute Tc (Cardinal transformation matrix) */
+           angle = THD_compute_oblique_angle(daxes->ijk_to_dicom_real, 0);
+
+           if(angle==0.0){
+              THD_dicom_card_xform(dset, &tmat, &tvec);
+              LOAD_MAT44(Tc,
+                  tmat.mat[0][0], tmat.mat[0][1], tmat.mat[0][2], tvec.xyz[0],
+                  tmat.mat[1][0], tmat.mat[1][1], tmat.mat[1][2], tvec.xyz[1],
+                  tmat.mat[2][0], tmat.mat[2][1], tmat.mat[2][2], tvec.xyz[2]);
+              daxes->ijk_to_dicom_real = Tc;
+           }
+      }
+   }
+   
    if( ISVALID_MAT44(daxes->ijk_to_dicom_real) ){
      UNLOAD_MAT44(daxes->ijk_to_dicom_real, ftemp[0],ftemp[1],ftemp[2],ftemp[3],
                                        ftemp[4],ftemp[5],ftemp[6],ftemp[7],
