@@ -468,8 +468,6 @@ ENTRY("THD_open_nifti") ;
    /*-- make an AFNI dataset! --*/
 
    dset = EDIT_empty_copy(NULL) ;
-   /* copy transformation matrix to dataset structure */
-   dset->daxes->ijk_to_dicom_real = ijk_to_dicom44;
 
    ppp  = THD_trailname(pathname,0) ;               /* strip directory */
    MCW_strncpy( prefix , ppp , THD_MAX_PREFIX ) ;   /* to make prefix */
@@ -506,6 +504,10 @@ ENTRY("THD_open_nifti") ;
                                                         : HEAD_ANAT_TYPE ,
                     ADN_none ) ;
 
+   /* copy transformation matrix to dataset structure */
+   /* moved after setting grid     4 Apr 2014 [rickr,drg] */
+   dset->daxes->ijk_to_dicom_real = ijk_to_dicom44;
+
    /* not a time dependent dataset */
 
    if( ntt < 2 ){
@@ -748,6 +750,11 @@ ENTRY("THD_load_nifti") ;
 
    dkptr = dblk->diskptr ;
 
+   /* purge any existing bricks [10 Mar 2014] */
+
+   STATUS("purging existing data bricks (if any)") ;
+   THD_purge_datablock(dblk,DATABLOCK_MEM_ANY) ;
+
    STATUS("calling nifti_image_read_bricks") ;
    NBL.nbricks = 0 ;
 
