@@ -166,6 +166,8 @@ typedef struct {
    char *comment;
    char *atlas_type;  /* web or NULL for now, for web type, dset is http webpage address */
    char *orient;  /* string to specify xyz order requests - Elsevier's web version uses "RSA"*/
+   char *supp_web_info; /* string specifying base webpage for supplemental structure info */
+   char *supp_web_type; /* extension suffix for webpages - pdf, html,... */
    int atlas_found;
    ATLAS_DSET_HOLDER *adh;
 } ATLAS; /*!< All char * should be initialized when .niml file is loaded,
@@ -209,6 +211,14 @@ typedef struct {
 /* is the atlas a web type */
 #define ATL_WEB_TYPE(xa) (strcasecmp((ATL_TYPE_S(xa)),"web")== 0)
 
+/* is there supplemental information on Internet */
+#define ATL_SUPP_WEB_INFO(xa) ((xa) && (xa)->supp_web_info ? 1: 0)
+#define ATL_SUPP_WEB_INFO_S(xa) ( (ATL_SUPP_WEB_INFO(xa)) ? \
+          ((xa)->supp_web_info)   : "No supplemental info " )
+#define ATL_SUPP_WEB_TYPE(xa) ((xa)->supp_web_type ? 1: 0)
+#define ATL_SUPP_WEB_TYPE_S(xa) ( (ATL_SUPP_WEB_TYPE(xa)) ? \
+          ((xa)->supp_web_type)   : ".html" )
+
 #define ATL_FOUND(xa) ( (xa)  ? \
                            ((xa)->atlas_found) : 0 )
 
@@ -345,6 +355,8 @@ char *get_updated_help_file(int force_recreate, byte verb, char *progname,
                             int shtp);
 int prog_complete_command (char *prog, char *ofile, int shtp);
 char **approx_str_sort_readmes(char *str, int *N_r);
+char **unique_str(char **words, int N_words, byte ci, 
+                  byte noae, int *N_unq, int **isort_out);
 char *find_readme_file(char *str);
 int view_text_file(char *progname);
 void view_prog_help(char *prog);
@@ -459,6 +471,7 @@ char *find_atlas_niml_file(char * nimlname, int nini);
 ATLAS_LIST *env_atlas_list(void);
 char **env_space_list(int *);
 int env_dec_places(void);
+char * search_quotes(char *in_str);
 
 char *Current_Atlas_Default_Name(void);
 char **Atlas_Names_List(ATLAS_LIST *atl);
@@ -470,7 +483,7 @@ char *elsevier_query(float xx, float yy, float zz, ATLAS *atlas);
 char *elsevier_query_request(float xx, float yy, float zz, ATLAS *atlas, int el_req_type);
 void wami_query_web(ATLAS *atlas, ATLAS_COORD ac, ATLAS_QUERY *wami);
 
-char * whereami_XML_get(char *data, char *name);
+char * whereami_XML_get(char *data, char *name, char **next);
 int whereami_browser(char *url);
 char *cleanup_url(char *url);
 void set_wami_web_found(int found);
@@ -482,10 +495,18 @@ char * get_wami_webpage(void);
 void open_wami_webpage(void);
 int AFNI_wami_output_mode(void);
 void set_AFNI_wami_output_mode(int webflag);
+char * atlas_suppinfo_webpage(ATLAS *atlas, char *blab);
+
 size_t CURL_read_URL_http ( char *url, char **data);
 void set_wami_minprob(float val);
 float get_wami_minprob(void);
 float Get_PMap_Factor(void);
+int show_neurosynth_link();
+int show_linkrbrain_link();
+char * neurosynth_coords_link(float x, float y, float z);
+int make_linkrbrain_xml(float *coords, int ncoords, 
+    char *src_space, char *dest_space, char *linkrbrain_xml, int linkr_corr_type);
+int send_linkrbrain_xml(char *linkrbrain_xml, char *linkrbrain_results);
 
 /* Transforms for going from one space to another */
 #if 0
